Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React = require("react");
var rxjs_1 = require("rxjs");
function validateBindableProperty(property) {
    if (BindableInput.wx.isProperty(property) && property.isReadOnly) {
        BindableInput.wx.handleError('BindableInput bound to ReadOnlyProperty', property);
    }
}
exports.validateBindableProperty = validateBindableProperty;
var BindableInput = /** @class */ (function (_super) {
    tslib_1.__extends(BindableInput, _super);
    function BindableInput() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.changedSubscription = rxjs_1.Subscription.EMPTY;
        return _this;
    }
    BindableInput.prototype.componentWillMount = function () {
        var _this = this;
        validateBindableProperty(this.props.boundProperty);
        if (this.wx.isProperty(this.props.boundProperty)) {
            this.changedSubscription = this.props.boundProperty.changed.subscribe(function (x) {
                _this.forceUpdate();
            });
        }
    };
    BindableInput.prototype.componentWillReceiveProps = function (nextProps) {
        validateBindableProperty(nextProps.boundProperty);
    };
    BindableInput.prototype.componentWillUnmount = function () {
        this.changedSubscription = rxjs_1.Subscription.unsubscribe(this.changedSubscription);
    };
    BindableInput.prototype.render = function () {
        var _this = this;
        var _a = this.restProps(function (x) {
            var boundProperty = x.boundProperty, converter = x.converter, valueProperty = x.valueProperty, onChangeProperty = x.onChangeProperty, valueGetter = x.valueGetter, valueSetter = x.valueSetter;
            return {
                boundProperty: boundProperty,
                converter: converter,
                valueProperty: valueProperty,
                onChangeProperty: onChangeProperty,
                valueGetter: valueGetter,
                valueSetter: valueSetter,
            };
        }), className = _a.className, children = _a.children, props = _a.props, rest = _a.rest;
        var bindProps = {};
        var value = this.getValue();
        // NOTE: react says the value of a bindable control should not be null, but
        //       instead empty string (undefined is ok though, so === is required).
        bindProps[props.valueProperty] = value == null ? '' : value;
        bindProps[props.onChangeProperty] = function (e) {
            _this.setValue(e);
        };
        return React.cloneElement(React.Children.only(children), Object.assign({ className: className }, rest, bindProps));
    };
    BindableInput.prototype.getValue = function () {
        try {
            return this.props.valueGetter(this.props.boundProperty);
        }
        catch (e) {
            this.wx.handleError(e);
            return undefined;
        }
    };
    BindableInput.prototype.setValue = function (event) {
        try {
            var target = event.target;
            var value = target[this.props.valueProperty];
            if (this.props.converter != null) {
                value = this.props.converter(value);
            }
            this.props.valueSetter(value, this.props.boundProperty);
            this.forceUpdate();
        }
        catch (e) {
            this.wx.handleError(e);
        }
    };
    BindableInput.displayName = 'BindableInput';
    BindableInput.defaultProps = {
        valueProperty: 'value',
        onChangeProperty: 'onChange',
        valueGetter: function (property) {
            return BindableInput.wx.isProperty(property) ? property.value : property;
        },
        valueSetter: function (value, property) {
            if (BindableInput.wx.isProperty(property)) {
                property.value = value;
            }
        },
    };
    return BindableInput;
}(React.Component));
exports.BindableInput = BindableInput;
//# sourceMappingURL=BindableInput.js.map