Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React = require("react");
var react_bootstrap_1 = require("react-bootstrap");
var rxjs_1 = require("rxjs");
var ContentTooltip_1 = require("../ContentTooltip/ContentTooltip");
var CommandButton = /** @class */ (function (_super) {
    tslib_1.__extends(CommandButton, _super);
    function CommandButton() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.canExecuteSubscription = rxjs_1.Subscription.EMPTY;
        return _this;
    }
    CommandButton.prototype.getCommand = function () {
        var cmd = this.props.command;
        return cmd instanceof Function ? cmd() : cmd;
    };
    CommandButton.prototype.getParam = function () {
        var param = this.props.commandParameter;
        return param instanceof Function ? param() : param;
    };
    CommandButton.prototype.componentWillMount = function () {
        var _this = this;
        var cmd = this.getCommand();
        if (cmd != null) {
            this.canExecuteSubscription = cmd.canExecuteObservable
                .merge(cmd.conditionObservable)
                .subscribe(function () {
                _this.forceUpdate();
            });
        }
    };
    CommandButton.prototype.componentWillUnmount = function () {
        this.canExecuteSubscription = rxjs_1.Subscription.unsubscribe(this.canExecuteSubscription);
    };
    CommandButton.prototype.render = function () {
        var rest = this.restProps(function (x) {
            var onClick = x.onClick, command = x.command, commandParameter = x.commandParameter, stopPropagation = x.stopPropagation, preventDefault = x.preventDefault, plain = x.plain, compact = x.compact, tooltip = x.tooltip, tooltipPlacement = x.tooltipPlacement, disabled = x.disabled, componentClass = x.componentClass;
            return {
                onClick: onClick,
                command: command,
                commandParameter: commandParameter,
                stopPropagation: stopPropagation,
                preventDefault: preventDefault,
                plain: plain,
                compact: compact,
                tooltip: tooltip,
                tooltipPlacement: tooltipPlacement,
                disabled: disabled,
                componentClass: componentClass,
            };
        }).rest;
        return this.renderButton(rest);
    };
    CommandButton.prototype.renderButton = function (rest) {
        var cmd = this.props.disabled === true ? undefined : this.getCommand();
        var canExecute = cmd == null
            ? // no command was supplied so check both href and onClick to see if this button is enabled
                String.isNullOrEmpty(rest.href) === false ||
                    this.props.onClick != null
            : // use the command to see if this button is enabled
                cmd.canExecuteFor(this.props.commandParameter);
        var disabled = this.props.disabled || canExecute !== true;
        var classNames = this.wxr.classNames('CommandButton', this.props.className, {
            plain: this.props.plain,
            compact: this.props.compact,
        });
        var button = (React.createElement(react_bootstrap_1.Button, tslib_1.__assign({}, rest, { className: classNames, disabled: disabled, onClick: disabled ? undefined : this.handleClick.bind(this), componentClass: this.getComponentClass() }), this.props.children));
        if (this.props.tooltip == null || this.props.tooltip === false) {
            return button;
        }
        var ttId = this.props.id ? this.props.id + "-tt" : undefined;
        var tooltipContent = this.props.tooltip instanceof Function
            ? this.props.tooltip(disabled)
            : this.props.tooltip;
        return (React.createElement(ContentTooltip_1.ContentTooltip, { id: ttId, className: this.props.className, content: tooltipContent, placement: this.props.tooltipPlacement }, button));
    };
    CommandButton.prototype.getComponentClass = function () {
        if (this.props.componentClass != null) {
            return this.props.componentClass;
        }
        if (this.props.command == null &&
            String.isNullOrEmpty(this.props.href) === false) {
            return 'a';
        }
        return 'div';
    };
    CommandButton.prototype.handleClick = function (e) {
        var _this = this;
        var cmd = this.getCommand();
        if (cmd == null) {
            if (this.props.onClick != null) {
                // onClick was supplied so we don't override default click handling unless explicitly defined
                if (this.props.stopPropagation === true) {
                    e.stopPropagation();
                }
                if (this.props.preventDefault === true) {
                    e.preventDefault();
                }
                this.props.onClick(e);
            }
            else if (String.isNullOrEmpty(this.props.href) === false) {
                // we stop propagation by default (this prevents things below the button from being activated)
                if (this.props.stopPropagation !== false) {
                    e.stopPropagation();
                }
                // we don't disable default click behaviour (this allows href's to be managed with default behaviour)
                if (this.props.preventDefault === true) {
                    e.preventDefault();
                }
            }
        }
        else {
            // command was supplied so we disable default click handling unless explicitly disabled
            if (this.props.stopPropagation !== false) {
                e.stopPropagation();
            }
            if (this.props.preventDefault !== false) {
                e.preventDefault();
            }
            cmd.execute(this.getParam(), function () {
                if (_this.props.onClick != null) {
                    _this.props.onClick(e);
                }
            });
        }
    };
    CommandButton.displayName = 'CommandButton';
    CommandButton.defaultProps = {
        tooltipPlacement: 'top',
    };
    return CommandButton;
}(React.Component));
exports.CommandButton = CommandButton;
//# sourceMappingURL=CommandButton.js.map