Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React = require("react");
var react_bootstrap_1 = require("react-bootstrap");
var ContentTooltip = /** @class */ (function (_super) {
    tslib_1.__extends(ContentTooltip, _super);
    function ContentTooltip() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ContentTooltip.prototype.render = function () {
        var content = this.props.content;
        var context = this.props.context || React.Children.only(this.props.children);
        if (String.isNullOrEmpty(content) || content === false) {
            return context;
        }
        if (React.isValidElement(content)) {
            if (React.isType(content, ContentTooltip)) {
                return React.cloneElement(content, {}, context);
            }
            if (React.isType(content, react_bootstrap_1.OverlayTrigger)) {
                return this.renderOverlayTrigger(content, context);
            }
            if (React.isType(content, react_bootstrap_1.Popover) || React.isType(content, react_bootstrap_1.Tooltip)) {
                return this.renderOverlay(content, context);
            }
        }
        return this.renderOverlayContent(content, context);
    };
    // content is an OverlayTrigger
    ContentTooltip.prototype.renderOverlayTrigger = function (content, context) {
        var props = this.trimProps({
            placement: this.props.placement,
            trigger: this.props.trigger,
        });
        return React.cloneElement(content, props, context);
    };
    // content is an Overlay (Popover, Tooltip)
    ContentTooltip.prototype.renderOverlay = function (content, context) {
        var props = this.trimProps({
            id: this.props.id || content.props.id || context.id,
            className: this.wxr.classNames(this.props.className, content.props.className),
            placement: this.props.placement || content.props.placement,
            title: this.props.title,
        });
        if (!React.isType(content, react_bootstrap_1.Popover)) {
            delete props.title;
        }
        var overlay = React.cloneElement(content, props);
        var overlayTrigger = (React.createElement(react_bootstrap_1.OverlayTrigger, { overlay: overlay, placement: overlay.props.placement }));
        return this.renderOverlayTrigger(overlayTrigger, context);
    };
    ContentTooltip.prototype.renderOverlayContent = function (content, context) {
        if (!React.isValidElement(content) && Object.isObject(content)) {
            var props = content;
            var Component = props.title != null ||
                props.popover ||
                this.props.title != null ||
                this.props.popover
                ? react_bootstrap_1.Popover
                : react_bootstrap_1.Tooltip;
            return this.renderOverlay(React.createElement(Component, tslib_1.__assign({ id: props.id || this.props.id }, props)), context);
        }
        if (this.props.title != null || this.props.popover) {
            var popover = React.createElement(react_bootstrap_1.Popover, { id: this.props.id }, content);
            return this.renderOverlay(popover, context);
        }
        var tooltip = React.createElement(react_bootstrap_1.Tooltip, { id: this.props.id }, content);
        return this.renderOverlay(tooltip, context);
    };
    return ContentTooltip;
}(React.Component));
exports.ContentTooltip = ContentTooltip;
//# sourceMappingURL=ContentTooltip.js.map