// tslint:disable:max-classes-per-file
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React = require("react");
var react_bootstrap_1 = require("react-bootstrap");
// this is necessary to prevent a js error due to the Overlay trying to position
// the Popup.
// The Overlay attempts to pass on a bunch of props to its child and these props
// are not supported by the child node. We end up with a React 'Unknown props'
// error. So to circumvent this issue, we use this container wrapper that consumes
// all those Overlay injected props and only passes on the className and children.
// We still need to 'support' all the PopoverProps to eliminate the error.
var ContextMenuContainer = /** @class */ (function (_super) {
    tslib_1.__extends(ContextMenuContainer, _super);
    function ContextMenuContainer() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ContextMenuContainer.prototype.render = function () {
        return (React.createElement("div", { className: this.wxr.classNames('ContextMenu-container', this.props.className) }, this.props.children));
    };
    return ContextMenuContainer;
}(React.Component));
var ArrowOffset = 20;
var ContextMenu = /** @class */ (function (_super) {
    tslib_1.__extends(ContextMenu, _super);
    function ContextMenu(props) {
        var _this = _super.call(this, props) || this;
        _this.handleClick = _this.handleClick.bind(_this);
        _this.handleHide = _this.handleHide.bind(_this);
        _this.state = {
            isVisible: false,
        };
        return _this;
    }
    ContextMenu.prototype.handleClick = function (e) {
        // check if the right mouse button was clicked
        var isVisible = e.button === 2;
        if (isVisible === true) {
            // prevent other actions from happening
            e.stopPropagation();
            e.preventDefault();
            var left_1 = e.pageX;
            var top_1 = e.pageY - ArrowOffset;
            // update our state
            this.setState(function (prevState, props) {
                return {
                    isVisible: isVisible,
                    left: left_1,
                    top: top_1,
                };
            });
        }
    };
    ContextMenu.prototype.handleHide = function () {
        this.setState(function (prevState, props) {
            return {
                isVisible: false,
                left: undefined,
                top: undefined,
            };
        });
    };
    ContextMenu.prototype.render = function () {
        var menuItems = React.Children.toArray(this.props.children);
        return (React.createElement("div", { className: "ContextMenu" },
            React.createElement("div", { className: "ContextMenu-target", onContextMenu: this.handleClick }, menuItems.shift()),
            React.createElement(react_bootstrap_1.Overlay, { show: this.state.isVisible, rootClose: true, onHide: this.handleHide },
                React.createElement(ContextMenuContainer, null, this.renderMenu(menuItems)))));
    };
    ContextMenu.prototype.renderMenu = function (menuItems) {
        var _this = this;
        return this.wxr.renderConditional(this.state.isVisible === true, function () {
            return (React.createElement(react_bootstrap_1.Popover, { id: _this.props.id, placement: "right", title: _this.props.header, arrowOffsetTop: ArrowOffset, positionLeft: _this.state.left, positionTop: _this.state.top },
                React.createElement("ul", { className: "dropdown-menu" }, _this.renderMenuItems(menuItems))));
        });
    };
    ContextMenu.prototype.renderMenuItems = function (menuItems) {
        var onSelect = this.props.onSelect;
        if (onSelect) {
            // if onSelect is provided we need to inject it into all the menu items
            return React.Children.map(menuItems, function (x) {
                return React.cloneElement(x, { onSelect: function () { return onSelect(x.props); } });
            });
        }
        return menuItems;
    };
    ContextMenu.displayName = 'ContextMenu';
    return ContextMenu;
}(React.Component));
exports.ContextMenu = ContextMenu;
//# sourceMappingURL=ContextMenu.js.map