import { Iterable } from 'ix';
import { Observable } from 'rxjs';
import { ObjectComparer, SortDirection } from '../../../Utils/Compare';
import { Command, IterableLike, ObservableLike, ObservableOrValue, ReadOnlyProperty } from '../../../WebRx';
import { HandlerRoutingStateChanged, RoutingStateHandler } from '../../React';
import { ListItemsViewModel } from '../ListItems/ListItemsViewModel';
import { PageRequest, PagerRoutingState, PagerViewModel } from '../Pager/PagerViewModel';
export interface SortArgs {
    field: string;
    direction: SortDirection;
}
export interface DataSourceRequest<TContext = any> {
    page?: PageRequest;
    sort?: SortArgs;
    context?: TContext;
}
export interface DataSourceResponse<T> {
    items: IterableLike<T>;
    count: number;
}
export interface DataGridRoutingState {
    pager?: PagerRoutingState;
    sorting?: SortArgs;
}
export declare class DataGridViewModel<T, TRequestContext = any> extends ListItemsViewModel<T> implements RoutingStateHandler<DataGridRoutingState> {
    static displayName: string;
    static DefaultRateLimit: number;
    protected readonly processRequests: Command;
    protected readonly comparer: ObjectComparer<T>;
    readonly pager: PagerViewModel | null;
    readonly isLoading: ReadOnlyProperty<boolean>;
    readonly requests: ReadOnlyProperty<DataSourceRequest<TRequestContext> | undefined>;
    readonly responses: ReadOnlyProperty<DataSourceResponse<T> | undefined>;
    readonly projectedSource: ReadOnlyProperty<IterableLike<T>>;
    readonly projectedCount: ReadOnlyProperty<number>;
    readonly sorting: ReadOnlyProperty<SortArgs>;
    readonly sort: Command<SortArgs>;
    readonly toggleSortDirection: Command<string>;
    /**
     * @param source data source.
     * @param pager pager. if omitted a default pager will be created. use null for no pager.
     * @param context request context included in projection requests. if included requests are bound to context events.
     * @param comparer custom object comparer. if omitted a default object comparer will be used.
     */
    constructor(source: ObservableLike<IterableLike<T>>, pager?: PagerViewModel | null, context?: ObservableLike<TRequestContext>, comparer?: string | ObjectComparer<T>, rateLimit?: number);
    isRoutingStateHandler(): boolean;
    createRoutingState(changed?: HandlerRoutingStateChanged): DataGridRoutingState;
    applyRoutingState(state: DataGridRoutingState): void;
    getItemsSourceProperty(): ReadOnlyProperty<IterableLike<T>>;
    protected isSortChanged(sort: SortArgs | undefined): boolean;
    protected getReverseSortDirection(field: string, request: DataSourceRequest | undefined): SortDirection;
    protected getRequest(source: IterableLike<T>, page: PageRequest | undefined, sort: SortArgs | undefined, context: TRequestContext | undefined): DataSourceRequest<TRequestContext> | undefined;
    protected getRequests(context?: ObservableLike<TRequestContext>): Observable<DataSourceRequest<TRequestContext> | undefined>;
    protected getResponse(request: DataSourceRequest<TRequestContext> | undefined): ObservableOrValue<DataSourceResponse<T> | undefined>;
    protected getResponseFromItems(items: Iterable<T>, request: DataSourceRequest<TRequestContext>): ObservableOrValue<DataSourceResponse<T> | undefined>;
    protected getResponses(requests?: ObservableLike<DataSourceRequest<TRequestContext> | undefined>, rateLimit?: number): Observable<DataSourceResponse<T>>;
}
