Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React = require("react");
var react_bootstrap_1 = require("react-bootstrap");
var react_fa_1 = require("react-fa");
var React_1 = require("../../React");
var BindableInput_1 = require("../BindableInput/BindableInput");
var CommandButton_1 = require("../CommandButton/CommandButton");
var ContentTooltip_1 = require("../ContentTooltip/ContentTooltip");
var InlineEditView = /** @class */ (function (_super) {
    tslib_1.__extends(InlineEditView, _super);
    function InlineEditView() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    InlineEditView.prototype.handleKeyDown = function (e) {
        switch (e.keyCode) {
            case 13: // ENTER key
                this.viewModel.save.execute(null);
                break;
            case 27: // ESC key
                this.viewModel.cancel.execute(null);
                break;
            default:
                break;
        }
    };
    InlineEditView.prototype.focusAndSelectControlText = function (instance) {
        this.wxr.focusElement(instance, function (x) {
            if (x.select instanceof Function) {
                x.select();
            }
        });
    };
    InlineEditView.prototype.updateOn = function (viewModel) {
        return [
            viewModel.isEditing.changed,
            viewModel.hasSavingError.changed,
            viewModel.save.canExecuteObservable,
        ];
    };
    InlineEditView.prototype.render = function () {
        var _this = this;
        return this.wxr.renderConditional(this.viewModel.isEditing, function () { return _this.renderEditor(); }, function () { return _this.renderValue(); });
    };
    InlineEditView.prototype.renderErrorTooltip = function () {
        var _this = this;
        return (React.createElement("div", { className: "InlineEditView-errorContent" }, this.wxr.renderConditional(this.props.errorContent instanceof Function, function () {
            return _this.props.errorContent(_this.viewModel, _this);
        }, function () { return _this.props.errorContent; })));
    };
    InlineEditView.prototype.renderEditor = function () {
        var _this = this;
        var _a = this.restProps(function (x) {
            var controlId = x.controlId, inputType = x.inputType, placeholder = x.placeholder, converter = x.converter, valueProperty = x.valueProperty, onChangeProperty = x.onChangeProperty, valueGetter = x.valueGetter, valueSetter = x.valueSetter, keyboard = x.keyboard, clickToEdit = x.clickToEdit, template = x.template, editTemplate = x.editTemplate, errorContent = x.errorContent, errorPlacement = x.errorPlacement;
            return {
                controlId: controlId,
                inputType: inputType,
                placeholder: placeholder,
                converter: converter,
                valueProperty: valueProperty,
                onChangeProperty: onChangeProperty,
                valueGetter: valueGetter,
                valueSetter: valueSetter,
                keyboard: keyboard,
                clickToEdit: clickToEdit,
                template: template,
                editTemplate: editTemplate,
                errorContent: errorContent,
                errorPlacement: errorPlacement,
            };
        }), className = _a.className, props = _a.props, rest = _a.rest;
        return (React.createElement(react_bootstrap_1.FormGroup, tslib_1.__assign({}, rest, { className: this.wxr.classNames('InlineEditView', className) }),
            React.createElement(react_bootstrap_1.InputGroup, null,
                this.wxr.renderConditional(this.viewModel.hasSavingError, function () { return (React.createElement(ContentTooltip_1.ContentTooltip, { id: (props.controlId || 'inlineedit') + "-tt", className: "InlineEditView-popover alert-danger", placement: props.errorPlacement, popover: true, content: _this.renderErrorTooltip() },
                    React.createElement(react_bootstrap_1.InputGroup.Addon, { className: "InlineEditView-error" },
                        React.createElement(react_fa_1.Icon, { className: "alert-danger", name: "exclamation" })))); }),
                this.renderBindableInput(),
                React.createElement(react_bootstrap_1.InputGroup.Button, null,
                    React.createElement(CommandButton_1.CommandButton, { bsSize: this.props.bsSize, bsStyle: "success", command: this.viewModel.save },
                        React.createElement(react_fa_1.Icon, { name: "check" })),
                    React.createElement(CommandButton_1.CommandButton, { bsSize: this.props.bsSize, bsStyle: "danger", command: this.viewModel.cancel },
                        React.createElement(react_fa_1.Icon, { name: "times" }))))));
    };
    InlineEditView.prototype.renderBindableInput = function () {
        var _this = this;
        var props = this.restProps(function (x) {
            var converter = x.converter, valueProperty = x.valueProperty, onChangeProperty = x.onChangeProperty, valueGetter = x.valueGetter, valueSetter = x.valueSetter;
            return {
                converter: converter,
                valueProperty: valueProperty,
                onChangeProperty: onChangeProperty,
                valueGetter: valueGetter,
                valueSetter: valueSetter,
            };
        }).props;
        var onKeyDown = this.props.keyboard === true
            ? function (e) { return _this.handleKeyDown(e); }
            : undefined;
        return (React.createElement(BindableInput_1.BindableInput, tslib_1.__assign({}, props, { boundProperty: this.viewModel.editValue, onKeyDown: onKeyDown, disabled: this.viewModel.save.canExecute === false }), React.cloneElement(this.props.editTemplate(this.viewModel.editValue.value, this), {
            // ref: (x: Element | null) => this.focusAndSelectControlText(x),
            ref: function (x) { return _this.focusAndSelectControlText(x); },
        })));
    };
    InlineEditView.prototype.renderValue = function () {
        var _this = this;
        var _a = this.restProps(function (x) {
            var controlId = x.controlId, inputType = x.inputType, placeholder = x.placeholder, converter = x.converter, valueProperty = x.valueProperty, onChangeProperty = x.onChangeProperty, valueGetter = x.valueGetter, valueSetter = x.valueSetter, keyboard = x.keyboard, clickToEdit = x.clickToEdit, template = x.template, editTemplate = x.editTemplate, errorContent = x.errorContent, errorPlacement = x.errorPlacement;
            return {
                controlId: controlId,
                inputType: inputType,
                placeholder: placeholder,
                converter: converter,
                valueProperty: valueProperty,
                onChangeProperty: onChangeProperty,
                valueGetter: valueGetter,
                valueSetter: valueSetter,
                keyboard: keyboard,
                clickToEdit: clickToEdit,
                template: template,
                editTemplate: editTemplate,
                errorContent: errorContent,
                errorPlacement: errorPlacement,
            };
        }), className = _a.className, props = _a.props, rest = _a.rest;
        var displayContent = (React.createElement("span", null, this.props.template(this.viewModel.value.value, this)));
        return this.wxr.renderConditional(props.clickToEdit === true, function () { return (React.createElement(CommandButton_1.CommandButton, tslib_1.__assign({}, rest, { className: _this.wxr.classNames('InlineEditView', className), bsStyle: "link", command: _this.viewModel.edit }), displayContent)); }, function () { return displayContent; });
    };
    InlineEditView.displayName = 'InlineEditView';
    InlineEditView.defaultProps = {
        inputType: 'text',
        placeholder: 'Enter New Value...',
        template: function (x, view) { return x.toString(); },
        editTemplate: function (x, view) { return (React.createElement(react_bootstrap_1.FormControl, { type: view.props.inputType, placeholder: view.props.placeholder })); },
        errorContent: (React.createElement("div", null,
            React.createElement("div", null,
                React.createElement("strong", null, "Sorry, your change was not saved.")),
            React.createElement("div", null, "Please try again."))),
        errorPlacement: 'right',
    };
    return InlineEditView;
}(React_1.BaseView));
exports.InlineEditView = InlineEditView;
//# sourceMappingURL=InlineEditView.js.map