import { Iterable } from 'ix';
import { Observable } from 'rxjs';
import { ObjectComparer } from '../../../Utils/Compare';
import { IterableLike, ObservableLike, ObservableOrValue } from '../../../WebRx';
import { HandlerRoutingStateChanged, RoutingStateHandler } from '../../React';
import { DataGridRoutingState, DataGridViewModel, DataSourceRequest, DataSourceResponse } from '../DataGrid/DataGridViewModel';
import { PagerViewModel } from '../Pager/PagerViewModel';
import { SearchRequest, SearchRoutingState, SearchViewModel } from '../Search/SearchViewModel';
export interface ItemListPanelRequestContext {
    search?: SearchRequest;
}
export declare type ItemListPanelContext<TRequestContext> = ItemListPanelRequestContext & TRequestContext;
export interface ItemListPanelRoutingState extends DataGridRoutingState {
    search?: SearchRoutingState;
}
export declare class ItemListPanelViewModel<T, TRequestContext = any> extends DataGridViewModel<T, ItemListPanelContext<TRequestContext>> implements RoutingStateHandler<ItemListPanelRoutingState> {
    static displayName: string;
    protected static getItemListPanelSearch(search: SearchViewModel | undefined | null): SearchViewModel | null;
    protected static createItemListPanelContext(search: SearchRequest): ItemListPanelRequestContext;
    protected static getItemListPanelContext(search: SearchViewModel | null): Observable<ItemListPanelRequestContext>;
    protected static getDataGridContext<TRequestContext>(search: SearchViewModel | null, context?: ObservableLike<TRequestContext>): Observable<ItemListPanelContext<TRequestContext>>;
    static getSearchRequest<TRequestContext>(request: DataSourceRequest<ItemListPanelContext<TRequestContext>>): SearchRequest | undefined;
    static filterItems<T>(items: Iterable<T>, searchRequest: SearchRequest, filterer: (item: T, search: SearchRequest) => boolean): Iterable<T>;
    protected readonly filterer: undefined | ((item: T, search: SearchRequest) => boolean);
    readonly search: SearchViewModel | null;
    /**
     * @param source data source.
     * @param filterer filter predicate. executed for each item when the search context is available.
     * @param search search handler. if omitted a default search handler will be created. use null for no search handling.
     * @param pager pager. if omitted a default pager will be created. use null for no pager.
     * @param context request context included in projection requests. if included requests are bound to context events.
     * @param comparer custom object comparer. if omitted a default object comparer will be used.
     */
    constructor(source: ObservableLike<IterableLike<T>>, filterer?: (item: T, search: SearchRequest) => boolean, search?: SearchViewModel | null, pager?: PagerViewModel | null, context?: ObservableLike<TRequestContext>, comparer?: string | ObjectComparer<T>, rateLimit?: number);
    isRoutingStateHandler(): boolean;
    createRoutingState(changed?: HandlerRoutingStateChanged): ItemListPanelRoutingState;
    applyRoutingState(state: ItemListPanelRoutingState): void;
    getResponseFromItems(items: Iterable<T>, request: DataSourceRequest<ItemListPanelContext<TRequestContext>>): ObservableOrValue<DataSourceResponse<T> | undefined>;
    protected getFilteredItems(items: Iterable<T>, searchRequest: SearchRequest): Iterable<T>;
}
