import { Iterable } from 'ix';
import { IterableLike, ObservableLike } from '../../../WebRx';
import { SearchRequest, SearchViewModel } from '../Search/SearchViewModel';
import { ItemListPanelViewModel } from './ItemListPanelViewModel';
export declare function filterTreeItems<T>(items: IterableLike<T> | undefined, itemsSource: (item: T) => IterableLike<T> | undefined, itemsAssign: (item: T, items: Iterable<T>) => T, filter: (item: T) => boolean): Iterable<T>;
export declare class TreeItemListPanelViewModel<T, TRequestContext = any> extends ItemListPanelViewModel<T, TRequestContext> {
    protected readonly itemsSource: (item: T) => IterableLike<T> | undefined;
    protected readonly itemsAssign: (item: T, items: Iterable<T>) => T;
    static displayName: string;
    /**
     * @param source data source.
     * @param itemsSource delegate to produce sub-items from a source item.
     * @param itemsAssign delegate to produce a source item with the sub-items assigned.
     * @param filterer filter predicate. executed for each item when the search context is available.
     * @param search search handler. if omitted a default search handler will be created. use null for no search handling.
     * @param context request context included in projection requests. if included requests are bound to context events.
     */
    constructor(source: ObservableLike<IterableLike<T>>, itemsSource: (item: T) => IterableLike<T> | undefined, itemsAssign: (item: T, items: Iterable<T>) => T, filterer?: (item: T, search: SearchRequest) => boolean, search?: SearchViewModel | null, context?: ObservableLike<TRequestContext>, rateLimit?: number);
    getItems(): Iterable<T>;
    getItemsForIndicies(indicies: IterableLike<number>): undefined;
    getIndiciesForItems(items: IterableLike<T>): undefined;
    getFilteredItems(items: Iterable<T>, searchRequest: SearchRequest): Iterable<T>;
    protected flattenItems(item: T): Iterable<T>;
}
