Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var ix_1 = require("ix");
var TreeListItemsViewModel_1 = require("../ListItems/TreeListItemsViewModel");
var ItemListPanelViewModel_1 = require("./ItemListPanelViewModel");
function filterTreeItems(items, itemsSource, itemsAssign, filter) {
    if (items == null) {
        return ix_1.Iterable.empty();
    }
    return ix_1.Iterable.from(items)
        .map(function (x) {
        var result = filterTreeItems(itemsSource(x), itemsSource, itemsAssign, filter);
        if (result.some(function () { return true; }) || filter(x)) {
            return itemsAssign(x, result);
        }
        return undefined;
    })
        .filterNull();
}
exports.filterTreeItems = filterTreeItems;
var TreeItemListPanelViewModel = /** @class */ (function (_super) {
    tslib_1.__extends(TreeItemListPanelViewModel, _super);
    /**
     * @param source data source.
     * @param itemsSource delegate to produce sub-items from a source item.
     * @param itemsAssign delegate to produce a source item with the sub-items assigned.
     * @param filterer filter predicate. executed for each item when the search context is available.
     * @param search search handler. if omitted a default search handler will be created. use null for no search handling.
     * @param context request context included in projection requests. if included requests are bound to context events.
     */
    function TreeItemListPanelViewModel(source, itemsSource, itemsAssign, filterer, search, context, rateLimit) {
        var _this = _super.call(this, source, filterer, search, null, context, undefined, rateLimit) || this;
        _this.itemsSource = itemsSource;
        _this.itemsAssign = itemsAssign;
        return _this;
    }
    TreeItemListPanelViewModel.prototype.getItems = function () {
        var _this = this;
        return ix_1.Iterable.from(this.getItemsSource()).flatMap(function (x) {
            return _this.flattenItems(x);
        });
    };
    TreeItemListPanelViewModel.prototype.getItemsForIndicies = function (indicies) {
        return undefined;
    };
    TreeItemListPanelViewModel.prototype.getIndiciesForItems = function (items) {
        return undefined;
    };
    TreeItemListPanelViewModel.prototype.getFilteredItems = function (items, searchRequest) {
        var _this = this;
        return filterTreeItems(items, this.itemsSource, this.itemsAssign, function (x) {
            return _this.filterer(x, searchRequest);
        });
    };
    TreeItemListPanelViewModel.prototype.flattenItems = function (item) {
        return TreeListItemsViewModel_1.flattenItems(item, this.itemsSource);
    };
    TreeItemListPanelViewModel.displayName = 'TreeItemListPanelViewModel';
    return TreeItemListPanelViewModel;
}(ItemListPanelViewModel_1.ItemListPanelViewModel));
exports.TreeItemListPanelViewModel = TreeItemListPanelViewModel;
//# sourceMappingURL=TreeItemListPanelViewModel.js.map