Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var ix_1 = require("ix");
var React = require("react");
var Panel_1 = require("../Panel");
var ItemsPresenter = /** @class */ (function (_super) {
    tslib_1.__extends(ItemsPresenter, _super);
    function ItemsPresenter() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ItemsPresenter.defaultItemTemplate = function (item, index, context) {
        return React.createElement("div", { key: index }, String.stringify(item));
    };
    ItemsPresenter.defaultPanelTemplate = function (itemTemplates, itemsPresenter) {
        return (React.createElement(Panel_1.StackPanel, { itemClassName: itemsPresenter.props.itemClassName, itemStyle: itemsPresenter.props.itemStyle, itemProps: itemsPresenter.props.itemProps, compact: itemsPresenter.props.compact, emptyContent: itemsPresenter.props.emptyContent }, itemTemplates));
    };
    ItemsPresenter.defaultViewTemplate = function (itemsPanel, itemsPresenter) {
        var _a = itemsPresenter.restProps(function (x) {
            var itemsSource = x.itemsSource, viewTemplate = x.viewTemplate, itemsPanelTemplate = x.itemsPanelTemplate, itemTemplate = x.itemTemplate, itemClassName = x.itemClassName, itemStyle = x.itemStyle, itemProps = x.itemProps, compact = x.compact, emptyContent = x.emptyContent;
            return {
                itemsSource: itemsSource,
                viewTemplate: viewTemplate,
                itemsPanelTemplate: itemsPanelTemplate,
                itemTemplate: itemTemplate,
                itemClassName: itemClassName,
                itemStyle: itemStyle,
                itemProps: itemProps,
                compact: compact,
                emptyContent: emptyContent,
            };
        }), className = _a.className, props = _a.props, rest = _a.rest;
        // allow items presenter to optionally fill its child panel by omitting the presenter wrapper
        if (React.isValidElement(itemsPanel) &&
            itemsPanel.props.fill) {
            var _b = itemsPanel.props, fill = _b.fill, panelProps = tslib_1.__rest(_b, ["fill"]);
            return React.createElement(itemsPanel.type, tslib_1.__assign({}, panelProps));
        }
        return (React.createElement("div", tslib_1.__assign({}, rest, { className: itemsPresenter.wxr.classNames('ItemsPresenter', className) }), itemsPanel));
    };
    ItemsPresenter.prototype.render = function () {
        return this.renderViewTemplate();
    };
    ItemsPresenter.prototype.getItemsPanelTemplate = function () {
        if (this.props.itemsPanelTemplate != null) {
            return this.props.itemsPanelTemplate;
        }
        if (this.props.itemsSource != null &&
            React.Children.count(this.props.children) === 1) {
            var itemsPanel_1 = React.Children.only(this.props.children);
            if (React.isValidElement(itemsPanel_1)) {
                return function (items) {
                    var children = items.concat(React.Children.toArray(itemsPanel_1.props.children));
                    return React.cloneElement(itemsPanel_1, {}, children);
                };
            }
            return function () { return itemsPanel_1; };
        }
        return ItemsPresenter.defaultPanelTemplate;
    };
    ItemsPresenter.prototype.renderItemTemplates = function () {
        var template = this.props.itemTemplate || ItemsPresenter.defaultItemTemplate;
        if (this.props.itemsSource == null) {
            return {
                items: undefined,
                itemTemplates: React.Children.toArray(this.props.children),
            };
        }
        var items = ix_1.Iterable.from(this.props.itemsSource).toArray();
        var itemTemplates = items.map(function (x, i) {
            var item = template(x, i);
            return item != null && React.isValidElement(item) && item.key == null
                ? React.cloneElement(item, { key: i })
                : item;
        });
        return { items: items, itemTemplates: itemTemplates };
    };
    ItemsPresenter.prototype.renderPanelTemplate = function () {
        var template = this.getItemsPanelTemplate();
        var _a = this.renderItemTemplates(), items = _a.items, itemTemplates = _a.itemTemplates;
        return template(itemTemplates, this, items);
    };
    ItemsPresenter.prototype.renderViewTemplate = function () {
        var template = this.props.viewTemplate || ItemsPresenter.defaultViewTemplate;
        var itemsPanel = this.renderPanelTemplate();
        if (React.isValidElement(itemsPanel)) {
            var itemsPresenterPanelProps = Object.trim({
                itemClassName: this.props.itemClassName,
                itemStyle: this.props.itemStyle,
                itemProps: this.props.itemProps,
                compact: this.props.compact,
                emptyContent: this.props.emptyContent,
                fill: this.props.fill,
            });
            var itemsPanelProps = React.isValidElement(itemsPanel)
                ? itemsPanel.props
                : {};
            return template(React.cloneElement(itemsPanel, tslib_1.__assign({}, itemsPresenterPanelProps, itemsPanelProps)), this);
        }
        return template(itemsPanel, this);
    };
    ItemsPresenter.displayName = 'ItemsPresenter';
    return ItemsPresenter;
}(React.Component));
exports.ItemsPresenter = ItemsPresenter;
//# sourceMappingURL=ItemsPresenter.js.map