Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var ix_1 = require("ix");
var React = require("react");
var react_bootstrap_1 = require("react-bootstrap");
var react_fa_1 = require("react-fa");
var ItemsPresenter_1 = require("./ItemsPresenter");
var TreeItem = /** @class */ (function (_super) {
    tslib_1.__extends(TreeItem, _super);
    function TreeItem(props) {
        var _this = _super.call(this, props) || this;
        _this.toggleExpansion = _this.toggleExpansion.bind(_this);
        _this.state = {
            isExpanded: _this.props.startExpanded || false,
        };
        return _this;
    }
    TreeItem.generateKey = function (fragment, index) {
        if (React.isValidElement(fragment) && fragment.key != null) {
            return fragment.key;
        }
        return index;
    };
    TreeItem.renderTreeItem = function (item, index, props) {
        return (React.createElement(TreeItem, { key: index, item: item, index: index, itemsSource: props.itemsSource, expanderIconTemplate: props.expanderIconTemplate, headerTemplate: props.headerTemplate, itemsTemplate: props.itemsTemplate, depth: props.depth == null ? 0 : props.depth + 1, startExpanded: props.startExpanded, overrideExpanded: props.overrideExpanded, expandedIconName: props.expandedIconName, collapsedIconName: props.collapsedIconName, clickToExpand: props.clickToExpand, viewTemplate: props.viewTemplate, itemsPanelTemplate: props.itemsPanelTemplate, itemTemplate: props.itemTemplate, itemClassName: props.itemClassName, itemStyle: props.itemStyle, itemProps: props.itemProps, compact: props.compact, emptyContent: props.emptyContent }));
    };
    TreeItem.defaultHeaderTemplate = function (item, index, indent, expander, headerContent, view) {
        return (React.createElement("div", { className: "TreeItem-Header", onClick: view.props.clickToExpand
                ? view.toggleExpansion.bind(view)
                : undefined },
            indent,
            React.createElement("div", { className: "TreeItem-Expander" }, expander),
            React.createElement("div", { className: "TreeItem-HeaderContent" }, headerContent)));
    };
    TreeItem.defaultItemsTemplate = function (items, view) {
        var isExpanded = view.getIsExpanded();
        var template = function (item, index) {
            return TreeItem.renderTreeItem(item, index, view.props);
        };
        return isExpanded === false || items == null ? (false) : (React.createElement(ItemsPresenter_1.ItemsPresenter, { itemsSource: items, viewTemplate: view.props.viewTemplate, itemsPanelTemplate: view.props.itemsPanelTemplate, itemTemplate: template, itemClassName: view.props.itemClassName, itemStyle: view.props.itemStyle, itemProps: view.props.itemProps, compact: view.props.compact, emptyContent: view.props.emptyContent }));
    };
    TreeItem.defaultExpanderIconTemplate = function (isExpanded, expandedIconName, collapsedIconName) {
        if (expandedIconName === void 0) { expandedIconName = TreeItem.defaultExpandedIconName; }
        if (collapsedIconName === void 0) { collapsedIconName = TreeItem.defaultCollapsedIconName; }
        var iconName = isExpanded === true ? expandedIconName : collapsedIconName;
        return React.createElement(react_fa_1.Icon, { name: iconName, size: "lg", fixedWidth: true });
    };
    TreeItem.prototype.componentWillReceiveProps = function (nextProps, nextContext) {
        if (nextProps.startExpanded != null &&
            nextProps.startExpanded !== this.state.isExpanded) {
            this.setState(function (prevState, props) {
                // props.startExpanded should not be null (as per above) but we do a check just in case
                if (props.startExpanded == null) {
                    return prevState;
                }
                return {
                    isExpanded: props.startExpanded,
                };
            });
        }
    };
    TreeItem.prototype.render = function () {
        var _a = this.restProps(function (x) {
            var item = x.item, index = x.index, itemsSource = x.itemsSource, expanderIconTemplate = x.expanderIconTemplate, headerTemplate = x.headerTemplate, itemsTemplate = x.itemsTemplate, depth = x.depth, startExpanded = x.startExpanded, overrideExpanded = x.overrideExpanded, expandedIconName = x.expandedIconName, collapsedIconName = x.collapsedIconName, clickToExpand = x.clickToExpand, viewTemplate = x.viewTemplate, itemsPanelTemplate = x.itemsPanelTemplate, itemTemplate = x.itemTemplate, itemClassName = x.itemClassName, itemStyle = x.itemStyle, itemProps = x.itemProps, compact = x.compact, emptyContent = x.emptyContent;
            return {
                item: item,
                index: index,
                itemsSource: itemsSource,
                expanderIconTemplate: expanderIconTemplate,
                headerTemplate: headerTemplate,
                itemsTemplate: itemsTemplate,
                depth: depth,
                startExpanded: startExpanded,
                overrideExpanded: overrideExpanded,
                expandedIconName: expandedIconName,
                collapsedIconName: collapsedIconName,
                clickToExpand: clickToExpand,
                viewTemplate: viewTemplate,
                itemsPanelTemplate: itemsPanelTemplate,
                itemTemplate: itemTemplate,
                itemClassName: itemClassName,
                itemStyle: itemStyle,
                itemProps: itemProps,
                compact: compact,
                emptyContent: emptyContent,
            };
        }), className = _a.className, props = _a.props, rest = _a.rest, children = _a.children;
        var headerContent = this.renderHeaderContent();
        var key = TreeItem.generateKey(headerContent, this.props.index);
        var items = this.props.itemsSource(this.props.item);
        var indent = this.renderIndent();
        var expander = this.renderExpander(items);
        var header = this.renderHeader(indent, expander, headerContent);
        var treeItems = this.renderItems(items);
        return (React.createElement("div", tslib_1.__assign({ key: key }, rest, { className: this.wxr.classNames('TreeItem', className) }),
            header,
            treeItems && (React.createElement("div", { className: "TreeItem-Expansion" },
                React.createElement("div", { className: "TreeItem-Items" }, treeItems)))));
    };
    TreeItem.prototype.getIsExpanded = function () {
        if (this.props.overrideExpanded != null) {
            // if an override prop is provided, then check to see if it is set to true
            var override = this.props.overrideExpanded instanceof Function
                ? this.props.overrideExpanded()
                : this.props.overrideExpanded;
            if (override) {
                // override is set to true so return true for isExpanded state (overridden)
                return true;
            }
        }
        // either no override prop is provided or the override result was false
        // so fallback onto standard component state
        return this.state.isExpanded || false;
    };
    TreeItem.prototype.toggleExpansion = function (e) {
        var _this = this;
        e.stopPropagation();
        this.setState(function (prevState, props) {
            return {
                isExpanded: !_this.state.isExpanded,
            };
        });
    };
    TreeItem.prototype.renderIndent = function () {
        return ix_1.Iterable.range(0, this.props.depth || 0)
            .map(function (x, i) {
            return React.createElement("div", { key: i, className: "TreeItem-Indent" });
        })
            .defaultIfEmpty(false)
            .toArray();
    };
    TreeItem.prototype.renderExpanderIcon = function () {
        var template = this.props.expanderIconTemplate || TreeItem.defaultExpanderIconTemplate;
        var isExpanded = this.getIsExpanded();
        return template(isExpanded, this.props.expandedIconName, this.props.collapsedIconName);
    };
    TreeItem.prototype.renderExpander = function (items) {
        var isEmpty = items == null || ix_1.Iterable.from(items).isEmpty();
        return isEmpty ? (false) : (React.createElement(react_bootstrap_1.Button, { bsStyle: "link", onClick: this.toggleExpansion }, this.renderExpanderIcon()));
    };
    TreeItem.prototype.renderHeaderContent = function () {
        var itemTemplate = this.props.itemTemplate || ItemsPresenter_1.ItemsPresenter.defaultItemTemplate;
        return itemTemplate(this.props.item, this.props.index, this.state);
    };
    TreeItem.prototype.renderHeader = function (indent, expander, headerContent) {
        var template = this.props.headerTemplate || TreeItem.defaultHeaderTemplate;
        return template(this.props.item, this.props.index, indent, expander, headerContent, this);
    };
    TreeItem.prototype.renderItems = function (items) {
        var template = this.props.itemsTemplate || TreeItem.defaultItemsTemplate;
        return template(items, this);
    };
    TreeItem.defaultProps = {
        depth: 0,
    };
    TreeItem.defaultExpandedIconName = 'minus-square-o';
    TreeItem.defaultCollapsedIconName = 'plus-square-o';
    return TreeItem;
}(React.Component));
exports.TreeItem = TreeItem;
//# sourceMappingURL=TreeItem.js.map