Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var classNames = require("classnames");
var ix_1 = require("ix");
var React = require("react");
var Utils_1 = require("../../../Utils");
var TablePanel_1 = require("../Panel/TablePanel");
var GridViewColumn_1 = require("./GridViewColumn");
var ListItemsViewTemplate_1 = require("./ListItemsViewTemplate");
var PanelView_1 = require("./PanelView");
var GridView = /** @class */ (function (_super) {
    tslib_1.__extends(GridView, _super);
    function GridView(props) {
        var _this = _super.call(this, props) || this;
        _this.logger = Utils_1.Logging.getLogger(GridView.displayName);
        _this.renderTablePanel = _this.renderTablePanel.bind(_this);
        return _this;
    }
    GridView.prototype.render = function () {
        var _a = this.restProps(function (x) {
            var headerTemplate = x.headerTemplate, cellTemplate = x.cellTemplate, bordered = x.bordered, condensed = x.condensed, hover = x.hover, responsive = x.responsive, striped = x.striped, bsClass = x.bsClass, fixedLayout = x.fixedLayout, listItems = x.listItems, itemsProps = x.itemsProps;
            return {
                headerTemplate: headerTemplate,
                cellTemplate: cellTemplate,
                bordered: bordered,
                condensed: condensed,
                hover: hover,
                responsive: responsive,
                striped: striped,
                bsClass: bsClass,
                fixedLayout: fixedLayout,
                listItems: listItems,
                itemsProps: itemsProps,
            };
        }), className = _a.className, children = _a.children, props = _a.props, rest = _a.rest;
        this.columns = this.getColumnDefinitions();
        if (this.columns == null) {
            return null;
        }
        var gridProps = this.getItemsProps();
        gridProps.itemsPanelTemplate =
            gridProps.itemsPanelTemplate || this.renderTablePanel.bind(this);
        gridProps.itemTemplate =
            gridProps.itemTemplate || this.renderTableRow.bind(this);
        return (React.createElement(PanelView_1.PanelView, tslib_1.__assign({ className: classNames('Grid', className), itemsPanelTemplate: this.renderTablePanel, listItems: props.listItems, itemsProps: gridProps }, this.trimProps(rest)), children));
    };
    GridView.prototype.renderTablePanel = function (itemTemplates, itemsPresenter, items) {
        var props = this.restProps(function (x) {
            var bordered = x.bordered, condensed = x.condensed, hover = x.hover, responsive = x.responsive, striped = x.striped, bsClass = x.bsClass, fixedLayout = x.fixedLayout;
            return {
                bordered: bordered,
                condensed: condensed,
                hover: hover,
                responsive: responsive,
                striped: striped,
                bsClass: bsClass,
                fixedLayout: fixedLayout,
            };
        }).props;
        return (React.createElement(TablePanel_1.TablePanel, tslib_1.__assign({ header: this.renderTableHeaderRow() }, this.trimProps(props)), itemTemplates));
    };
    GridView.prototype.getColumnDefinitions = function () {
        var count = React.Children.count(this.props.children);
        if (count === 0) {
            // try and auto-gen columns
            var item = this.getListItems()
                .getItems()
                .first();
            if (item == null) {
                this.logger.warn('Unable to Autogenerate Columns');
                return undefined;
            }
            return ix_1.Iterable.from(Object.keys(item))
                .orderBy(function (x) { return x; })
                .map(function (x) { return React.createElement(GridViewColumn_1.GridViewColumn, { key: x, field: x }); })
                .toArray();
        }
        if (count === 1) {
            var elem = React.Children.only(this.props.children);
            if (React.isType(elem, GridViewColumn_1.GridViewColumns)) {
                return React.Children.toArray(elem.props.children);
            }
        }
        return React.Children.toArray(this.props.children);
    };
    GridView.prototype.renderTableHeaderRow = function () {
        if (this.columns == null) {
            return undefined;
        }
        var renderHeaders = ix_1.Iterable.from(this.columns).some(function (x) {
            return GridViewColumn_1.GridViewColumn.canRenderHeader(x);
        });
        if (renderHeaders) {
            var props_1 = this.trimProps({
                itemTemplate: this.props.headerTemplate,
            });
            return (React.createElement("tr", null, this.columns.map(function (x) {
                if (React.isValidElement(x)) {
                    return React.createElement(x.type, tslib_1.__assign({ key: x.key }, x.props, props_1));
                }
                return '';
            })));
        }
        return undefined;
    };
    GridView.prototype.renderTableRow = function (item, index) {
        var columns = this.getColumnDefinitions();
        var props = this.trimProps({
            item: item,
            itemTemplate: this.props.cellTemplate,
        });
        return columns == null
            ? undefined
            : this.renderListItem(React.createElement("tr", null, columns.map(function (x) {
                if (React.isValidElement(x)) {
                    return React.createElement(x.type, tslib_1.__assign({ key: x.key }, x.props, props));
                }
                return '';
            })), item, PanelView_1.PanelView.getSelectedProps);
    };
    GridView.displayName = 'GridView';
    GridView.Columns = GridViewColumn_1.GridViewColumns;
    return GridView;
}(ListItemsViewTemplate_1.ListItemsViewTemplate));
exports.GridView = GridView;
//# sourceMappingURL=GridView.js.map