// tslint:disable:max-classes-per-file
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React = require("react");
var ContentTooltip_1 = require("../ContentTooltip/ContentTooltip");
var NavButton_1 = require("../NavButton/NavButton");
var GridViewColumns = /** @class */ (function (_super) {
    tslib_1.__extends(GridViewColumns, _super);
    function GridViewColumns() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    GridViewColumns.prototype.render = function () {
        return this.props.children;
    };
    return GridViewColumns;
}(React.Component));
exports.GridViewColumns = GridViewColumns;
var GridColumnComponentType;
(function (GridColumnComponentType) {
    GridColumnComponentType[GridColumnComponentType["Header"] = 0] = "Header";
    GridColumnComponentType[GridColumnComponentType["Cell"] = 1] = "Cell";
    GridColumnComponentType[GridColumnComponentType["Content"] = 2] = "Content";
    GridColumnComponentType[GridColumnComponentType["Tooltip"] = 3] = "Tooltip";
})(GridColumnComponentType = exports.GridColumnComponentType || (exports.GridColumnComponentType = {}));
var GridViewColumn = /** @class */ (function (_super) {
    tslib_1.__extends(GridViewColumn, _super);
    function GridViewColumn() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    GridViewColumn.canRenderHeader = function (column) {
        return (React.isValidType(column, GridViewColumn) &&
            (column.props.header != null || column.props.field != null));
    };
    GridViewColumn.sanitizeFragment = function (content) {
        if (content == null) {
            return React.createElement("span", null, "\u00A0");
        }
        return content;
    };
    GridViewColumn.renderItemField = function (item, field) {
        if (String.isNullOrEmpty(field)) {
            return undefined;
        }
        return item[field];
    };
    GridViewColumn.prototype.render = function () {
        var col = this.props.item == null ? this.renderHeader() : this.renderCell();
        return col == null || React.isValidElement(col) ? (col) : (React.createElement("div", null, col || null));
    };
    GridViewColumn.prototype.renderContent = function (type, content, item, field) {
        var fragment = (React.createElement("div", { className: "GridViewColumn-" + type + "Content" }, GridViewColumn.sanitizeFragment(content)));
        if (this.props.itemTemplate == null) {
            return fragment;
        }
        return this.props.itemTemplate(fragment, item, field);
    };
    GridViewColumn.prototype.renderHeader = function (headerTemplate) {
        headerTemplate = headerTemplate || this.props.headerTemplate;
        var template = headerTemplate || (function (x) { return x; });
        var headerOrField = this.props.header || this.props.field;
        var content = headerOrField == null && this.props.headerTemplate == null
            ? undefined
            : template(headerOrField);
        var headerContent = this.renderContent('header', content, undefined, this.props.field);
        var tooltipContent = this.renderHeaderTooltip(headerContent);
        var style = Object.assign({ width: this.props.width }, this.props.headerStyle);
        var props = Object.assign({ style: style }, this.props.headerProps);
        return (React.createElement("th", tslib_1.__assign({ className: this.getClassNameForComponent(GridColumnComponentType.Header) }, props), this.renderTooltip(tooltipContent, headerContent)));
    };
    GridViewColumn.prototype.renderHeaderTooltip = function (context) {
        if (this.props.headerTooltipTemplate == null) {
            return undefined;
        }
        if (this.props.headerTooltipTemplate instanceof Function) {
            return this.props.headerTooltipTemplate(this, context);
        }
        return this.props.headerTooltipTemplate;
    };
    GridViewColumn.prototype.renderCell = function (cellTemplate) {
        cellTemplate = cellTemplate || this.props.cellTemplate;
        var template = cellTemplate || (function (x, f) { return GridViewColumn.renderItemField(x, f); });
        var content = template(this.props.item, this.props.field);
        var cellContent = this.renderContent('cell', content, this.props.item, this.props.field);
        var tooltipContent = this.renderCellTooltip(cellContent);
        var style = Object.assign({ width: this.props.width }, this.props.cellStyle);
        var props = Object.assign({ style: style }, this.props.cellProps);
        return (React.createElement("td", tslib_1.__assign({ className: this.getClassNameForComponent(GridColumnComponentType.Cell) }, props), this.renderTooltip(tooltipContent, cellContent)));
    };
    GridViewColumn.prototype.renderCellTooltip = function (context) {
        if (this.props.cellTooltipTemplate == null) {
            return undefined;
        }
        return this.props.cellTooltipTemplate(this.props.item, this.props.field, context);
    };
    GridViewColumn.prototype.renderTooltip = function (content, context) {
        if (content == null) {
            return context;
        }
        var id = this.props.id || this.props.field;
        if (React.isValidType(content, ContentTooltip_1.ContentTooltip)) {
            var child = content.props.context == null &&
                React.Children.count(content.props.children) === 0
                ? context
                : undefined;
            return React.cloneElement(content, tslib_1.__assign({ id: id, className: this.getClassNameForComponent(GridColumnComponentType.Content) }, content.props), child);
        }
        return (React.createElement(ContentTooltip_1.ContentTooltip, { id: id, className: this.getClassNameForComponent(GridColumnComponentType.Tooltip), content: content }, context));
    };
    GridViewColumn.prototype.getClassNameForComponent = function (type) {
        return this.props.className;
    };
    GridViewColumn.displayName = 'GridViewColumn';
    return GridViewColumn;
}(React.Component));
exports.GridViewColumn = GridViewColumn;
var NavButtonColumn = /** @class */ (function (_super) {
    tslib_1.__extends(NavButtonColumn, _super);
    function NavButtonColumn() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    NavButtonColumn.prototype.renderCell = function () {
        return _super.prototype.renderCell.call(this, this.renderNavButton.bind(this));
    };
    NavButtonColumn.prototype.getClassNameForComponent = function (type) {
        return this.wxr.classNames('NavButtonColumn', _super.prototype.getClassNameForComponent.call(this, type));
    };
    NavButtonColumn.prototype.renderNavButton = function (item, field) {
        var href = this.props.href instanceof Function
            ? this.props.href(item, field)
            : this.props.href;
        return React.createElement(NavButton_1.NavButton, { href: href, compact: true });
    };
    return NavButtonColumn;
}(GridViewColumn));
exports.NavButtonColumn = NavButtonColumn;
//# sourceMappingURL=GridViewColumn.js.map