Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var ix_1 = require("ix");
var React = require("react");
var rxjs_1 = require("rxjs");
var Compare_1 = require("../../../Utils/Compare");
var WebRx_1 = require("../../../WebRx");
var SelectableListItem = /** @class */ (function (_super) {
    tslib_1.__extends(SelectableListItem, _super);
    function SelectableListItem() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.isSelectedSubscription = rxjs_1.Subscription.EMPTY;
        return _this;
    }
    SelectableListItem.prototype.subscribeToUpdates = function (listItems) {
        var _this = this;
        this.unsubscribeFromUpdates();
        this.isSelectedSubscription = WebRx_1.wx
            .whenAny(listItems.selectedItems, function (x) { return x; })
            // skip the first result (we already a render queued)
            .skip(1)
            // remove nulls (shouldn't be possible)
            .filterNull()
            .subscribe(function () { return _this.forceUpdate(); });
    };
    SelectableListItem.prototype.unsubscribeFromUpdates = function () {
        if (this.isSelectedSubscription !== rxjs_1.Subscription.EMPTY) {
            this.isSelectedSubscription = rxjs_1.Subscription.unsubscribe(this.isSelectedSubscription);
        }
    };
    SelectableListItem.prototype.componentDidMount = function () {
        this.subscribeToUpdates(this.props.listItems);
    };
    SelectableListItem.prototype.componentWillUpdate = function (nextProps) {
        this.subscribeToUpdates(nextProps.listItems);
    };
    SelectableListItem.prototype.componentWillUnmount = function () {
        this.unsubscribeFromUpdates();
    };
    SelectableListItem.prototype.render = function () {
        var elem = React.Children.only(this.props.children);
        return React.cloneElement(elem, this.getListItemProps(elem));
    };
    SelectableListItem.prototype.getIsSelected = function () {
        var items = this.props.listItems.selectedItems.value;
        if (items == null) {
            return false;
        }
        return items.indexOf(this.props.item) >= 0;
    };
    SelectableListItem.prototype.getListItemProps = function (elem) {
        var _this = this;
        var onClickElement = elem.props.onClick;
        return Object.assign({}, this.props.selectedProps(this.getIsSelected(), elem), {
            onClick: function (e) {
                _this.handleClick(e, onClickElement);
            },
        });
    };
    // tslint:disable-next-line:ban-types
    SelectableListItem.prototype.handleClick = function (e, onClickElement) {
        e.stopPropagation();
        e.preventDefault();
        if (e.ctrlKey) {
            this.props.listItems.selectItems.execute(ix_1.Iterable.from(this.props.listItems.selectedItems.value)
                .startWith(this.props.item)
                .filterNull()
                .distinct(undefined, Compare_1.compare)
                .toArray());
        }
        else if (e.shiftKey) {
            document.getSelection().removeAllRanges();
            var from = this.props.listItems.selectedItems.value[0] || this.props.item;
            this.props.listItems.selectRange.execute({ from: from, to: this.props.item });
        }
        else {
            this.props.listItems.selectItem.execute(this.props.item);
        }
        if (onClickElement != null) {
            onClickElement(e);
        }
    };
    SelectableListItem.defaultProps = {
        selectedProps: function () { return ({}); },
    };
    return SelectableListItem;
}(React.Component));
exports.SelectableListItem = SelectableListItem;
//# sourceMappingURL=SelectableListItem.js.map