Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React = require("react");
var react_bootstrap_1 = require("react-bootstrap");
var rxjs_1 = require("rxjs");
var React_1 = require("../../React");
function renderLoadable(isLoading, loadingComponent, loadedComponent) {
    var action;
    if (loadingComponent instanceof Function) {
        action = loadingComponent;
    }
    else if (String.isString(loadingComponent)) {
        var text_1 = loadingComponent;
        action = function () { return React.createElement(Loading, { text: text_1 }); };
    }
    else if (!React.isValidElement(loadingComponent) &&
        Object.isObject(loadingComponent)) {
        var props_1 = loadingComponent;
        action = function () { return React.createElement(Loading, tslib_1.__assign({}, props_1)); };
    }
    else {
        action = function () { return React.createElement(Loading, null); };
    }
    return React_1.wxr.renderConditional(isLoading, action, loadedComponent);
}
exports.renderLoadable = renderLoadable;
function renderSizedLoadable(isLoading, text, fontSize, loadedComponent) {
    return renderLoadable(isLoading, {
        text: text,
        fontSize: fontSize,
    }, loadedComponent);
}
exports.renderSizedLoadable = renderSizedLoadable;
function renderGridLoadable(isLoading, text, fontSize, loadedComponent) {
    return renderLoadable(isLoading, {
        text: text,
        fontSize: fontSize,
        componentClass: react_bootstrap_1.Grid,
    }, loadedComponent);
}
exports.renderGridLoadable = renderGridLoadable;
var Loading = /** @class */ (function (_super) {
    tslib_1.__extends(Loading, _super);
    function Loading() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.changedSubscription = rxjs_1.Subscription.EMPTY;
        return _this;
    }
    Loading.prototype.componentDidMount = function () {
        var _this = this;
        if (this.wx.isProperty(this.props.progress) === true) {
            this.changedSubscription = this.props.progress.changed.subscribe(function () {
                _this.forceUpdate();
            });
        }
    };
    Loading.prototype.componentWillUnmount = function () {
        this.changedSubscription = rxjs_1.Subscription.unsubscribe(this.changedSubscription);
    };
    Loading.prototype.render = function () {
        var _a = this.restProps(function (x) {
            var progress = x.progress, text = x.text, fontSize = x.fontSize, componentClass = x.componentClass;
            return { progress: progress, text: text, fontSize: fontSize, componentClass: componentClass };
        }), className = _a.className, children = _a.children, props = _a.props, rest = _a.rest;
        var Component = props.componentClass;
        return (React.createElement(Component, tslib_1.__assign({}, rest, { className: this.wxr.classNames('Loading', className) }),
            React.createElement(react_bootstrap_1.ProgressBar, { style: { fontSize: props.fontSize }, active: true, now: this.getProgressValue(), label: props.text }, children)));
    };
    Loading.prototype.getProgressValue = function () {
        return this.wx.isProperty(this.props.progress) === true
            ? this.props.progress.value
            : this.props.progress;
    };
    Loading.displayName = 'Loading';
    Loading.renderLoadable = renderLoadable;
    Loading.renderSizedLoadable = renderSizedLoadable;
    Loading.renderGridLoadable = renderGridLoadable;
    Loading.defaultProps = {
        progress: 100,
        text: 'Loading...',
        componentClass: 'div',
    };
    return Loading;
}(React.Component));
exports.Loading = Loading;
//# sourceMappingURL=Loading.js.map