Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React = require("react");
var react_bootstrap_1 = require("react-bootstrap");
var react_dom_1 = require("react-dom");
var react_fa_1 = require("react-fa");
var React_1 = require("../../React");
var CommandButton_1 = require("../CommandButton/CommandButton");
var ProfilePicture_1 = require("../ProfilePicture/ProfilePicture");
var SearchView_1 = require("../Search/SearchView");
var SearchViewModel_1 = require("../Search/SearchViewModel");
var Sidebar_1 = require("./Sidebar");
var PageHeaderView = /** @class */ (function (_super) {
    tslib_1.__extends(PageHeaderView, _super);
    function PageHeaderView() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.containerRef = React.createRef();
        _this.navBarRef = React.createRef();
        return _this;
    }
    PageHeaderView.prototype.updateOn = function (viewModel) {
        return [
            viewModel.sidebarMenus.changed,
            viewModel.navbarMenus.changed,
            viewModel.navbarActions.changed,
            viewModel.helpMenuItems.changed,
            viewModel.adminMenuItems.changed,
            viewModel.userMenuItems.changed,
            viewModel.isSidebarVisible.changed,
            viewModel.menuItemChanged.results,
        ];
    };
    PageHeaderView.prototype.isSidebarEnabled = function () {
        return String.isNullOrEmpty(this.props.branduri);
    };
    PageHeaderView.prototype.isActionDisabled = function (item) {
        var isDisabled = true;
        if (item.command == null && String.isNullOrEmpty(item.uri) === false) {
            isDisabled = false;
        }
        else if (item.command != null && item.command.canExecute === true) {
            isDisabled = false;
        }
        return isDisabled;
    };
    PageHeaderView.prototype.getOrderedActions = function (items) {
        return (items || []).sort(function (a, b) { return (a.order || 0) - (b.order || 0); });
    };
    PageHeaderView.prototype.getVisibleActions = function (items) {
        var _this = this;
        return this.getOrderedActions((items || []).filter(function (x) {
            return x.visibleWhenDisabled === true || _this.isActionDisabled(x) === false;
        }));
    };
    PageHeaderView.prototype.loaded = function () {
        var _this = this;
        _super.prototype.loaded.call(this);
        if (!this.props.responsive) {
            window.onresize = function () {
                _this.layoutNavBar();
            };
            this.layoutNavBar();
        }
    };
    PageHeaderView.prototype.render = function () {
        var _this = this;
        var _a = this.restProps(function (x) {
            var brand = x.brand, branduri = x.branduri, responsive = x.responsive;
            return { brand: brand, branduri: branduri, responsive: responsive };
        }), className = _a.className, props = _a.props, rest = _a.rest;
        return (React.createElement("div", tslib_1.__assign({ ref: this.containerRef }, rest, { className: this.wxr.classNames('PageHeader', className) }),
            React.createElement(react_bootstrap_1.Navbar, { ref: this.navBarRef, fixedTop: !props.responsive, fluid: true },
                React.createElement(react_bootstrap_1.Navbar.Header, null,
                    React.createElement(react_bootstrap_1.Navbar.Brand, null, this.renderBrandButton()),
                    React.createElement(react_bootstrap_1.Navbar.Toggle, null)),
                React.createElement(react_bootstrap_1.Navbar.Collapse, null,
                    React.createElement(react_bootstrap_1.Nav, { className: "PageHeader-routedMenus" }, this.renderRoutedMenus()),
                    React.createElement(react_bootstrap_1.Nav, { className: "PageHeader-siteMenus", pullRight: true },
                        this.renderHelpMenu(),
                        this.renderAdminMenu(),
                        this.renderUserMenu()),
                    this.renderSearch(),
                    this.renderRoutedActions())),
            this.wxr.renderConditional(this.isSidebarEnabled(), function () {
                return _this.renderSidebar();
            })));
    };
    PageHeaderView.prototype.layoutNavBar = function () {
        var container = this.containerRef.current;
        var navBar = this.navBarRef.current &&
            react_dom_1.findDOMNode(this.navBarRef.current);
        if (container && navBar) {
            var rect = container.getBoundingClientRect();
            container.style.paddingBottom = navBar.clientHeight + 1 + "px";
        }
    };
    PageHeaderView.prototype.renderBrandButton = function () {
        var isSidebarEnabled = this.isSidebarEnabled();
        var active = isSidebarEnabled && this.viewModel.isSidebarVisible.value;
        var command = isSidebarEnabled ? this.viewModel.toggleSideBar : undefined;
        return (React.createElement(CommandButton_1.CommandButton, { className: "PageHeader-brand", bsStyle: "link", active: active, href: this.props.branduri, command: command }, this.props.brand));
    };
    PageHeaderView.prototype.renderHeaderMenu = function (id, header, items, noCaret, className) {
        var _this = this;
        if (noCaret === void 0) { noCaret = false; }
        var visibleItems = this.getVisibleActions(items);
        return this.wxr.renderConditional(visibleItems.length > 0, function () { return (React.createElement(react_bootstrap_1.NavDropdown, { id: id, key: id, title: header, noCaret: noCaret, className: _this.wxr.classNames("PageHeader-" + id, className) }, visibleItems.map(function (x) { return (React.createElement(react_bootstrap_1.MenuItem, { key: x.id, disabled: _this.isActionDisabled(x), href: x.uri, onSelect: String.isNullOrEmpty(x.uri) === true
                ? _this.bindEventToCommand(function (vm) { return vm.menuItemSelected; }, function () { return x; })
                : null },
            _this.renderHeaderCommandActionIcon(x),
            x.header)); }))); });
    };
    PageHeaderView.prototype.renderHeaderCommandActionIcon = function (item, className, fixedWidth) {
        if (fixedWidth === void 0) { fixedWidth = true; }
        var stackProps = { className: className };
        var props = tslib_1.__assign({ fixedWidth: fixedWidth }, stackProps);
        if (Array.isArray(item.iconName)) {
            var names = item.iconName.filter(function (x) { return String.isNullOrEmpty(x) === false; });
            if (names.length === 0) {
                return null;
            }
            else if (names.length === 1) {
                return React.createElement(react_fa_1.Icon, tslib_1.__assign({}, props, { name: names[0] }));
            }
            else {
                return (React.createElement(react_fa_1.IconStack, tslib_1.__assign({}, stackProps),
                    React.createElement(react_fa_1.Icon, { name: names[0], stack: "2x", fixedWidth: fixedWidth }),
                    React.createElement(react_fa_1.Icon, { name: names[1], stack: "1x", fixedWidth: fixedWidth })));
            }
        }
        else {
            return String.isNullOrEmpty(item.iconName) ? null : (React.createElement(react_fa_1.Icon, tslib_1.__assign({}, props, { name: item.iconName })));
        }
    };
    PageHeaderView.prototype.renderRoutedMenus = function () {
        var _this = this;
        return this.wxr.renderConditional(this.viewModel.navbarMenus.value.length > 0, function () {
            return _this.getOrderedActions(_this.viewModel.navbarMenus.value)
                .map(function (x) {
                return _this.renderHeaderMenu(x.id, x.header, x.items);
            })
                .filterNull();
        });
    };
    PageHeaderView.prototype.renderHelpMenu = function () {
        return this.renderHeaderMenu('helpMenu', React.createElement(react_fa_1.Icon, { name: "question-circle", size: "2x" }), this.viewModel.helpMenuItems.value, true);
    };
    PageHeaderView.prototype.renderAdminMenu = function () {
        return this.renderHeaderMenu('adminMenu', React.createElement(react_fa_1.Icon, { name: "cog", size: "2x" }), this.viewModel.adminMenuItems.value, true, 'hover-spin');
    };
    PageHeaderView.prototype.renderUserMenu = function () {
        var profilePicture = (React.createElement(ProfilePicture_1.ProfilePicture, { src: this.viewModel.userImage, title: this.viewModel.userDisplayName, iconSize: "2x", size: 30 }));
        return this.renderHeaderMenu('userMenu', profilePicture, this.viewModel.userMenuItems.value, true);
    };
    PageHeaderView.prototype.renderSearch = function () {
        if (this.viewModel.search instanceof SearchViewModel_1.SearchViewModel) {
            return (React.createElement(react_bootstrap_1.Navbar.Form, { pullRight: true },
                React.createElement(SearchView_1.SearchView, { viewModel: this.viewModel.search })));
        }
        return null;
    };
    PageHeaderView.prototype.renderRoutedActions = function () {
        var _this = this;
        var visibleActions = this.getVisibleActions(this.viewModel.navbarActions.value);
        return (React.createElement(react_bootstrap_1.Navbar.Form, { className: "PageHeader-routedActions", pullRight: true }, this.wxr.renderConditional(visibleActions.length > 0, function () {
            return visibleActions.map(function (x) { return (React.createElement(CommandButton_1.CommandButton, { key: x.id, className: "PageHeader-actionButton", bsStyle: x.bsStyle, href: x.uri, command: x.command, commandParameter: x.commandParameter },
                _this.renderHeaderCommandActionIcon(x, 'PageHeader-actionHeaderIcon', false),
                React.createElement("span", { className: "PageHeader-actionHeaderText" }, x.header))); });
        })));
    };
    PageHeaderView.prototype.renderSidebar = function () {
        var _this = this;
        return (React.createElement(Sidebar_1.Sidebar, { isVisible: this.viewModel.isSidebarVisible.value, header: this.props.brand, onHide: this.bindEventToCommand(function (x) { return x.toggleSideBar; }, function () { return false; }) }, this.wxr.renderConditional(this.viewModel.isSidebarVisible, function () {
            return _this.getOrderedActions(_this.viewModel.sidebarMenus.value)
                .map(function (menu) {
                var visibleActions = _this.getVisibleActions(menu.items);
                return _this.wxr.renderConditional(visibleActions.length > 0, function () { return (React.createElement(react_bootstrap_1.Nav, { key: menu.id },
                    React.createElement(react_bootstrap_1.NavItem, { className: "PageHeader-sidebarSection", disabled: true }, menu.header),
                    visibleActions.map(function (x) { return (React.createElement(react_bootstrap_1.NavItem, { key: x.id, disabled: _this.isActionDisabled(x), href: x.uri, onClick: String.isNullOrEmpty(x.uri) === true
                            ? _this.bindEventToCommand(function (vm) { return vm.menuItemSelected; }, function () { return x; })
                            : _this.bindEventToCommand(function (vm) { return vm.toggleSideBar; }, function () { return false; }) },
                        _this.renderHeaderCommandActionIcon(x),
                        x.header)); }))); });
            })
                .filterNull();
        })));
    };
    PageHeaderView.displayName = 'PageHeaderView';
    PageHeaderView.defaultProps = {
        brand: 'webrx-react Rocks!!!',
    };
    return PageHeaderView;
}(React_1.BaseView));
exports.PageHeaderView = PageHeaderView;
//# sourceMappingURL=PageHeaderView.js.map