Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React = require("react");
var react_bootstrap_1 = require("react-bootstrap");
var react_bootstrap_2 = require("react-bootstrap");
var React_1 = require("../../React");
var PagerViewModel_1 = require("./PagerViewModel");
exports.StandardPagerComponentOrder = [
    'info',
    'controls',
    'limit',
];
var PagerView = /** @class */ (function (_super) {
    tslib_1.__extends(PagerView, _super);
    function PagerView() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.renderFunctions = {
            info: _this.renderInfo.bind(_this),
            controls: _this.renderControls.bind(_this),
            limit: _this.renderLimit.bind(_this),
        };
        return _this;
    }
    PagerView.prototype.updateOn = function (viewModel) {
        return [
            viewModel.itemCount.changed,
            viewModel.limit.changed,
            viewModel.offset.changed,
            viewModel.pageCount.changed,
            viewModel.selectedPage.changed,
        ];
    };
    PagerView.prototype.render = function () {
        var _this = this;
        var _a = this.restProps(function (x) {
            var info = x.info, limits = x.limits, dropdownProps = x.dropdownProps, order = x.order, emptyInfo = x.emptyInfo;
            return { info: info, limits: limits, dropdownProps: dropdownProps, order: order, emptyInfo: emptyInfo };
        }), className = _a.className, props = _a.props, rest = _a.rest;
        var pagerProps = Object.rest(rest, function (x) {
            var activePage = x.activePage, boundaryLinks = x.boundaryLinks, bsSize = x.bsSize, bsStyle = x.bsStyle, buttonComponentClass = x.buttonComponentClass, ellipsis = x.ellipsis, first = x.first, items = x.items, last = x.last, maxButtons = x.maxButtons, next = x.next, onSelect = x.onSelect, prev = x.prev;
            return {
                activePage: activePage,
                boundaryLinks: boundaryLinks,
                bsSize: bsSize,
                bsStyle: bsStyle,
                buttonComponentClass: buttonComponentClass,
                ellipsis: ellipsis,
                first: first,
                items: items,
                last: last,
                maxButtons: maxButtons,
                next: next,
                onSelect: onSelect,
                prev: prev,
            };
        });
        return this.wxr.renderConditional(this.shouldRenderPager(), function () { return (React.createElement("div", tslib_1.__assign({}, pagerProps.rest, { className: _this.wxr.classNames('Pager', className) }),
            React.createElement(react_bootstrap_1.Grid, { fluid: true }, _this.renderComponents()))); });
    };
    PagerView.prototype.isEmpty = function () {
        return (this.viewModel.itemCount.value || 0) === 0;
    };
    PagerView.prototype.renderComponents = function () {
        var _this = this;
        if (this.shouldRenderEmpty()) {
            return this.renderEmpty();
        }
        var types = this.props.order || [];
        var components = types.map(function (x) { return _this.renderComponent(x); });
        if (components.length === 1) {
            return (React.createElement(react_bootstrap_1.Row, null,
                React.createElement(react_bootstrap_1.Col, { className: "Pager-col Pager-col-center" }, components[0])));
        }
        if (components.length === 2) {
            return (React.createElement(react_bootstrap_1.Row, null,
                React.createElement(react_bootstrap_1.Col, { className: "Pager-col Pager-col-left", md: 6 }, components[0]),
                React.createElement(react_bootstrap_1.Col, { className: "Pager-col Pager-col-right", md: 6 }, components[1])));
        }
        return (React.createElement(react_bootstrap_1.Row, null,
            React.createElement(react_bootstrap_1.Col, { className: "Pager-col Pager-col-left", md: 4 }, components[0]),
            React.createElement(react_bootstrap_1.Col, { className: "Pager-col Pager-col-center", md: 4 }, components[1]),
            React.createElement(react_bootstrap_1.Col, { className: "Pager-col Pager-col-right", md: 4 }, components[2])));
    };
    PagerView.prototype.shouldRenderPager = function () {
        return ((this.props.order || []).length > 0 &&
            (this.shouldRenderEmpty() ||
                this.shouldRenderInfo() ||
                this.shouldRenderControls() ||
                this.shouldRenderLimit()));
    };
    PagerView.prototype.shouldRenderEmpty = function () {
        return (this.isEmpty() && String.isNullOrEmpty(this.props.emptyInfo) === false);
    };
    PagerView.prototype.shouldRenderInfo = function () {
        return this.props.info === true && (this.viewModel.limit.value || 0) > 0;
    };
    PagerView.prototype.shouldRenderControls = function () {
        return (this.viewModel.pageCount.value || 1) > 1;
    };
    PagerView.prototype.shouldRenderLimit = function () {
        return ((this.props.limits || []).length > 1 &&
            (this.viewModel.itemCount.value || 0) > 0);
    };
    PagerView.prototype.renderComponent = function (type) {
        return type && this.renderFunctions[type]();
    };
    PagerView.prototype.renderEmpty = function () {
        return (React.createElement(react_bootstrap_1.Row, null,
            React.createElement(react_bootstrap_1.Col, { className: "Pager-col" },
                React.createElement("div", { className: "Pager-empty" }, this.props.emptyInfo))));
    };
    PagerView.prototype.renderInfo = function () {
        var _this = this;
        return this.wxr.renderConditional(this.shouldRenderInfo(), function () {
            var start = _this.viewModel.offset.value + 1;
            var end = Math.min(_this.viewModel.itemCount.value, _this.viewModel.offset.value + (_this.viewModel.limit.value || 0));
            var total = _this.viewModel.itemCount.value;
            return (React.createElement("div", { className: "Pager-info" }, "Showing Items " + start + " through " + end + " of " + total));
        }, function () { return ''; });
    };
    PagerView.prototype.renderControls = function () {
        var _this = this;
        var props = this.restProps(function (x) {
            var activePage = x.activePage, bsSize = x.bsSize, bsStyle = x.bsStyle, buttonComponentClass = x.buttonComponentClass, ellipsis = x.ellipsis, first = x.first, items = x.items, last = x.last, maxButtons = x.maxButtons, next = x.next, prev = x.prev;
            return {
                activePage: activePage,
                bsSize: bsSize,
                bsStyle: bsStyle,
                buttonComponentClass: buttonComponentClass,
                ellipsis: ellipsis,
                first: first,
                items: items,
                last: last,
                maxButtons: maxButtons,
                next: next,
                prev: prev,
            };
        }).props;
        return this.wxr.renderConditional(this.shouldRenderControls(), function () { return (React.createElement(react_bootstrap_2.Pagination, tslib_1.__assign({ items: _this.viewModel.pageCount.value, activePage: _this.viewModel.selectedPage.value, onSelect: _this.bindEventToCommand(function (x) { return x.selectPage; }) }, Object.assign(props, {
            items: _this.viewModel.pageCount.value,
            activePage: _this.viewModel.selectedPage.value,
        })))); }, function () { return ''; });
    };
    PagerView.prototype.renderLimit = function () {
        var _this = this;
        return this.wxr.renderConditional(this.shouldRenderLimit(), function () { return (React.createElement(react_bootstrap_2.DropdownButton, tslib_1.__assign({ id: _this.props.id || 'pager-limit-selector' }, _this.props.dropdownProps, { title: _this.renderLimitTitle(), onSelect: _this.bindEventToProperty(function (x) { return x.limit; }) }), _this.props.limits.map(function (x, i) { return (React.createElement(react_bootstrap_2.MenuItem, { key: i, eventKey: x, selected: _this.viewModel.limit.value === x }, x || 'All')); }))); }, function () { return ''; });
    };
    PagerView.prototype.renderLimitTitle = function () {
        return "Items per Page (" + (this.viewModel.limit.value || 'All') + ")";
    };
    PagerView.displayName = 'PagerView';
    PagerView.defaultProps = {
        first: true,
        prev: true,
        next: true,
        last: true,
        boundaryLinks: true,
        maxButtons: 3,
        info: true,
        limits: PagerViewModel_1.StandardLimits,
        order: exports.StandardPagerComponentOrder,
        emptyInfo: 'No Items to Display',
    };
    return PagerView;
}(React_1.BaseView));
exports.PagerView = PagerView;
//# sourceMappingURL=PagerView.js.map