Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React_1 = require("../../React");
exports.StandardLimits = [10, 25, 0];
exports.AlwaysPagedLimits = exports.StandardLimits.filter(function (x) { return x != null && x > 0; });
var PagerViewModel = /** @class */ (function (_super) {
    tslib_1.__extends(PagerViewModel, _super);
    function PagerViewModel(defaultLimit) {
        if (defaultLimit === void 0) { defaultLimit = exports.StandardLimits[0]; }
        var _this = _super.call(this) || this;
        _this.defaultLimit = defaultLimit;
        _this.updateCount = _this.wx.command();
        _this.selectPage = _this.wx.command();
        _this.itemCount = _this.wx
            .whenAny(_this.updateCount, function (x) { return x; })
            .filterNull()
            .toProperty(0);
        _this.limit = _this.wx.property(defaultLimit);
        _this.pageCount = _this.wx
            .whenAny(_this.itemCount, _this.limit, function (ic, l) { return ({ ic: ic, l: l }); })
            // if we have a valid item count and limit, then calculate the page count (default to zero)
            .map(function (x) {
            return x.ic != null && x.l != null && x.ic > 0 && x.l > 0
                ? Math.ceil(x.ic / x.l)
                : 0;
        })
            .toProperty(0);
        _this.selectedPage = _this.wx
            .whenAny(_this.selectPage.results, function (x) { return x; })
            .filterNull()
            .toProperty(1);
        _this.offset = _this.wx
            .whenAny(_this.selectedPage, _this.limit, function (sp, l) { return ({ sp: sp, l: l }); })
            // if we have a valid page then calculate the offset (default limit to zero to result in a zero offset)
            .map(function (x) { return (x.sp != null && x.sp > 0 ? (x.sp - 1) * (x.l || 0) : 0); })
            .toProperty(0);
        _this.requests = _this.wx
            .whenAny(_this.offset, _this.limit, function (offset, limit) { return ({
            offset: offset,
            limit: limit,
        }); })
            .toProperty();
        _this.addSubscription(_this.wx
            .whenAny(_this.pageCount, function (x) { return x; })
            .filterNull()
            .map(function () { return 1; })
            .invokeCommand(_this.selectPage));
        _this.addSubscription(_this.wx
            .whenAny(_this.selectedPage, _this.limit, function (sp, l) { return ({ sp: sp, l: l }); })
            .filter(function (x) { return x.sp != null && x.l != null; })
            .subscribe(function (x) {
            _this.notifyChanged(x);
        }));
        return _this;
    }
    PagerViewModel.prototype.isRoutingStateHandler = function () {
        return true;
    };
    PagerViewModel.prototype.createRoutingState = function (changed) {
        return Object.trim({
            limit: this.getRoutingStateValue(this.limit.value, this.defaultLimit),
            page: this.getRoutingStateValue(this.selectedPage.value, 1),
        });
    };
    PagerViewModel.prototype.applyRoutingState = function (state) {
        this.limit.value = state.limit == null ? this.defaultLimit : state.limit;
        if (this.selectedPage.value !== state.page) {
            // 0 is not a valid page so we can just use || fallback notation
            this.selectPage.execute(state.page || 1);
        }
    };
    PagerViewModel.displayName = 'PagerViewModel';
    return PagerViewModel;
}(React_1.BaseViewModel));
exports.PagerViewModel = PagerViewModel;
//# sourceMappingURL=PagerViewModel.js.map