Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var ix_1 = require("ix");
var React = require("react");
var Layout = require("./GridLayout");
var Panel_1 = require("./Panel");
var GridLayout_1 = require("./GridLayout");
exports.RowDefinition = GridLayout_1.RowDefinition;
exports.ColumnDefinition = GridLayout_1.ColumnDefinition;
var Grid = /** @class */ (function (_super) {
    tslib_1.__extends(Grid, _super);
    function Grid() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Grid.prototype.render = function () {
        var _a = this.props, border = _a.border, rest = tslib_1.__rest(_a, ["border"]);
        var bordered = { 'Grid-Border': border === true };
        var compact = { compact: this.props.compact };
        return this.renderPanel(this.wxr.classNames('Grid', bordered, compact), rest);
    };
    Grid.prototype.renderItems = function (allChildren, componentClass) {
        var _this = this;
        var _a = this.getLayout(), children = _a.children, rows = _a.rows, cols = _a.cols;
        return ix_1.Iterable.range(0, rows.length)
            .map(function (row) {
            var colItems = ix_1.Iterable.range(0, cols.length)
                .map(function (col) {
                return _this.renderColumn(row, col, rows, cols, children);
            })
                .toArray();
            return _this.renderRow(row, rows, colItems);
        })
            .defaultIfEmpty(this.renderEmpty())
            .toArray();
    };
    Grid.prototype.renderRow = function (row, rows, colItems) {
        var def = rows[row];
        var t = def.itemTemplate;
        var context = { row: row, index: row };
        var itemClassName = Panel_1.Panel.getPanelItemPropValue(def.itemClassName, context);
        var itemStyle = Panel_1.Panel.getPanelItemPropValue(def.itemStyle, context);
        var itemProps = Panel_1.Panel.getPanelItemPropValue(def.itemProps, context) || {};
        var compact = { compact: this.props.compact };
        var itemTemplate = def.itemTemplate;
        var layoutStyle = Object.assign({}, itemStyle, {
            height: this.getCellLayoutValue(def),
        });
        var fragment = (React.createElement("div", tslib_1.__assign({ className: this.wxr.classNames('Grid-Row', compact, itemClassName), style: layoutStyle, "data-grid-row": row, key: Layout.GridLayoutDefinition.generateKey(row) }, itemProps), colItems));
        if (itemTemplate == null) {
            return fragment;
        }
        return itemTemplate(fragment, context);
    };
    Grid.prototype.renderColumn = function (row, column, rows, cols, children) {
        var def = cols[column];
        var cellItems = [];
        var index = 0;
        while (index < children.length) {
            var child = children[index];
            var childProps = child != null && child.props != null ? child.props : undefined;
            var desiredRow = (childProps == null ? undefined : childProps['data-grid-row']) || 0;
            var desiredCol = (childProps == null ? undefined : childProps['data-grid-column']) || 0;
            if (desiredRow === row && desiredCol === column) {
                var cellItem = React.isValidElement(child)
                    ? React.cloneElement(child, {
                        'data-grid-row': undefined,
                        'data-grid-column': undefined,
                    })
                    : child;
                cellItems.push(cellItem);
                children.splice(index, 1);
            }
            else {
                ++index;
            }
        }
        var context = { row: row, column: column, index: index };
        var itemClassName = Panel_1.Panel.getPanelItemPropValue(def.itemClassName, context);
        var itemStyle = Panel_1.Panel.getPanelItemPropValue(def.itemStyle, context);
        var itemProps = Panel_1.Panel.getPanelItemPropValue(def.itemProps, context) || {};
        var compact = { compact: this.props.compact };
        var itemTemplate = def.itemTemplate;
        var layoutStyle = Object.assign({}, itemStyle, {
            width: this.getCellLayoutValue(def),
        });
        var fragment = (React.createElement("div", tslib_1.__assign({ className: this.wxr.classNames('Grid-Column', compact, itemClassName), style: layoutStyle, "data-grid-column": column, key: Layout.GridLayoutDefinition.generateKey(row, column) }, itemProps), _super.prototype.renderItems.call(this, cellItems)));
        if (itemTemplate == null) {
            return fragment;
        }
        return itemTemplate(fragment, context);
    };
    Grid.prototype.getCellLayoutValue = function (def) {
        return def.stretch ? (def.amount || 1) * 100 + "%" : def.amount;
    };
    Grid.prototype.getLayout = function () {
        var children = React.Children.toArray(this.props.children);
        var rows;
        var cols;
        var index = 0;
        while (index < children.length) {
            var elem = children[index];
            if (React.isType(elem, Layout.GridRowDefinitions)) {
                rows = elem;
                children.splice(index, 1);
            }
            else if (React.isType(elem, Layout.GridColumnDefinitions)) {
                cols = elem;
                children.splice(index, 1);
            }
            else {
                ++index;
            }
            if (rows != null && cols != null) {
                break;
            }
        }
        return {
            children: children,
            rows: this.getLayoutDefinitions(rows),
            cols: this.getLayoutDefinitions(cols),
        };
    };
    Grid.prototype.getLayoutDefinitions = function (elem) {
        if (elem == null) {
            return [new Layout.GridLayoutDefinition()];
        }
        return React.Children.map(elem.props.children, function (x) {
            return new Layout.GridLayoutDefinition(x, elem);
        });
    };
    Grid.displayName = 'Grid';
    /**
     * Use this component to define your Row collection
     * i.e., <Grid.Rows><RowDefinition /></Grid.Rows>
     */
    Grid.Rows = Layout.GridRowDefinitions;
    /**
     * Use this component to define your Column collection
     * i.e., <Grid.Columns><ColumnDefinition /></Grid.Columns>
     */
    Grid.Columns = Layout.GridColumnDefinitions;
    return Grid;
}(Panel_1.Panel));
exports.Grid = Grid;
//# sourceMappingURL=Grid.js.map