// tslint:disable:max-classes-per-file
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React = require("react");
/**
 * a component to define a grid row, optionally with a static height value
 * if height is omitted, height will auto stretch to consume available space
 */
var RowDefinition = /** @class */ (function (_super) {
    tslib_1.__extends(RowDefinition, _super);
    function RowDefinition() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return RowDefinition;
}(React.Component));
exports.RowDefinition = RowDefinition;
/**
 * a component to define the row layout component collection
 */
var GridRowDefinitions = /** @class */ (function (_super) {
    tslib_1.__extends(GridRowDefinitions, _super);
    function GridRowDefinitions() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return GridRowDefinitions;
}(React.Component));
exports.GridRowDefinitions = GridRowDefinitions;
/**
 * a component to define a grid column, optionally with a width value
 * if width is omitted, width will auto stretch to consume '1*' of space
 */
var ColumnDefinition = /** @class */ (function (_super) {
    tslib_1.__extends(ColumnDefinition, _super);
    function ColumnDefinition() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ColumnDefinition.defaultProps = {
        width: '*',
    };
    return ColumnDefinition;
}(React.Component));
exports.ColumnDefinition = ColumnDefinition;
/**
 * a component to define the column layout component collection
 */
var GridColumnDefinitions = /** @class */ (function (_super) {
    tslib_1.__extends(GridColumnDefinitions, _super);
    function GridColumnDefinitions() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return GridColumnDefinitions;
}(React.Component));
exports.GridColumnDefinitions = GridColumnDefinitions;
/**
 * this class is used internally to compute the grid layout metadata
 */
var GridLayoutDefinition = /** @class */ (function () {
    function GridLayoutDefinition(definition, definitionGroup) {
        var _a = this.getLayoutParam(definition), val = _a.val, type = _a.type;
        var _b = this.getAmountAndStretch(val), amount = _b.amount, stretch = _b.stretch;
        if (type === RowDefinition && stretch === true) {
            amount = undefined;
            stretch = false;
        }
        else if (type === ColumnDefinition &&
            amount == null &&
            stretch === true) {
            amount = 1;
        }
        this.amount = amount;
        this.stretch = stretch;
        if (definitionGroup != null) {
            this.itemClassName = definitionGroup.props.itemClassName;
            this.itemStyle = definitionGroup.props.itemStyle;
            this.itemProps = definitionGroup.props.itemProps;
            this.compact = definitionGroup.props.compact;
            this.itemTemplate = definitionGroup.props.itemTemplate;
        }
        if (definition != null) {
            if (definition.props.itemClassName != null) {
                this.itemClassName = definition.props.itemClassName;
            }
            if (definition.props.itemStyle != null) {
                this.itemStyle = definition.props.itemStyle;
            }
            if (definition.props.itemProps != null) {
                this.itemProps = definition.props.itemProps;
            }
            if (definition.props.compact != null) {
                this.compact = definition.props.compact;
            }
            if (definition.props.itemTemplate != null) {
                this.itemTemplate = definition.props.itemTemplate;
            }
        }
    }
    GridLayoutDefinition.generateKey = function (row, column) {
        return row + "." + (column || '');
    };
    GridLayoutDefinition.prototype.getAmountAndStretch = function (val) {
        if (String.isString(val)) {
            if (val === '*') {
                return { amount: 1, stretch: true };
            }
            else if (val.length > 1) {
                if (val[val.length - 1] === '*') {
                    return { amount: parseInt(val), stretch: true };
                }
                else {
                    return { amount: parseInt(val), stretch: false };
                }
            }
            else {
                return { amount: parseInt(val), stretch: false };
            }
        }
        else if (val != null) {
            return { amount: val, stretch: false };
        }
        else {
            return { amount: undefined, stretch: true };
        }
    };
    GridLayoutDefinition.prototype.getLayoutParam = function (definition) {
        if (definition != null &&
            React.isValidElement(definition)) {
            return {
                val: definition.props.height || definition.props.width,
                type: definition.type,
            };
        }
        else {
            return { val: undefined, type: undefined };
        }
    };
    return GridLayoutDefinition;
}());
exports.GridLayoutDefinition = GridLayoutDefinition;
//# sourceMappingURL=GridLayout.js.map