/// <reference types="react" />
import * as React from 'react';
/**
 * A panel item context is metadata passed to a panel item to give some
 * context as to where in the container it is being rendered.
 */
export interface PanelItemContext {
    index: number;
}
export declare type PanelFragment = React.ReactNode;
/**
 * panel item prop can be statically assigned or dynamically determined
 * based on a provided item context
 */
export declare type PanelItemProp<TValue, TContext extends PanelItemContext = PanelItemContext> = TValue | ((context: TContext) => TValue);
/**
 * panel items can be wrapped in a different component
 * this allows composing new items with an existing panel
 */
export declare type PanelItemTemplate<TContext extends PanelItemContext = PanelItemContext> = (fragment: PanelFragment, context: TContext) => PanelFragment;
/**
 * panel item props allow component to inject props to the rendered
 * block structure.
 */
export interface PanelItemProps<T = {}, TContext extends PanelItemContext = PanelItemContext> {
    /**
     * apply custom class name to the corresponding panel item
     */
    itemClassName?: PanelItemProp<string, TContext>;
    /**
     * apply custom style to the corresponding panel item
     */
    itemStyle?: PanelItemProp<React.CSSProperties, TContext>;
    /**
     * apply custom props to the corresponding panel item
     */
    itemProps?: PanelItemProp<T, TContext>;
}
export interface PanelTemplateProps<T extends PanelItemContext = PanelItemContext> {
    /**
     * apply a custom template to the renderd panel item
     */
    itemTemplate?: PanelItemTemplate<T>;
}
export interface PanelRenderProps {
    compact?: boolean;
    /**
     * apply a custom content to an empty panel with no items to render
     */
    emptyContent?: PanelFragment;
}
export interface PanelProps<T = {}, TContext extends PanelItemContext = PanelItemContext> extends PanelItemProps<T, TContext>, PanelTemplateProps<TContext>, PanelRenderProps {
    fill?: boolean;
}
export declare abstract class Panel<TProps extends PanelProps> extends React.Component<TProps> {
    static displayName: string;
    static defaultComponentClass: string;
    static getPanelItemPropValue<TValue, TContext extends PanelItemContext>(prop: (PanelItemProp<TValue, TContext>) | undefined, context: TContext): TValue | undefined;
    protected renderPanel(panelClassName?: string, panelProps?: PanelProps, componentClass?: React.ReactType): JSX.Element;
    protected renderItems(children?: React.ReactNode, componentClass?: React.ReactType): React.ReactNode[];
    protected renderEmpty(): false | JSX.Element;
    protected renderItem(itemTemplate: PanelFragment, index: number, componentClass?: React.ReactType): PanelFragment;
    protected renderItemFragment(itemTemplate: PanelFragment, context: PanelItemContext, Component: React.ReactType): PanelFragment;
    protected getItemKey(itemTemplate: PanelFragment, index: number): any;
}
