Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React = require("react");
var Panel = /** @class */ (function (_super) {
    tslib_1.__extends(Panel, _super);
    function Panel() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Panel.getPanelItemPropValue = function (prop, context) {
        if (prop instanceof Function) {
            return prop(context);
        }
        return prop;
    };
    Panel.prototype.renderPanel = function (panelClassName, panelProps, componentClass) {
        var _a = React.Component.restProps(panelProps || this.props, function (x) {
            var itemClassName = x.itemClassName, itemStyle = x.itemStyle, itemProps = x.itemProps, itemTemplate = x.itemTemplate, compact = x.compact, emptyContent = x.emptyContent;
            return {
                itemClassName: itemClassName,
                itemStyle: itemStyle,
                itemProps: itemProps,
                itemTemplate: itemTemplate,
                compact: compact,
                emptyContent: emptyContent,
            };
        }), className = _a.className, children = _a.children, props = _a.props, rest = _a.rest;
        var Component = componentClass || Panel.defaultComponentClass;
        var componentClassName = this.wxr.classNames('Panel', { compact: props.compact }, panelClassName, className);
        return (React.createElement(Component, tslib_1.__assign({}, rest, { className: componentClassName }), this.renderItems(children)));
    };
    Panel.prototype.renderItems = function (children, componentClass) {
        var _this = this;
        var props = this.props;
        return React.Children.map(children || this.props.children, function (x, i) {
            return _this.renderItem(x, i, componentClass);
        })
            .asIterable()
            .filter(function (x) { return (x ? true : false); })
            .defaultIfEmpty(this.renderEmpty())
            .toArray();
    };
    Panel.prototype.renderEmpty = function () {
        if (this.props.emptyContent) {
            return (React.createElement("div", { key: "empty", className: "Panel-empty" }, this.props.emptyContent));
        }
        return false;
    };
    Panel.prototype.renderItem = function (itemTemplate, index, componentClass) {
        var context = { index: index };
        var fragment = this.renderItemFragment(itemTemplate, context, componentClass == null ? Panel.defaultComponentClass : componentClass);
        if (this.props.itemTemplate == null) {
            return fragment;
        }
        return this.props.itemTemplate(fragment, context);
    };
    Panel.prototype.renderItemFragment = function (itemTemplate, context, Component) {
        if (itemTemplate) {
            var key = this.getItemKey(itemTemplate, context.index);
            var className = this.wxr.classNames('Panel-Item', Panel.getPanelItemPropValue(this.props.itemClassName, context));
            var style = Panel.getPanelItemPropValue(this.props.itemStyle, context);
            var props = Panel.getPanelItemPropValue(this.props.itemProps, context) || {};
            return Component === '' && React.isValidElement(itemTemplate) ? (React.cloneElement(itemTemplate, tslib_1.__assign({ key: key, className: className, style: style }, props))) : (React.createElement(Component, tslib_1.__assign({ key: key, className: className, style: style }, props), itemTemplate));
        }
        return null;
    };
    Panel.prototype.getItemKey = function (itemTemplate, index) {
        var key = index;
        if (itemTemplate != null &&
            React.isValidElement(itemTemplate) &&
            itemTemplate.key != null) {
            key = itemTemplate.key;
        }
        return key;
    };
    Panel.displayName = 'Panel';
    Panel.defaultComponentClass = 'div';
    return Panel;
}(React.Component));
exports.Panel = Panel;
//# sourceMappingURL=Panel.js.map