/// <reference types="react" />
import * as React from 'react';
import { GridRenderProps } from './Grid';
import { GridColumnContext, GridRowContext } from './GridLayout';
import { Panel, PanelFragment, PanelItemContext, PanelItemProp, PanelProps } from './Panel';
export interface UniformRowItemProps<T = {}> {
    /**
     * apply custom row class name to the corresponding panel item
     */
    rowClassName?: PanelItemProp<string, GridRowContext>;
    /**
     * apply custom row style to the corresponding panel item
     */
    rowStyle?: PanelItemProp<React.CSSProperties, GridRowContext>;
    /**
     * apply custom row props to the corresponding panel item
     */
    rowProps?: PanelItemProp<T, GridRowContext>;
}
export interface UniformColumnItemProps<T = {}> {
    /**
     * apply custom column class name to the corresponding panel item
     */
    columnClassName?: PanelItemProp<string, GridColumnContext>;
    /**
     * apply custom column style to the corresponding panel item
     */
    columnStyle?: PanelItemProp<React.CSSProperties, GridColumnContext>;
    /**
     * apply custom column props to the corresponding panel item
     */
    columnProps?: PanelItemProp<T, GridColumnContext>;
}
export interface UniformGridPanelProps<T = {}, TContext extends PanelItemContext = PanelItemContext> extends PanelProps<T, TContext>, UniformRowItemProps<T>, UniformColumnItemProps<T>, GridRenderProps {
    /**
     * number of columns in the grid
     */
    gridColumns: number;
    /**
     * number of rows in the grid
     */
    gridRows: number;
    /**
     * which column index to start rendering panel items within (in the first row)
     */
    firstColumn?: number;
    /**
     * True to render rows which have no corresponding panel items to render
     */
    renderEmptyRows?: boolean;
    /**
     * template to render an empty panel item cell
     * default template renders an &nbsp; block
     */
    emptyCellTemplate?: (row: number, column: number) => PanelFragment;
}
export interface UniformGridPanelComponentProps extends React.HTMLProps<any>, UniformGridPanelProps {
}
export declare class UniformGridPanel extends Panel<UniformGridPanelComponentProps> {
    static displayName: string;
    static defaultProps: Partial<UniformGridPanelProps>;
    static defaultEmptyTemplate(row: number, column: number): PanelFragment;
    render(): JSX.Element;
    renderItems(children?: React.ReactNode, componentClass?: React.ReactType): JSX.Element[];
    protected renderEmptyCell(row: number, column: number, index: number): React.ReactNode;
}
