Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React = require("react");
var react_bootstrap_1 = require("react-bootstrap");
var react_fa_1 = require("react-fa");
var dataUriPrefix = 'data:image;base64,';
var ProfilePicture = /** @class */ (function (_super) {
    tslib_1.__extends(ProfilePicture, _super);
    function ProfilePicture() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ProfilePicture.prototype.getImageSource = function () {
        var src = this.props.src || this.props.defaultSrc;
        if (String.isNullOrEmpty(src)) {
            return src;
        }
        // check if the source uri is already formatted
        if (src.indexOf(dataUriPrefix) >= 0 || /^https?:/.test(src)) {
            return src;
        }
        // otherwise assume we need to format a data uri
        return "" + dataUriPrefix + src;
    };
    ProfilePicture.prototype.render = function () {
        var imageSource = this.getImageSource();
        return (imageSource && this.renderImage(imageSource)) || this.renderIcon();
    };
    ProfilePicture.prototype.renderIcon = function () {
        var _a = this.restProps(function (x) {
            var src = x.src, size = x.size, responsive = x.responsive, rounded = x.rounded, circle = x.circle, thumbnail = x.thumbnail, defaultSrc = x.defaultSrc, defaultIcon = x.defaultIcon, iconSize = x.iconSize, style = x.style;
            return {
                src: src,
                size: size,
                responsive: responsive,
                rounded: rounded,
                circle: circle,
                thumbnail: thumbnail,
                defaultSrc: defaultSrc,
                defaultIcon: defaultIcon,
                iconSize: iconSize,
                style: style,
            };
        }), className = _a.className, props = _a.props, rest = _a.rest;
        var iconClassNames = {
            'ProfilePicture-iconThumbnail': props.thumbnail,
            'ProfilePicture-iconRounded': props.rounded,
        };
        var iconStyle = Object.assign({}, props.style, {
            width: props.size,
            height: props.size,
            lineHeight: props.size + "px",
        });
        return (React.createElement("div", tslib_1.__assign({}, rest, { style: iconStyle, className: this.wxr.classNames('ProfilePicture', 'ProfilePicture-icon', iconClassNames, className) }),
            React.createElement(react_fa_1.Icon, { name: props.defaultIcon, size: props.iconSize })));
    };
    ProfilePicture.prototype.renderImage = function (imageSource) {
        var _a = this.restProps(function (x) {
            var src = x.src, size = x.size, responsive = x.responsive, rounded = x.rounded, circle = x.circle, thumbnail = x.thumbnail, defaultSrc = x.defaultSrc, defaultIcon = x.defaultIcon, iconSize = x.iconSize, style = x.style;
            return {
                src: src,
                size: size,
                responsive: responsive,
                rounded: rounded,
                circle: circle,
                thumbnail: thumbnail,
                defaultSrc: defaultSrc,
                defaultIcon: defaultIcon,
                iconSize: iconSize,
                style: style,
            };
        }), className = _a.className, props = _a.props, rest = _a.rest;
        var responsiveStyle = Object.assign({}, props.style, {
            width: this.props.responsive === true ? 'auto' : props.size,
            height: this.props.responsive === true ? '100%' : props.size,
            lineHeight: this.props.responsive === true ? null : props.size + "px",
        });
        var imageProps = {
            src: imageSource,
            responsive: props.responsive,
            rounded: props.rounded,
            circle: props.circle,
            thumbnail: props.thumbnail,
        };
        var imageStyle = this.props.responsive
            ? undefined
            : { maxHeight: props.size, maxWidth: props.size };
        return (React.createElement("div", tslib_1.__assign({}, rest, { className: this.wxr.classNames('ProfilePicture', 'ProfilePicture-image', className), style: responsiveStyle }),
            React.createElement(react_bootstrap_1.Image, tslib_1.__assign({}, imageProps, { style: imageStyle }))));
    };
    ProfilePicture.displayName = 'ProfilePicture';
    ProfilePicture.defaultProps = {
        size: 65,
        responsive: false,
        rounded: false,
        circle: false,
        thumbnail: false,
        defaultIcon: 'user',
        iconSize: '4x',
    };
    return ProfilePicture;
}(React.Component));
exports.ProfilePicture = ProfilePicture;
//# sourceMappingURL=ProfilePicture.js.map