Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React = require("react");
var react_bootstrap_1 = require("react-bootstrap");
var RTG = require("react-transition-group");
var React_1 = require("../../React");
var Breadcrumbs_1 = require("./Breadcrumbs");
var RouteHandlerViewModel_1 = require("./RouteHandlerViewModel");
var RouteHandlerView = /** @class */ (function (_super) {
    tslib_1.__extends(RouteHandlerView, _super);
    function RouteHandlerView(props) {
        var _this = _super.call(this, props) || this;
        if (_this.props.viewMap[RouteHandlerViewModel_1.DefaultKey] == null) {
            _this.props.viewMap[RouteHandlerViewModel_1.DefaultKey] = function () { return _this.renderError('View Not Found'); };
        }
        if (_this.props.viewMap[''] == null) {
            _this.props.viewMap[''] = function () { return _this.renderError('Route Not Found'); };
        }
        if (_this.props.viewMap[RouteHandlerViewModel_1.SplashKey] == null) {
            _this.props.viewMap[RouteHandlerViewModel_1.SplashKey] = null;
        }
        return _this;
    }
    RouteHandlerView.prototype.getViewKey = function () {
        if (this.viewModel.isLoading.value === true) {
            return RouteHandlerViewModel_1.SplashKey;
        }
        else {
            var component = this.viewModel.routedComponent.value;
            if (React_1.isRoutableViewModel(component)) {
                return component.getRoutingKey();
            }
            else if (React_1.isViewModel(component)) {
                this.logger.warn('Routing to Non-Routable View Model', component);
                return component.getDisplayName();
            }
            else if (String.isString(component)) {
                return component;
            }
            else {
                return '';
            }
        }
    };
    RouteHandlerView.prototype.updateOn = function (viewModel) {
        return [
            viewModel.isLoading.changed,
            viewModel.routedComponent.changed,
            viewModel.routingBreadcrumbs.changed,
        ];
    };
    RouteHandlerView.prototype.render = function () {
        var _a = this.restProps(function (x) {
            var viewMap = x.viewMap, responsive = x.responsive;
            return { viewMap: viewMap, responsive: responsive };
        }), className = _a.className, rest = _a.rest;
        var key = this.getViewKey();
        return (React.createElement("div", tslib_1.__assign({}, rest, { className: this.wxr.classNames('RouteHandler', className) }),
            React.createElement(RTG.TransitionGroup, null,
                React.createElement(RTG.CSSTransition, { key: key, classNames: "view", exit: false, timeout: { enter: 250 } },
                    React.createElement("div", { className: "RouteHandler-viewContainer" },
                        React.createElement(Breadcrumbs_1.Breadcrumbs, { items: this.viewModel.routingBreadcrumbs.value, pinnable: true }),
                        this.renderRoutedView(key))))));
    };
    RouteHandlerView.prototype.renderRoutedView = function (key) {
        var component = this.viewModel.routedComponent.value;
        var activator = this.props.viewMap[key];
        if (activator == null && key !== RouteHandlerViewModel_1.SplashKey) {
            activator = this.props.viewMap[RouteHandlerViewModel_1.DefaultKey];
        }
        var view = activator;
        if (activator instanceof Function) {
            view = activator(component, this.props.responsive);
        }
        this.logger.debug("Rendering routed view for '" + Object.getName(component) + "' (" + key + ")");
        return (React.createElement("div", { className: "RouteHandler-view" }, view || this.renderError('Catastrophic Failure')));
    };
    RouteHandlerView.prototype.renderError = function (text) {
        return (React.createElement(react_bootstrap_1.Grid, { className: "RouteHandler-error" },
            React.createElement(react_bootstrap_1.Alert, { bsStyle: "danger" },
                React.createElement("h4", null, text))));
    };
    RouteHandlerView.displayName = 'RouteHandlerView';
    return RouteHandlerView;
}(React_1.BaseView));
exports.RouteHandlerView = RouteHandlerView;
//# sourceMappingURL=RouteHandlerView.js.map