Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
require("ix");
var rxjs_1 = require("rxjs");
var RoutingStateChanged_1 = require("../../../Events/RoutingStateChanged");
var RouteManager_1 = require("../../../Routing/RouteManager");
var Utils_1 = require("../../../Utils");
var React_1 = require("../../React");
exports.SplashKey = 'Splash';
exports.DefaultKey = '*';
var RouteHandlerViewModel = /** @class */ (function (_super) {
    tslib_1.__extends(RouteHandlerViewModel, _super);
    function RouteHandlerViewModel(routingMap, routingStateRateLimit) {
        if (routingStateRateLimit === void 0) { routingStateRateLimit = 25; }
        var _this = _super.call(this) || this;
        _this.routingMap = routingMap;
        _this.currentRoute = _this.wx
            .whenAny(RouteManager_1.routeManager.currentRoute, function (x) { return x; })
            // a null route will never get processed so we can filter them out back here
            .filterNull()
            .toProperty();
        var loadComponentParams = _this.wx
            .whenAny(_this.currentRoute, function (x) { return x; })
            // we also need to filter out null routes here (this should only occur for the first event)
            .filterNull()
            .debounceTime(100)
            .map(function (x) {
            // load the routed activator from our routing map
            return _this.getActivator(x);
        })
            .catch(function (e) {
            _this.alertForError(e, "Error Activating Route");
            // we ran into a serious problem, don't propagate the event onward
            // this would happen if the routing map has bad RegExp data in it (perhaps?)
            return rxjs_1.Observable.empty();
        })
            .flatMap(function (x) {
            // check if our activator is actually a redirect
            var result = _this.handleRedirect(x);
            if (result == null) {
                // the activator was a redirect, so don't propagate the event onward
                return rxjs_1.Observable.empty();
            }
            else {
                // this is a normal activator so just push this event onward
                return rxjs_1.Observable.of(result);
            }
        })
            .scan(function (x, next) {
            // build a prev/next pair of activator results
            // we will use the previous activator to confirm if we create a new view model or not
            // we use || null here to ensure our undefined values are always null (consistency)
            return {
                prev: x.next || null,
                next: next || null,
            };
        }, 
        // this initializes the scan operator with an empty object (the first `x` value)
        {})
            // publish is required here because we use this Observable in multiple streams
            // we use publish instead of share so we can kick this engine off at the end of stream composition
            .publish();
        var canLoadComponent = _this.wx
            .whenAny(loadComponentParams, function (x) { return x; })
            .map(function (x) {
            // this condition should always be true, but we put it here just in case
            // we should never end up with loadComponentParams events with a null next activator
            return x.next != null;
        })
            // we must share here because we use this Observable in multiple streams
            .share();
        _this.loadComponent = _this.wx.command(canLoadComponent, function (p) {
            return _this.wx
                .getObservableResultOrAlert(function () {
                // we need to construct an activated component structure so we can send routing state
                // into the viewModel (if it exists)
                return {
                    activator: p.next,
                    // NOTE: getComponent can return null
                    component: _this.getComponent(p.prev, p.next),
                };
            })
                .map(function (x) {
                // send the routing state to the view model (if it exists)
                // this function will return the view model so we can just return it directly
                return _this.updateComponentRoutingState(x.activator, x.component);
            });
        });
        _this.routedComponent = _this.loadComponent.results
            .scan(function (prev, curr) {
            // try and dispose of our current routed component
            // providing it isn't the same component (which should never happen)
            if (prev !== curr) {
                rxjs_1.Subscription.unsubscribe(prev);
            }
            return curr;
        }, undefined)
            .toProperty();
        _this.routingBreadcrumbs = _this.wx
            .whenAny(_this.routedComponent, function (x) { return x; })
            .map(function (x) {
            return React_1.isRoutableViewModel(x)
                ? _this.wx.whenAny(x.breadcrumbs, function (y) { return y; })
                : rxjs_1.Observable.of(undefined);
        })
            .switch()
            .toProperty();
        // when a route changes we enter loading mode and wait until the load finishes
        // at the very least, loadComponent should execute and result in a null view model
        // at which point we can exit loading mode
        _this.isLoading = rxjs_1.Observable.merge(_this.currentRoute.changed.map(function () { return true; }), _this.loadComponent.results.map(function () { return false; }))
            // only show the loading screen if we are really taking a while to load
            // this will make the initial loading screen appear for at least 500ms as
            // well as prevent really short flashes of the loading screen
            .debounceTime(500)
            // initially begin in loading mode
            .toProperty(true);
        _this.addSubscription(_this.wx
            // whenever there are new view model loading params and we can load our view model (which should be always)
            // we can project out just the loading params in preperation for the load command
            .whenAny(loadComponentParams, canLoadComponent, function (params, canLoad) { return ({
            params: params,
            canLoad: canLoad,
        }); })
            .filter(function (x) { return x.canLoad === true; })
            .map(function (x) { return x.params; })
            // debouce here to prevent loading param flux from hammering the load command
            // this may not be necessary for most components, but a small price to pay if a component
            // performs a lot of routing state changes in a short amount of time
            .debounceTime(100)
            .invokeCommand(_this.loadComponent));
        // this handles document title changes for any routed component
        _this.addSubscription(_this.wx
            // for every routed component
            .whenAny(_this.routedComponent, function (x) { return x; })
            // skip the initial null stream element
            .skip(1)
            // wait for routing to settle down
            .debounceTime(100)
            .subscribe(function (component) {
            if (React_1.isRoutableViewModel(component)) {
                // we have a routable component, so watch the documentTitle observable property
                _this.addSubscription(_this.wx
                    .whenAny(component.documentTitle, function (x) {
                    if (String.isNullOrEmpty(x)) {
                        // there isn't any title set (BAD) so warn and use a sane default
                        _this.logger.warn(Object.getName(component) + " does not provide a custom routed browser title");
                        return Object.getName(component);
                    }
                    return x;
                })
                    // give rapid title changes a bit to settle down
                    .debounceTime(100)
                    .subscribe(function (x) {
                    _this.updateDocumentTitle(component, x);
                }));
            }
            else {
                // we don't have a routable component
                // so try and generate a reasonable static title
                var title = void 0;
                if (component == null) {
                    // this shouldn't happen in practice
                    title = 'No Routed Component';
                }
                else if (String.isString(component)) {
                    title = component;
                }
                else {
                    title = Object.getName(component);
                }
                _this.updateDocumentTitle(component, title);
            }
        }));
        _this.addSubscription(Utils_1.PubSub.observe(RoutingStateChanged_1.RoutingStateChangedKey)
            .debounceTime(routingStateRateLimit)
            .withLatestFrom(_this.wx.whenAny(_this.routedComponent, function (x) { return x; }), _this.wx.whenAny(_this.currentRoute, function (x) { return x; }), function (change, component, route) { return ({ change: change, component: component, route: route }); })
            .map(function (x) {
            if (React_1.isRoutableViewModel(x.component)) {
                return {
                    state: x.component.createRoutingState(x.change),
                    route: x.route,
                };
            }
            return undefined;
        })
            .filterNull()
            .subscribe(function (x) {
            _this.navTo(x.route.path, x.state, true);
        }, function (error) {
            _this.alertForError(error, 'Routing State Changed Error');
        }));
        // connect the primary observable to allow the routing engine to start processing routes
        _this.addSubscription(loadComponentParams.connect());
        return _this;
    }
    RouteHandlerViewModel.prototype.updateDocumentTitle = function (component, title) {
        this.logger.debug("Updating document title for component: " + title, component);
        document.title = title;
    };
    RouteHandlerViewModel.prototype.getActivator = function (route) {
        var _this = this;
        var activator;
        // we shouldn't ever hit this function with a null route, but play safe anyways
        if (route != null) {
            this.logger.debug("Loading View Model Activator for '" + route.path + "'...", route);
            // default by just fetching the mapped route directly
            activator = this.routingMap[route.path];
            // if there is no directly mapped route, check for a RegExp route
            if (activator == null) {
                var result = Object.keys(this.routingMap)
                    .asIterable()
                    .filter(function (x) { return x != null && x.length > 0 && x[0] === '^'; })
                    .map(function (x) { return ({ key: x, match: new RegExp(x, 'i').exec(route.path) }); })
                    .filter(function (x) { return x.match != null; })
                    .map(function (x) { return ({
                    key: x.key,
                    match: x.match,
                    activator: _this.routingMap[x.key],
                }); })
                    .first();
                if (result != null) {
                    // if we found a regex match route then set the match properties on the route
                    route.match = result.match;
                    var activatorPath = result.activator != null && result.activator.path != null
                        ? " ('" + result.activator.path + "')"
                        : '';
                    this.logger.debug("Matched RegExp Routing Path '" + route.path + "' with '" + result.key + "'" + activatorPath, route);
                    activator = result.activator;
                }
            }
            // if we found no matching activator yet
            if (activator == null) {
                // warn about a missing route configuration
                this.logger.warn("No activator for '" + route.path + "', falling back to default route", route);
                // fallback on to the default route (this could also be null)
                activator = Object.assign({ creator: function () { return undefined; } }, this.routingMap[exports.DefaultKey]);
            }
        }
        // if our route was null (should never happen) always return a null value
        // otherwise merge the route with the activator to create the RoutedActivator
        // we default to the route path, but allow the activator to override that path
        return route == null
            ? undefined
            : Object.assign({ route: route, path: route.path }, activator);
    };
    RouteHandlerViewModel.prototype.handleRedirect = function (activator) {
        // a redirect is essentially a valid activator with only a path (and no creator)
        var isRedirect = activator != null &&
            activator.route != null &&
            String.isNullOrEmpty(activator.path) === false &&
            activator.creator == null;
        if (isRedirect === true) {
            // this is a redirect route
            this.logger.debug("Redirecting from '" + activator.route.path + "' to '" + activator.path + "'", activator);
            // inform the routing manager of the redirection
            RouteManager_1.routeManager.navTo(activator.path, undefined, true);
            // return null to stop processing this route
            return null;
        }
        else {
            // not a redirect so just return the activator unchanged
            return activator;
        }
    };
    RouteHandlerViewModel.prototype.getComponent = function (prev, next) {
        if (next == null || next.route == null || next.creator == null) {
            // invalid activator, return null (this shouldn't happen)
            return null;
        }
        // routing activator paths are unique, so we can compare them for best results
        else if (prev != null && prev.path === next.path) {
            this.logger.debug("Using same view model for route change (" + prev.route.path + " -> " + next.route.path + ")", next);
            // our old activator matches our new activator, so return the current view model
            // we perform a null check on our observable property just in case, it should never be null
            return this.routedComponent == null ? null : this.routedComponent.value;
        }
        else {
            this.logger.debug("Loading view model for route '" + next.route.path + "'", next);
            // create a new component for the route using the activator function
            return next.creator(next.route);
        }
    };
    RouteHandlerViewModel.prototype.updateComponentRoutingState = function (activator, component) {
        // our activator should never be null at this point, but our component certainly could be
        if (activator != null && React_1.isRoutableViewModel(component)) {
            this.logger.debug("Updating routing state for '" + activator.route.path + "'", activator, component);
            // initialize the routing state to default as an empty object
            activator.route.state = activator.route.state || {};
            // assigning the route in the state (so the routed view model can access route properties)
            activator.route.state.route = activator.route;
            // start the routing state assignment
            component.applyRoutingState(activator.route.state);
        }
        return component;
    };
    RouteHandlerViewModel.displayName = 'RouteHandlerViewModel';
    return RouteHandlerViewModel;
}(React_1.BaseViewModel));
exports.RouteHandlerViewModel = RouteHandlerViewModel;
//# sourceMappingURL=RouteHandlerViewModel.js.map