Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var rxjs_1 = require("rxjs");
var React_1 = require("../../React");
var SearchViewModel = /** @class */ (function (_super) {
    tslib_1.__extends(SearchViewModel, _super);
    function SearchViewModel(liveSearchTimeout, isCaseInsensitive) {
        if (liveSearchTimeout === void 0) { liveSearchTimeout = 500; }
        if (isCaseInsensitive === void 0) { isCaseInsensitive = true; }
        var _this = _super.call(this) || this;
        _this.liveSearchTimeout = liveSearchTimeout;
        _this.isCaseInsensitive = isCaseInsensitive;
        _this.processRequests = _this.wx.command();
        _this.filter = _this.wx.property('');
        // search is executed when a search is to be performed
        _this.search = _this.wx.command();
        _this.clear = _this.wx.command();
        _this.requests = _this.wx
            // project search executions into filter values
            .whenAny(_this.search.results
            // debounce a little extra to protect against too many search invocations
            .debounceTime(250), 
        // "gate" the requests until the processRequests command is invoked
        _this.processRequests.results.take(1), function (x) { return x; })
            // then map into a search request with regex
            .map(function (x) {
            return ({
                filter: _this.filter.value,
                regex: _this.createRegex(_this.filter.value),
            });
        })
            .toProperty();
        _this.searchPending = rxjs_1.Observable.merge(rxjs_1.Observable.merge(_this.filter.changed.map(function () { return true; }), _this.search.results.map(function () { return true; })), _this.requests.changed.map(function () { return false; })).toProperty(false);
        // check to see if we should queue up an initial search
        _this.processRequests.results
            .take(1)
            .filter(function () {
            // check if we have a non-empty filter
            return String.isNullOrEmpty(_this.filter.value) === false;
        })
            .invokeCommand(_this.search);
        _this.addSubscription(_this.wx.whenAny(_this.clear, function (x) { return x; }).subscribe(function () {
            _this.filter.value = '';
        }));
        if (_this.liveSearchTimeout > 0) {
            _this.addSubscription(
            // "gate" live search activation until the processRequests command is invoked
            _this.processRequests.results
                .take(1)
                .switchMap(function () { return _this.filter.changed; })
                .debounceTime(_this.liveSearchTimeout)
                .invokeCommand(_this.search));
        }
        _this.addSubscription(_this.wx.whenAny(_this.requests, function (x) { return x; }).subscribe(function (x) {
            _this.notifyChanged(x);
        }));
        return _this;
    }
    SearchViewModel.prototype.isRoutingStateHandler = function () {
        return true;
    };
    SearchViewModel.prototype.createRoutingState = function (changed) {
        return Object.trim({
            filter: this.getRoutingStateValue(this.filter.value, ''),
        });
    };
    SearchViewModel.prototype.applyRoutingState = function (state) {
        this.filter.value = state.filter == null ? '' : String(state.filter);
        // notify our streams that we're routed and may begin processing requests
        this.processRequests.execute();
    };
    Object.defineProperty(SearchViewModel.prototype, "filterRequests", {
        get: function () {
            return this.requests.changed.map(function (x) { return x.filter; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SearchViewModel.prototype, "regexRequests", {
        get: function () {
            return this.requests.changed.map(function (x) { return x.regex; });
        },
        enumerable: true,
        configurable: true
    });
    SearchViewModel.prototype.createRegex = function (filter) {
        var regex;
        // prefix a filter with ~ to disable standard regex parsing
        if (String.isNullOrEmpty(filter) === false && filter[0] !== '~') {
            try {
                regex = new RegExp(filter, this.isCaseInsensitive ? 'i' : undefined);
            }
            catch (e) {
                // drop out and create a safe regex
            }
        }
        if (regex == null) {
            if (filter[0] === '~' && filter.length > 1) {
                filter = filter.substr(1);
            }
            if (String.isNullOrEmpty(filter) === false) {
                // escape any regex syntax characters from the filter and create a "safe" regex
                // see: http://stackoverflow.com/a/6300234/2789877
                filter = filter.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g, '\\$&');
                regex = new RegExp(filter, this.isCaseInsensitive ? 'i' : undefined);
            }
        }
        return regex;
    };
    SearchViewModel.displayName = 'SearchViewModel';
    return SearchViewModel;
}(React_1.BaseViewModel));
exports.SearchViewModel = SearchViewModel;
//# sourceMappingURL=SearchViewModel.js.map