Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var moment = require("moment");
var React = require("react");
var react_bootstrap_1 = require("react-bootstrap");
var react_fa_1 = require("react-fa");
var CommandButton_1 = require("../CommandButton/CommandButton");
var TimeSpanInput = /** @class */ (function (_super) {
    tslib_1.__extends(TimeSpanInput, _super);
    function TimeSpanInput(props) {
        var _this = _super.call(this, props) || this;
        _this.handleReparse = _this.handleReparse.bind(_this);
        _this.handleIncreaseDuration = _this.handleIncreaseDuration.bind(_this);
        _this.handleDecreaseDuration = _this.handleDecreaseDuration.bind(_this);
        _this.state = {
            unit: props.initialUnit,
            input: TimeSpanInput.formatDuration(props.initialDuration, props.initialUnit),
            duration: props.initialDuration,
        };
        return _this;
    }
    TimeSpanInput.formatDuration = function (duration, unit, precision) {
        if (precision === void 0) { precision = TimeSpanInput.DefaultPrecision; }
        if (duration == null) {
            return '';
        }
        var value = duration.as(unit);
        var unitName = value === 1 ? unit.replace(/s$/, '') : unit;
        return Number(value.toFixed(precision)) + " " + unitName;
    };
    TimeSpanInput.parseDuration = function (state, props) {
        if (state.input == null) {
            return state;
        }
        if (state.input === '') {
            return Object.assign({}, state, {
                duration: undefined,
            });
        }
        var match = state.input.match(/^\s*([\d\.]+)(\s+(\w+))?\s*$/) || [];
        var _a = tslib_1.__read(match, 4), _1 = _a[0], value = _a[1], _2 = _a[2], unitInput = _a[3];
        if (Number.isNumeric(value)) {
            // only process if it's numeric and valid
            var numeric = Number(value);
            if (isNaN(numeric) || numeric < 0) {
                return Object.assign({}, state, {
                    duration: undefined,
                    error: TimeSpanInput.InvalidAmountError,
                });
            }
            if (String.isNullOrEmpty(unitInput)) {
                // single arg
                // just assume we're using the state units
                var duration = moment.duration(numeric, state.unit);
                var input = TimeSpanInput.formatDuration(duration, state.unit, props.precision);
                return Object.assign({}, state, {
                    input: input,
                    duration: duration,
                    error: undefined,
                });
            }
            else {
                // two args
                var unit = moment.normalizeUnits(unitInput);
                if (!String.isNullOrEmpty(unit)) {
                    if (/s$/.test(unit) === false) {
                        // pluralize the unit if it isn't already
                        unit = unit + "s";
                    }
                    if (props.units.indexOf(unit) >= 0) {
                        var duration = moment.duration(numeric, unit);
                        var input = TimeSpanInput.formatDuration(duration, unit, props.precision);
                        return Object.assign({}, state, {
                            unit: unit,
                            input: input,
                            duration: duration,
                            error: undefined,
                        });
                    }
                }
                return Object.assign({}, state, {
                    duration: undefined,
                    error: TimeSpanInput.InvalidUnitError,
                });
            }
        }
        return Object.assign({}, state, {
            duration: undefined,
            error: TimeSpanInput.InvalidFormatError,
        });
    };
    TimeSpanInput.prototype.componentDidUpdate = function (prevProps, prevState, prevContext) {
        var _this = this;
        if (this.props.onMomentDurationChanged != null &&
            prevState != null &&
            this.state != null) {
            if (prevState.duration != null) {
                if (this.state.duration == null ||
                    prevState.duration !== this.state.duration) {
                    this.props.onMomentDurationChanged(this.state.duration, this.state.unit);
                }
            }
            else if (this.state.duration != null) {
                if (prevState.duration == null ||
                    prevState.duration !== this.state.duration) {
                    this.props.onMomentDurationChanged(this.state.duration, this.state.unit);
                }
            }
        }
        if (this.props.onMomentUnitChanged != null &&
            prevState != null &&
            this.state != null) {
            if (prevState.unit != null) {
                if (this.state.unit == null || prevState.unit !== this.state.unit) {
                    this.props.onMomentUnitChanged(this.state.unit, this.state.duration);
                }
            }
            else if (this.state.unit != null) {
                if (prevState.unit == null || prevState.unit !== this.state.unit) {
                    this.props.onMomentUnitChanged(this.state.unit, this.state.duration);
                }
            }
        }
        if (this.props.duration != null &&
            prevState.duration !== this.props.duration) {
            this.setState(function (ps, p) {
                return Object.assign({}, ps, {
                    input: _this.props.duration == null
                        ? ps.input
                        : TimeSpanInput.formatDuration(_this.props.duration, ps.unit, _this.props.precision),
                    duration: _this.props.duration,
                });
            });
        }
    };
    TimeSpanInput.prototype.render = function () {
        var _a = this.restProps(function (x) {
            var bsClass = x.bsClass, bsSize = x.bsSize, controlId = x.controlId, duration = x.duration, units = x.units, initialUnit = x.initialUnit, initialDuration = x.initialDuration, precision = x.precision, onMomentDurationChanged = x.onMomentDurationChanged, onMomentUnitChanged = x.onMomentUnitChanged, reparseButton = x.reparseButton;
            return {
                bsClass: bsClass,
                bsSize: bsSize,
                controlId: controlId,
                duration: duration,
                units: units,
                initialUnit: initialUnit,
                initialDuration: initialDuration,
                precision: precision,
                onMomentDurationChanged: onMomentDurationChanged,
                onMomentUnitChanged: onMomentUnitChanged,
                reparseButton: reparseButton,
            };
        }), className = _a.className, props = _a.props, rest = _a.rest;
        return (React.createElement(react_bootstrap_1.FormGroup, tslib_1.__assign({}, rest, { className: this.wxr.classNames('TimeSpanInput', className), bsClass: props.bsClass, bsSize: props.bsSize, controlId: props.controlId, validationState: String.isNullOrEmpty(this.state.error) ? undefined : 'error' }),
            this.renderFormInput(),
            React.createElement(react_bootstrap_1.FormControl.Feedback, null),
            this.renderHelp()));
    };
    TimeSpanInput.prototype.renderFormInput = function () {
        return (React.createElement(react_bootstrap_1.InputGroup, null,
            this.renderTextInput(),
            this.renderButtons()));
    };
    TimeSpanInput.prototype.renderTextInput = function () {
        if (React.Children.count(this.props.children) === 1) {
            var formControl = React.Children.only(this.props.children);
            var props = Object.assign({
                value: this.state.input,
                onChange: this.handleInput.bind(this),
                onBlur: this.handleParse.bind(this),
            }, formControl.props);
            return React.cloneElement(formControl, props);
        }
        return (React.createElement(react_bootstrap_1.FormControl, { type: "text", placeholder: this.props.placeholder, value: this.state.input, onChange: this.handleInput.bind(this), onBlur: this.handleParse.bind(this) }));
    };
    TimeSpanInput.prototype.renderButtons = function () {
        var _this = this;
        return (React.createElement(react_bootstrap_1.InputGroup.Button, null,
            this.wxr.renderConditional(this.props.reparseButton, function () { return (React.createElement(CommandButton_1.CommandButton, { id: _this.props.id + "-reparse", className: "TimeSpanInput-adjustButton", onClick: _this.handleReparse, tooltip: "Reparse input" },
                React.createElement(react_fa_1.Icon, { name: "check" }))); }),
            this.renderDropdown(),
            React.createElement(CommandButton_1.CommandButton, { className: "TimeSpanInput-adjustButton", componentClass: "button", onClick: this.handleIncreaseDuration },
                React.createElement(react_fa_1.Icon, { name: "chevron-up" })),
            React.createElement(CommandButton_1.CommandButton, { className: "TimeSpanInput-adjustButton", componentClass: "button", onClick: this.handleDecreaseDuration },
                React.createElement(react_fa_1.Icon, { name: "chevron-down" }))));
    };
    TimeSpanInput.prototype.renderDropdown = function () {
        var _this = this;
        return (React.createElement(react_bootstrap_1.DropdownButton, { id: "TimeSpanInput-units-" + this.props.id, className: "TimeSpanInput-unitDropdown", title: this.state.unit, bsSize: this.props.bsSize, onSelect: this.handleChangeUnit.bind(this) }, this.props.units.map(function (x) { return (React.createElement(react_bootstrap_1.MenuItem, { key: x, eventKey: x, active: x === _this.state.unit }, x)); })));
    };
    TimeSpanInput.prototype.renderHelp = function () {
        return this.state.error && React.createElement(react_bootstrap_1.HelpBlock, null, this.state.error);
    };
    TimeSpanInput.prototype.handleChangeUnit = function (unit) {
        var _this = this;
        this.setState(function (prevState, props) {
            var duration = _this.cloneDurationFromState(prevState);
            var input = TimeSpanInput.formatDuration(duration, unit, _this.props.precision);
            return Object.assign({ error: undefined }, prevState, {
                unit: unit,
                input: input,
                duration: duration,
            });
        });
    };
    TimeSpanInput.prototype.handleReparse = function () {
        // this is a fake handler to simulate a request to parse, but it really only forces a blur
    };
    TimeSpanInput.prototype.handleIncreaseDuration = function () {
        var _this = this;
        this.setState(function (prevState, props) {
            var duration = _this.adjustDurationFromState(prevState, 1);
            var input = TimeSpanInput.formatDuration(duration, prevState.unit, _this.props.precision);
            return Object.assign({}, prevState, {
                input: input,
                duration: duration,
                error: duration.valueOf() < 0 ? TimeSpanInput.InvalidAmountError : undefined,
            });
        });
    };
    TimeSpanInput.prototype.handleDecreaseDuration = function () {
        var _this = this;
        this.setState(function (prevState, props) {
            var duration = _this.adjustDurationFromState(prevState, -1);
            var input = TimeSpanInput.formatDuration(duration, prevState.unit, _this.props.precision);
            return Object.assign({}, prevState, {
                input: input,
                duration: duration,
                error: duration.valueOf() < 0 ? TimeSpanInput.InvalidAmountError : undefined,
            });
        });
    };
    TimeSpanInput.prototype.handleInput = function (e) {
        var input = e.currentTarget.value;
        this.setState(function (prevState, props) {
            return Object.assign({}, prevState, {
                input: input,
            });
        });
    };
    TimeSpanInput.prototype.handleParse = function () {
        var _this = this;
        this.setState(function (prevState, props) {
            return Object.assign({}, prevState, TimeSpanInput.parseDuration(prevState, _this.props));
        });
    };
    TimeSpanInput.prototype.cloneDurationFromState = function (state) {
        if (state.duration == null) {
            return moment.duration(0, state.unit);
        }
        return state.duration.clone();
    };
    TimeSpanInput.prototype.adjustDurationFromState = function (state, amount) {
        return this.cloneDurationFromState(state).add(amount, state.unit);
    };
    TimeSpanInput.DefaultPrecision = 2;
    TimeSpanInput.InvalidFormatError = 'Invalid Duration Format';
    TimeSpanInput.InvalidAmountError = 'Invalid Duration Amount';
    TimeSpanInput.InvalidUnitError = 'Invalid Unit';
    TimeSpanInput.defaultProps = {
        units: ['seconds', 'minutes', 'hours', 'days'],
        initialUnit: 'days',
        precision: TimeSpanInput.DefaultPrecision,
    };
    return TimeSpanInput;
}(React.Component));
exports.TimeSpanInput = TimeSpanInput;
//# sourceMappingURL=TimeSpanInput.js.map