// tslint:disable:max-classes-per-file
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var rxjs_1 = require("rxjs");
var AppViewModel_1 = require("../Common/App/AppViewModel");
var React_1 = require("../React");
var RoutingMap = /** @class */ (function () {
    function RoutingMap(baseUri, defaultIconName) {
        if (baseUri === void 0) { baseUri = '#/demo'; }
        if (defaultIconName === void 0) { defaultIconName = 'flask'; }
        this.baseUri = baseUri;
        this.defaultIconName = defaultIconName;
        this.viewModelMap = {};
        this.menuMap = {};
    }
    RoutingMap.prototype.addRoute = function (menuName, path, name, activator, uri, iconName) {
        if (/^\w+$/.test(path)) {
            uri = uri || path;
            path = "^" + path + "$";
        }
        this.viewModelMap[path] = activator;
        var menu = (this.menuMap[menuName] =
            this.menuMap[menuName] ||
                {
                    id: menuName,
                    header: menuName + " Demos",
                    items: [],
                });
        menu.items.push({
            id: path,
            header: name,
            uri: this.getUri(path, uri),
            iconName: iconName || this.defaultIconName,
            order: menu.items.length,
        });
    };
    RoutingMap.prototype.getUri = function (path, uri) {
        return this.baseUri + "/" + (uri || path);
    };
    Object.defineProperty(RoutingMap.prototype, "menus", {
        get: function () {
            var _this = this;
            return Object.getOwnPropertyNames(this.menuMap).map(function (x) { return _this.menuMap[x]; });
        },
        enumerable: true,
        configurable: true
    });
    RoutingMap.displayName = 'RoutingMap';
    return RoutingMap;
}());
exports.RoutingMap = RoutingMap;
var ComponentDemoViewModel = /** @class */ (function (_super) {
    tslib_1.__extends(ComponentDemoViewModel, _super);
    function ComponentDemoViewModel(routeMap) {
        var _this = _super.call(this) || this;
        _this.routeMap = routeMap;
        _this.setComponent = _this.wx.command();
        _this.setColumns = _this.wx.command();
        _this.reRender = _this.wx.command();
        var demoAlert = _this.wx.command(rxjs_1.Observable.of(true), function (x) {
            return _this.createAlert(x, 'Demo Alert');
        });
        _this.demoAlertItem = {
            id: 'demo-alert-item',
            header: 'Generate Alert',
            command: demoAlert,
            visibleWhenDisabled: true,
            iconName: 'flask',
        };
        _this.columns = _this.wx.whenAny(_this.setColumns, function (x) { return x; }).toProperty(12);
        _this.routedComponent = _this.wx
            .whenAny(_this.setComponent, _this.reRender.results.startWith(undefined), function (x) { return x; })
            .toProperty(undefined, false);
        _this.componentRoute = _this.wx
            .whenAny(_this.routedComponent, function (x) { return (x == null ? undefined : x.componentRoute); })
            .toProperty(undefined, false);
        _this.component = _this.wx
            .whenAny(_this.routedComponent, function (x) { return x; })
            .map(function (x) {
            if (x == null) {
                return undefined;
            }
            // if our routed component handles routing state, then we can pass on the routed state
            if (React_1.isRoutingStateHandler(x.component)) {
                x.component.applyRoutingState(x.routingState);
            }
            return x.component;
        })
            .toProperty(undefined, false);
        _this.addSubscription(_this.wx.whenAny(_this.columns.changed, function (x) { return x; }).subscribe(function (x) {
            _this.notifyChanged(x);
        }));
        _this.addSubscription(_this.wx.whenAny(_this.component, function (x) { return x; }).subscribe(function () {
            if (_this.pageHeader != null) {
                _this.pageHeader.updateDynamicContent();
            }
        }));
        // set a default title
        _this.updateDocumentTitle.execute('Loading Demos...');
        // simulate some breadcrumbs
        _this.updateRoutingBreadcrumbs.execute([
            {
                key: 1,
                content: 'Here',
                href: '#/demo',
                title: 'title-based tooltips supported',
            },
            {
                key: 2,
                content: 'Are',
                href: '#/demo',
                tooltip: 'simple string tooltip overlays',
            },
            {
                key: 3,
                content: 'Some',
                href: '#/demo',
                tooltip: {
                    id: 'demo-tt',
                    title: 'Popovers!',
                    children: 'custom props-based tooltip overlays',
                },
            },
            {
                key: 4,
                content: 'Breadcrumbs',
                href: '#/demo',
                tooltip: { placement: 'top', children: 'regular tooltip' },
            },
        ]);
        // this is very similar to what the route handler does for title updates
        // we are essentially projecting the demo title and passing it up the chain
        _this.addSubscription(_this.wx
            .whenAny(_this.component, function (x) { return x; })
            .debounceTime(100)
            .subscribe(function (component) {
            if (React_1.isRoutableViewModel(component)) {
                _this.addSubscription(_this.wx
                    .whenAny(component.documentTitle, function (x) {
                    return String.isNullOrEmpty(x) ? Object.getName(component) : x;
                })
                    .debounceTime(100)
                    .map(function (x) { return "Demo: " + x; })
                    .invokeCommand(_this.updateDocumentTitle));
            }
            else {
                var title = void 0;
                if (component == null) {
                    title = 'No Routed Component';
                }
                else if (String.isString(component)) {
                    title = component;
                }
                else {
                    title = Object.getName(component);
                }
                _this.updateDocumentTitle.execute("Demo: " + title);
            }
        }));
        return _this;
    }
    ComponentDemoViewModel.prototype.getComponentRoute = function (state) {
        if (state == null || state.route == null) {
            return undefined;
        }
        if (Array.isArray(state.route.match) && state.route.match.length >= 2) {
            return state.route.match[1];
        }
        return undefined;
    };
    ComponentDemoViewModel.prototype.getRoutedComponent = function (state) {
        var _this = this;
        // extract the component route from the routing state
        var componentRoute = this.getComponentRoute(state);
        // if our component route is null then we can ignore activation
        // let the routing system perform some automated navigation to a new route
        if (componentRoute != null) {
            if (componentRoute === this.componentRoute.value &&
                this.component.value != null) {
                this.logger.debug("Using Previously Activated Component for \"" + componentRoute + "\"...");
                // if our component route has not changed then we can just use our previously activated component
                return {
                    componentRoute: componentRoute,
                    component: this.routedComponent.value.component,
                    // we may have routed with different routing state, so apply it here.
                    // it is possible that the routed component has no routing state, but because routing
                    // state must be set we need to then sanitize the value with an empty state object
                    routingState: state.state || {},
                };
            }
            else {
                this.logger.debug("Loading Component for \"" + componentRoute + "\"...");
                // default to using the routed component's state
                // it is possible that the routed component has no routing state, but because routing
                // state must be set we need to then sanitize the value with an empty state object
                var routingState = state.state || {};
                // try and load our component from the component map using a static route
                var activator = this.routeMap.viewModelMap[componentRoute];
                // if no static route was found then we can attempt an expression route
                if (activator == null) {
                    // project out the first expression route match (path, match, and activator)
                    var result = Object.keys(this.routeMap.viewModelMap)
                        .asIterable()
                        .filter(function (x) { return x != null && x.length > 0 && x[0] === '^'; })
                        .map(function (x) { return ({ path: x, regex: new RegExp(x, 'i') }); })
                        .map(function (x) { return ({ path: x.path, match: x.regex.exec(componentRoute) }); })
                        .filter(function (x) { return x.match != null; })
                        .map(function (x) { return ({
                        path: x.path,
                        match: x.match,
                        activator: _this.routeMap.viewModelMap[x.path],
                    }); })
                        .first();
                    if (result != null) {
                        // if we found an expression route match, assign our activator
                        activator = result.activator;
                        // and override the routed state's path and match with the demo's context
                        routingState = Object.assign({}, routingState, {
                            path: result.path,
                            match: result.match,
                        });
                    }
                }
                // activate our component from the routing state
                var component = activator == null ? null : activator(state);
                if (component != null) {
                    this.logger.debug("Loaded Component \"" + Object.getName(component) + "\"", component);
                }
                return {
                    componentRoute: componentRoute,
                    component: component,
                    routingState: routingState,
                };
            }
        }
        return undefined;
    };
    ComponentDemoViewModel.prototype.createRoutingState = function (changed) {
        var columns;
        if (this.columns.value !== 12) {
            columns = this.columns.value;
        }
        var state = {
            route: {
                path: "/demo/" + (this.componentRoute.value || ''),
            },
            columns: this.getRoutingStateValue(this.columns.value, 12),
            state: this.getRoutingStateValue(this.component.value, function (x) {
                if (React_1.isRoutingStateHandler(x)) {
                    return x.createRoutingState(changed.context);
                }
                return {};
            }) || {},
        };
        return Object.trim(state);
    };
    ComponentDemoViewModel.prototype.applyRoutingState = function (state) {
        var routedComponent = this.getRoutedComponent(state);
        // if there is no route, then route to help
        if (routedComponent == null ||
            String.isNullOrEmpty(routedComponent.componentRoute)) {
            this.navTo('#/demo/help');
        }
        else {
            this.setComponent.execute(routedComponent);
            this.setColumns.execute(state.columns == null ? 12 : state.columns);
        }
        // keep a handle on the current app page header
        // NOTE: we make this assignment every time we apply routing state,
        //       but it should be a no-op every time but the first
        this.pageHeader = AppViewModel_1.Current.header;
    };
    ComponentDemoViewModel.prototype.getSearch = function () {
        if (React_1.isRoutableViewModel(this.component.value)) {
            return this.component.value.getSearch();
        }
        return undefined;
    };
    ComponentDemoViewModel.prototype.getSidebarMenus = function () {
        return [
            {
                id: 'sidebar-demos',
                header: 'Integration Demos',
                items: [
                    {
                        id: 'todolist',
                        header: 'Todo List',
                        uri: '#/demo/todolist',
                        iconName: 'list',
                    },
                ],
            },
            {
                id: 'sidebar-1',
                header: 'Section 1',
                items: [
                    Object.assign({}, this.demoAlertItem, {
                        id: this.demoAlertItem.id + "-sidebar-1",
                        commandParameter: 'Sidebar Section 1 Menu Item',
                    }),
                ],
            },
            {
                id: 'sidebar-2',
                header: 'Section 2',
                items: [
                    Object.assign({}, this.demoAlertItem, {
                        id: this.demoAlertItem.id + "-sidebar-2-1",
                        commandParameter: 'Sidebar Section 2 Menu Item 1',
                    }),
                    Object.assign({}, this.demoAlertItem, {
                        id: this.demoAlertItem.id + "-sidebar-2-2",
                        commandParameter: 'Sidebar Section 2 Menu Item 2',
                    }),
                ],
            },
        ];
    };
    ComponentDemoViewModel.prototype.getNavbarMenus = function () {
        return this.routeMap.menus.concat({
            id: this.demoAlertItem.id + "-menu",
            header: 'Sample Routed Menu',
            order: -1,
            items: [
                Object.assign({}, this.demoAlertItem, {
                    id: this.demoAlertItem.id + "-menuitem",
                    commandParameter: 'Routed Menu Item',
                }),
            ],
        });
    };
    ComponentDemoViewModel.prototype.getNavbarActions = function () {
        return [
            {
                id: 'reRender',
                header: 'Re-Render',
                command: this.reRender,
                bsStyle: 'primary',
            },
            Object.assign({}, this.demoAlertItem, {
                id: this.demoAlertItem.id + "-navbar",
                commandParameter: 'Navbar Action',
            }),
        ];
    };
    ComponentDemoViewModel.prototype.getHelpMenuItems = function () {
        return [
            Object.assign({}, this.demoAlertItem, {
                id: this.demoAlertItem.id + "-help",
                commandParameter: 'Help Menu Item',
            }),
        ];
    };
    ComponentDemoViewModel.prototype.getAdminMenuItems = function () {
        return [
            Object.assign({}, this.demoAlertItem, {
                id: this.demoAlertItem.id + "-admin",
                commandParameter: 'Admin Menu Item',
            }),
        ];
    };
    ComponentDemoViewModel.prototype.getUserMenuItems = function () {
        return [
            Object.assign({}, this.demoAlertItem, {
                id: this.demoAlertItem.id + "-user",
                commandParameter: 'User Menu Item',
            }),
        ];
    };
    ComponentDemoViewModel.displayName = 'ComponentDemoViewModel';
    return ComponentDemoViewModel;
}(React_1.BaseRoutableViewModel));
exports.ComponentDemoViewModel = ComponentDemoViewModel;
//# sourceMappingURL=ComponentDemoViewModel.js.map