Object.defineProperty(exports, "__esModule", { value: true });
var moment = require("moment");
var rxjs_1 = require("rxjs");
var Routing_1 = require("../../Routing");
var Utils_1 = require("../../Utils");
var WebRx_1 = require("../../WebRx");
var Components = require("../Common");
var ComponentDemoViewModel_1 = require("./ComponentDemoViewModel");
var TodoListViewModel_1 = require("./TodoList/TodoListViewModel");
exports.demoRoutingMap = new ComponentDemoViewModel_1.RoutingMap();
function createSampleItem(cat, name, requiredBy, id) {
    if (id === void 0) { id = 0; }
    return {
        id: id,
        cat: cat,
        name: name,
        requiredBy: requiredBy,
    };
}
exports.createSampleItem = createSampleItem;
function cloneSampleItem(item) {
    return createSampleItem(item.cat, item.name, item.requiredBy, item.id);
}
exports.cloneSampleItem = cloneSampleItem;
function cloneSampleTreeItem(item, items) {
    item = cloneSampleItem(item);
    item.items = items.toArray();
    return item;
}
exports.cloneSampleTreeItem = cloneSampleTreeItem;
exports.sampleListData = [
    createSampleItem('item', 'test 1', 'now'),
    createSampleItem('item', 'test 2', 'tomorrow'),
    createSampleItem('item', 'test 3', 'yesterday'),
    createSampleItem('test', 'test 4', 'test4'),
    createSampleItem('test', 'test 5', 'test5'),
    createSampleItem('test', 'test 6', 'test6'),
    createSampleItem('test', 'test 7', 'test7'),
    createSampleItem('test', 'test 8', 'test8'),
    createSampleItem('test', 'test 9', 'test9'),
    createSampleItem('test', 'test 10', 'test10'),
    createSampleItem('test', 'test 11', 'test11'),
].map(function (x, i) { return Object.assign(x, { id: i + 1 }); });
exports.sampleTreeData = exports.sampleListData.map(function (x) {
    return Object.assign({
        items: exports.sampleListData.slice(0, 3).map(function (y) {
            return Object.assign({
                items: exports.sampleListData
                    .slice(0, 3)
                    .map(function (z) { return Object.assign({}, z, { name: x.name + ' (2)' }); }),
            }, y, { name: x.name + ' (1)' });
        }),
    }, x, { name: x.name + ' (0)' });
});
function sampleDataSource(request) {
    if (request == null) {
        return undefined;
    }
    var search = Components.ItemListPanelViewModel.getSearchRequest(request);
    if (search != null && search.filter === 'throw') {
        throw new Error('Simulated Coding Error');
    }
    return (rxjs_1.Observable.of(exports.sampleListData)
        .map(function (data) { return ({
        data: data,
        page: request.page,
        sort: request.sort,
        context: request.context,
    }); })
        .do(function (x) {
        var msg = [
            'Simulating Async Data Result...',
            "page = " + String.stringify(x.page),
            "sort = " + String.stringify(x.sort),
            "context = " + String.stringify(x.context),
        ].join('<br/>');
        Utils_1.Alert.create(msg, 'Async Data Source Demo', undefined, 2000);
    })
        // simulate async result delay
        .delay(2000)
        .map(function (x) {
        var query = x.data.asIterable();
        if (search != null) {
            if (search.filter === 'error') {
                throw new Error('Simulated Async Data Source Error');
            }
            query = query.filter(function (y) {
                return (y.name.indexOf(search.filter || '') >= 0 ||
                    y.requiredBy.indexOf(search.filter || '') >= 0);
            });
        }
        var count = query.count();
        if (x.sort != null &&
            x.sort.direction != null &&
            !String.isNullOrEmpty(x.sort.field)) {
            var field_1 = x.sort.field;
            if (x.sort.direction === Utils_1.Compare.SortDirection.Ascending) {
                query = query.orderBy(function (y) { return y[field_1]; });
            }
            else if (x.sort.direction === Utils_1.Compare.SortDirection.Descending) {
                query = query.orderByDescending(function (y) { return y[field_1]; });
            }
        }
        if (x.page != null) {
            var offset = x.page.offset || 0;
            var limit = x.page.limit || 0;
            if (offset > 0) {
                query = query.skip(offset);
            }
            if (limit > 0) {
                query = query.take(limit);
            }
        }
        var items = query.toArray();
        return {
            items: items,
            count: count,
        };
    }));
}
function sampleTreeDataSource(request) {
    if (request == null) {
        return undefined;
    }
    var search = Components.ItemListPanelViewModel.getSearchRequest(request);
    if (search != null && search.filter === 'throw') {
        throw new Error('Simulated Coding Error');
    }
    return (rxjs_1.Observable.of(exports.sampleTreeData)
        .map(function (data) { return ({
        data: data,
        context: request.context,
    }); })
        .do(function (x) {
        var msg = [
            'Simulating Async Data Result...',
            "context = " + String.stringify(x.context),
        ].join('<br/>');
        Utils_1.Alert.create(msg, 'Async Data Source Demo', undefined, 2000);
    })
        // simulate async result delay
        .delay(2000)
        .map(function (x) {
        var query = x.data.asIterable();
        if (search != null) {
            if (search.filter === 'error') {
                throw new Error('Simulated Async Data Source Error');
            }
            query = Components.filterTreeItems(query, function (y) { return y.items; }, function (y, i) { return cloneSampleTreeItem(y, i); }, function (y) {
                return (y.name.indexOf(search.filter || '') >= 0 ||
                    y.requiredBy.indexOf(search.filter || '') >= 0);
            });
        }
        var count = query.count();
        var items = query.toArray();
        return {
            items: items,
            count: count,
        };
    }));
}
exports.demoRoutingMap.addRoute('React', 'Loading', 'Loading', function (state) { return 'Loading'; });
exports.demoRoutingMap.addRoute('React', 'SizedLoading', 'Sized Loading', function (state) { return 'SizedLoading'; });
exports.demoRoutingMap.addRoute('React', 'Splash', 'Splash', function (state) { return 'Splash'; });
exports.demoRoutingMap.addRoute('React', 'CommandButton', 'Command Button', function (state) { return 'CommandButton'; });
exports.demoRoutingMap.addRoute('React', 'CommandButtonCondition', 'Command Button (condition)', function (state) {
    var condition = Components.CommandButton.wx.property(0);
    var cmd = Components.CommandButton.wx.command(function (x) {
        return ++condition.value;
    }, condition.changed, function (c, x) {
        return x == null || c <= x;
    }, condition.value);
    return {
        displayName: 'CommandButtonCondition',
        condition: condition,
        cmd: cmd,
    };
});
exports.demoRoutingMap.addRoute('React', 'Alert', 'Alert', function (state) { return 'Alert'; });
exports.demoRoutingMap.addRoute('React', 'ContextMenu', 'Context Menu', function (state) { return 'ContextMenu'; });
exports.demoRoutingMap.addRoute('React', 'ProfilePicture', 'Profile Picture', function (state) { return 'ProfilePicture'; });
exports.demoRoutingMap.addRoute('React', 'ItemsPanel', 'Items Panel', function (state) { return 'ItemsPanel'; });
exports.demoRoutingMap.addRoute('React', 'ItemsPanelBound', 'Items Panel (Bound)', function (state) { return 'ItemsPanelBound'; });
exports.demoRoutingMap.addRoute('React', 'ListGroupPanel', 'ListGroup Panel', function (state) { return 'ListGroupPanel'; });
exports.demoRoutingMap.addRoute('React', 'ListGroupPanelBound', 'ListGroup Panel (Bound)', function (state) { return 'ListGroupPanelBound'; });
exports.demoRoutingMap.addRoute('React', 'GridPanel', 'Grid Panel', function (state) { return 'GridPanel'; });
exports.demoRoutingMap.addRoute('React', 'StackPanel', 'Stack Panel', function (state) { return 'StackPanel'; });
exports.demoRoutingMap.addRoute('React', 'UniformGridPanel', 'Uniform Grid Panel', function (state) { return 'UniformGridPanel'; });
exports.demoRoutingMap.addRoute('React', 'WrapPanel', 'Wrap Panel', function (state) { return 'WrapPanel'; });
exports.demoRoutingMap.addRoute('React', 'ContentTooltip', 'Content Tooltip', function (state) { return 'ContentTooltip'; });
exports.demoRoutingMap.addRoute('React', 'NavButton', 'Nav Button', function (state) { return 'NavButton'; });
exports.demoRoutingMap.addRoute('React', 'TreeItem', 'Tree Item', function (state) { return 'TreeItem'; });
exports.demoRoutingMap.addRoute('React', 'TreeItemPresenter', 'Tree Item Presenter', function (state) { return 'TreeItemPresenter'; });
exports.demoRoutingMap.addRoute('React', 'HorizontalTreeItemPresenter', 'Tree Item Presenter (Horizontal)', function (state) { return 'HorizontalTreeItemPresenter'; });
exports.demoRoutingMap.addRoute('React', 'HorizontalItemsTreeItemPresenter', 'Tree Item Presenter (Horizontal Items)', function (state) { return 'HorizontalItemsTreeItemPresenter'; });
exports.demoRoutingMap.addRoute('React', 'HorizontalRootTreeItemPresenter', 'Tree Item Presenter (Horizontal Root)', function (state) { return 'HorizontalRootTreeItemPresenter'; });
exports.demoRoutingMap.addRoute('React', 'CommonPanel', 'Common Panel', function (state) { return 'CommonPanel'; });
exports.demoRoutingMap.addRoute('React', 'CommonPanelList', 'Common Panel (List)', function (state) { return 'CommonPanelList'; });
exports.demoRoutingMap.addRoute('React', 'CommonPanelTable', 'Common Panel (Table)', function (state) { return 'CommonPanelTable'; });
exports.demoRoutingMap.addRoute('React', 'CommonPanelTest', 'Common Panel (Test)', function (state) { return 'CommonPanelTest'; });
exports.demoRoutingMap.addRoute('React', 'ObservableWrapper', 'Observable Wrapper', function (state) { return 'ObservableWrapper'; });
exports.demoRoutingMap.addRoute('React', 'TimeSpanInput', 'Time Span Input', function (state) { return 'TimeSpanInput'; });
exports.demoRoutingMap.addRoute('webrx-react', 'Search', 'Search', function (state) {
    var viewModel = new Components.SearchViewModel();
    WebRx_1.wx.whenAny(viewModel.requests, function (x) { return x; })
        .filterNull()
        .subscribe(function (x) {
        Utils_1.Alert.create(String.stringify(x), 'Search Requested');
    });
    return viewModel;
});
exports.demoRoutingMap.addRoute('webrx-react', 'ItemsList', 'Items (List)', function (state) { return new Components.ItemsViewModel(exports.sampleListData); });
exports.demoRoutingMap.addRoute('webrx-react', 'ItemsWrap', 'Items (Wrap)', function (state) { return new Components.ItemsViewModel(exports.sampleListData); });
exports.demoRoutingMap.addRoute('webrx-react', 'ItemsUGrid', 'Items (Uniform Grid)', function (state) { return new Components.ItemsViewModel(exports.sampleListData); });
exports.demoRoutingMap.addRoute('webrx-react', 'ItemsHStack', 'Items (Horizontal Stack)', function (state) { return new Components.ItemsViewModel(exports.sampleListData); });
exports.demoRoutingMap.addRoute('webrx-react', 'ItemsGrid', 'Items (Grid)', function (state) { return new Components.ItemsViewModel(exports.sampleListData); });
exports.demoRoutingMap.addRoute('webrx-react', 'ItemsTree', 'Items (Tree)', function (state) { return new Components.ItemsViewModel(exports.sampleTreeData); });
exports.demoRoutingMap.addRoute('webrx-react', 'ListItemsDefault', 'ListItems (default)', function (state) { return new Components.ListItemsViewModel(exports.sampleListData); });
exports.demoRoutingMap.addRoute('webrx-react', 'ListItemsListGroup', 'ListItems (ListGroupPanel)', function (state) { return new Components.ListItemsViewModel(exports.sampleListData); });
exports.demoRoutingMap.addRoute('webrx-react', 'ListItemsPanel', 'ListItems (Default Panel)', function (state) { return new Components.ListItemsViewModel(exports.sampleListData); });
exports.demoRoutingMap.addRoute('webrx-react', 'ListItemsUGrid', 'ListItems (Uniform Grid)', function (state) { return new Components.ListItemsViewModel(exports.sampleListData); });
exports.demoRoutingMap.addRoute('webrx-react', 'ListItemsGrid', 'ListItems (Grid)', function (state) { return new Components.ListItemsViewModel(exports.sampleListData); });
exports.demoRoutingMap.addRoute('webrx-react', 'ListItemsGridAuto', 'ListItems (Auto Grid)', function (state) { return new Components.ListItemsViewModel(exports.sampleListData); });
exports.demoRoutingMap.addRoute('webrx-react', 'ListItemsTree', 'ListItems (Tree)', function (state) {
    return new Components.TreeListItemsViewModel(exports.sampleTreeData, function (x) { return x.items; });
});
exports.demoRoutingMap.addRoute('webrx-react', 'ModalDialog', 'Modal Dialog', function (state) {
    var createContext = WebRx_1.wx.command(function (x) { return "[" + moment().format() + "] " + x; });
    // we are simulating a modal being contained within another view model
    return {
        displayName: 'ModalDialogViewModel',
        viewModel: new Components.ModalDialogViewModel(createContext.results),
        createContext: createContext,
        accept: WebRx_1.wx.command(function (x) { return Utils_1.Alert.create(x, 'Modal Accepted', 'success'); }),
        reject: WebRx_1.wx.command(function (x) { return Utils_1.Alert.create(x, 'Modal Rejected', 'danger'); }),
    };
});
exports.demoRoutingMap.addRoute('webrx-react', 'Tabs', 'Tabs', function (state) { return new Components.TabsViewModel(); });
exports.demoRoutingMap.addRoute('webrx-react', 'StaticTabs', 'Static Tabs', function (state) { return new Components.TabsViewModel(); });
exports.demoRoutingMap.addRoute('webrx-react', 'DataGrid', 'Data Grid', function (state) {
    // const prop = wx.property<SampleData[]>(undefined, false);
    // simulate delayed loading
    var delay = 2000;
    var data = rxjs_1.Observable.of(exports.sampleListData)
        .do(function () {
        Utils_1.Alert.create('Simulating Delay', 'Delayed Observable Property List Loading', undefined, delay);
    })
        .delay(delay);
    return new Components.DataGridViewModel(data);
});
exports.demoRoutingMap.addRoute('webrx-react', 'DataGridAutoCol', 'Data Grid (Automatic Columns)', function (state) { return new Components.DataGridViewModel(exports.sampleListData); });
exports.demoRoutingMap.addRoute('webrx-react', 'DataGridNoPager', 'Data Grid (No Pager)', function (state) { return new Components.DataGridViewModel(exports.sampleListData, null); });
exports.demoRoutingMap.addRoute('webrx-react', 'DataGridPager', 'Data Grid (Custom Pager)', function (state) { return new Components.DataGridViewModel(exports.sampleListData); });
exports.demoRoutingMap.addRoute('webrx-react', 'DataGridList', 'Data Grid (List View)', function (state) { return new Components.DataGridViewModel(exports.sampleListData); });
exports.demoRoutingMap.addRoute('webrx-react', 'DataGridUGrid', 'Data Grid (Uniform Grid Panel)', function (state) { return new Components.DataGridViewModel(exports.sampleListData); });
exports.demoRoutingMap.addRoute('webrx-react', 'DataGridAsync', 'Data Grid (Async)', function (state) { return new Components.AsyncDataGridViewModel(sampleDataSource); });
exports.demoRoutingMap.addRoute('webrx-react', 'ItemListPanel', 'Item List Panel', function (state) {
    return new Components.ItemListPanelViewModel(exports.sampleListData, function (x, s) {
        return s.regex.test(x.name);
    });
});
exports.demoRoutingMap.addRoute('webrx-react', 'ItemListPanelList', 'Item List Panel (List)', function (state) {
    return new Components.ItemListPanelViewModel(exports.sampleListData, function (x, s) {
        return s.regex.test(x.name);
    });
});
exports.demoRoutingMap.addRoute('webrx-react', 'ItemListPanelTree', 'Item List Panel (Tree)', function (state) {
    return new Components.TreeItemListPanelViewModel(exports.sampleTreeData, function (x) { return x.items; }, function (x, items) { return cloneSampleTreeItem(x, items); }, function (x, s) { return s.regex.test(x.name); });
});
exports.demoRoutingMap.addRoute('webrx-react', 'AsyncItemListPanel', 'ItemListPanel (Async)', function (state) {
    return new Components.AsyncItemListPanelViewModel(sampleDataSource);
});
exports.demoRoutingMap.addRoute('webrx-react', 'AsyncItemListPanelTree', 'ItemListPanel (Async Tree)', function (state) {
    return new Components.AsyncTreeItemListPanelViewModel(sampleTreeDataSource, function (x) { return x.items; });
});
exports.demoRoutingMap.addRoute('webrx-react', 'InlineEdit', 'InlineEdit', function (state) {
    var editor = new Components.InlineEditViewModel(5);
    editor.save.results
        // handle post-save results
        .subscribe(function (x) {
        Utils_1.Alert.create("Saving value change: " + x, 'Inline Editor Demo');
    });
    return editor;
});
exports.demoRoutingMap.addRoute('webrx-react', 'InlineEditObject', 'InlineEdit (Object)', function (state) {
    var onSave = function (user) {
        if (user.rank === 7) {
            throw new Error('Simulated Coding Error');
        }
        return (rxjs_1.Observable.of(user)
            // simulate network delay
            .delay(1000)
            // simulate an API error
            .map(function (x) {
            if (x.rank === 6) {
                throw new Error('Simulated API Error');
            }
            else if (x.rank === 8) {
                throw new Error('Manual');
            }
            return x;
        })
            .catch(function (x) {
            if (x.message === 'Manual') {
                editor.setError.execute(true);
                Utils_1.Alert.createForError(x, 'Manual Error Handling');
                return rxjs_1.Observable.empty();
            }
            return rxjs_1.Observable.throw(x);
        }));
    };
    var editor = new Components.InlineEditViewModel({ name: 'Some Guy', rank: 5 }, onSave);
    editor.save.results
        // handle post-save results
        .subscribe(function (x) {
        Utils_1.Alert.create("Saving SampleUser Change: " + JSON.stringify(x, null, 2), 'Inline Editor Demo');
    });
    return editor;
});
exports.demoRoutingMap.viewModelMap['help'] = function () { return 'Help'; };
exports.demoRoutingMap.addRoute('webrx-react', 'todolist', 'Todo List', function (state) {
    return new TodoListViewModel_1.TodoListViewModel();
});
// inject the demo infrastructure into the app routing and view maps
Routing_1.RouteMap['/'] = { path: '/demo/' };
Routing_1.RouteMap['^/demo$'] = { path: '/demo/' };
// setup the demo route path pattern
Routing_1.RouteMap['^/demo/(.*)?'] = {
    path: '/demo',
    creator: function () { return new ComponentDemoViewModel_1.ComponentDemoViewModel(exports.demoRoutingMap); },
};
//# sourceMappingURL=RoutingMap.js.map