// tslint:disable:max-classes-per-file
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var rxjs_1 = require("rxjs");
var ItemListPanelViewModel_1 = require("../../Common/ItemListPanel/ItemListPanelViewModel");
var React_1 = require("../../React");
var TodoListStore_1 = require("./TodoListStore");
var TodoListViewModel = /** @class */ (function (_super) {
    tslib_1.__extends(TodoListViewModel, _super);
    function TodoListViewModel(store) {
        if (store === void 0) { store = TodoListStore_1.TodoListStore.default; }
        var _this = _super.call(this) || this;
        _this.store = store;
        _this.newItemContent = _this.wx.property('');
        _this.addItem = _this.wx.command(_this.wx.whenAny(_this.newItemContent, function (x) { return String.isNullOrEmpty(x) === false; }));
        _this.removeItem = _this.wx.command();
        var addedItems = _this.addItem.results
            .debounceTime(100)
            .withLatestFrom(_this.newItemContent.changed, function (_, x) { return x; })
            .filterNull()
            .flatMap(function (x) {
            return _this.wx.getObservableOrAlert(function () { return _this.store.addItem(x); }, 'Error Adding Item');
        })
            .map(function (x) { return new TodoItemViewModel(x); })
            .share();
        var removedItems = _this.removeItem.results
            .debounceTime(100)
            .filterNull()
            .flatMap(function (x) {
            return _this.wx.getObservableOrAlert(function () { return _this.store.removeItem(x.id); }, 'Error Removing Item');
        })
            .share();
        var initialItems = _this.wx
            .getObservableOrAlert(function () { return _this.store.getItems(); }, 'Error Fetching Items')
            .map(function (x) {
            return x.map(function (item) { return new TodoItemViewModel(item); });
        });
        var todoListItems = rxjs_1.Observable.merge(initialItems.map(function (reset) { return ({ reset: reset }); }), addedItems.map(function (add) { return ({ add: add }); }), removedItems.map(function (remove) { return ({ remove: remove }); })).scan(function (items, change) {
            if (change.reset) {
                _this.logger.info("Resetting List: " + change.reset.length + " Items", change);
                return change.reset;
            }
            else if (change.add) {
                _this.logger.info("Adding Item: " + change.add.content, change);
                return items.concat(change.add);
            }
            else if (change.remove) {
                _this.logger.info("Removing Item: " + change.remove, change);
                return items.filter(function (x) { return x.id !== change.remove; });
            }
            return items;
        }, []);
        _this.list = new ItemListPanelViewModel_1.ItemListPanelViewModel(todoListItems, function (x, s) {
            return x.filter(s.regex);
        });
        return _this;
    }
    TodoListViewModel.prototype.isRoutingStateHandler = function () {
        return true;
    };
    TodoListViewModel.prototype.createRoutingState = function (changed) {
        return this.list.createRoutingState(changed);
    };
    TodoListViewModel.prototype.applyRoutingState = function (state) {
        this.list.applyRoutingState(state);
    };
    TodoListViewModel.displayName = 'TodoListViewModel';
    return TodoListViewModel;
}(React_1.BaseViewModel));
exports.TodoListViewModel = TodoListViewModel;
var TodoItemViewModel = /** @class */ (function (_super) {
    tslib_1.__extends(TodoItemViewModel, _super);
    function TodoItemViewModel(arg, store) {
        if (store === void 0) { store = TodoListStore_1.TodoListStore.default; }
        var _this = _super.call(this) || this;
        _this.store = store;
        if (String.isString(arg)) {
            _this.id = TodoItemViewModel.nextId++;
            _this.content = arg;
        }
        else {
            _this.id = arg.id;
            _this.content = arg.content;
        }
        _this.toggleCompleted = _this.wx.command(function (completed) {
            return _this.wx.getObservableOrAlert(function () { return _this.store.setCompleted(_this.id, completed); }, 'Error Changing Completed');
        });
        _this.completed = _this.wx
            .whenAny(_this.toggleCompleted.results, function () { return true; })
            .scan(function (x) { return !x; }, false)
            .toProperty(String.isString(arg) ? false : arg.completed);
        return _this;
    }
    TodoItemViewModel.prototype.filter = function (regexp) {
        return [this.content].some(function (x) { return regexp.test(x); });
    };
    TodoItemViewModel.displayName = 'TodoItemViewModel';
    TodoItemViewModel.nextId = 1;
    return TodoItemViewModel;
}(React_1.BaseViewModel));
exports.TodoItemViewModel = TodoItemViewModel;
//# sourceMappingURL=TodoListViewModel.js.map