/// <reference types="react" />
import * as React from 'react';
import { Observable, Subscription } from 'rxjs';
import { AnonymousSubscription, TeardownLogic } from 'rxjs/Subscription';
import { ReactSpreadRestrictedProps, ReactSpreadResult } from '../../Extensions/React';
import { Alert, Logging } from '../../Utils';
import { Command, IterableLike, Property } from '../../WebRx';
import { BaseViewModel } from './BaseViewModel';
export interface ViewModelProps<T extends BaseViewModel> {
    viewModel: Readonly<T>;
}
export declare type ReactSpreadRestrictedViewModelProps = ReactSpreadRestrictedProps & Partial<ViewModelProps<any>>;
export declare const reactSpreadRestrictedViewModelProps: ReactSpreadRestrictedViewModelProps;
export interface BaseViewProps<TViewModel extends BaseViewModel, TView = any> extends ViewModelProps<TViewModel>, React.HTMLProps<TView> {
}
export interface ViewModelState<T extends BaseViewModel> {
    viewModel: T;
}
export declare abstract class BaseView<TViewProps extends ViewModelProps<TViewModel>, TViewModel extends BaseViewModel> extends React.Component<TViewProps, ViewModelState<TViewModel>> implements AnonymousSubscription {
    static displayName: string;
    private updateSubscription;
    private subscriptions;
    protected readonly createAlert: typeof Alert.create;
    protected readonly alertForError: typeof Alert.createForError;
    protected readonly logger: Logging.Logger;
    constructor(props: any);
    readonly viewModel: Readonly<TViewModel>;
    private getSubscriptions();
    componentWillMount(): void;
    componentDidMount(): void;
    componentWillReceiveProps(nextProps: Readonly<TViewProps>, nextContext: any): void;
    componentWillUpdate(nextProps: Readonly<TViewProps>, nextState: Readonly<ViewModelState<TViewModel>>, nextContext: any): void;
    componentDidUpdate(prevProps: Readonly<TViewProps>, prevState: Readonly<ViewModelState<TViewModel>>, prevContext: any): void;
    componentWillUnmount(): void;
    private initializeView();
    private loadedView();
    private updatingView(nextProps, nextState);
    private updatedView(prevProps, prevState);
    private cleanupView();
    protected initialize(): void;
    protected loaded(): void;
    protected updating(nextProps?: Readonly<TViewProps>, nextState?: Readonly<ViewModelState<TViewModel>>): void;
    protected updated(prevProps?: Readonly<TViewProps>, prevState?: Readonly<ViewModelState<TViewModel>>): void;
    protected cleanup(): void;
    protected subscribeToUpdates(props: Readonly<TViewProps>, state: Readonly<ViewModelState<TViewModel>>): void;
    protected updateOn(viewModel: Readonly<TViewModel>): IterableLike<Observable<any>>;
    protected getDisplayName(): string;
    protected getRateLimit(): number;
    protected renderView(): void;
    /**
     * Binds an observable to a command on the view model
     */
    protected bindObservableToCommand<TInput, TResult>(observable: Observable<TInput>, commandSelector: (viewModel: Readonly<TViewModel>) => Command<TResult>, onNext?: (value: TInput) => void, onError?: (exception: any) => void, onCompleted?: () => void): Subscription;
    /**
     * Binds a DOM event to an observable property on the view model
     */
    protected bindEventToProperty<TValue, TEvent extends Event | React.SyntheticEvent<this>>(targetSelector: (viewModel: Readonly<TViewModel>) => Property<TValue>, valueSelector?: (eventKey: any, event: TEvent) => TValue): any;
    /**
     * Binds a DOM event to an observable command on the view model
     */
    protected bindEventToCommand<TParameter, TCommand, TEvent extends Event | React.SyntheticEvent<this>>(commandSelector: (viewModel: Readonly<TViewModel>) => Command<TCommand>, paramSelector?: (eventKey: any, event: TEvent) => TParameter, conditionSelector?: (event: TEvent, eventKey: any) => boolean, onNext?: (value: TCommand) => void, onError?: (exception: any) => void, onCompleted?: () => void): any;
    protected createStateFromProps(props: Readonly<TViewProps>): Readonly<ViewModelState<TViewModel>>;
    protected getViewModelFromState(state: Readonly<ViewModelState<TViewModel>>): Readonly<TViewModel>;
    protected addSubscription<T extends TeardownLogic>(subscription: T): T;
    protected addSubscriptions<T extends TeardownLogic>(...subscriptions: T[]): T[];
    unsubscribe(): void;
    restProps<T, R extends ReactSpreadRestrictedProps = ReactSpreadRestrictedViewModelProps>(propsCreator?: (x: TViewProps) => T, restrictedProps?: R): ReactSpreadResult<TViewProps, T, R>;
}
