Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var ix_1 = require("ix");
var React = require("react");
var rxjs_1 = require("rxjs");
var React_1 = require("../../Extensions/React");
var Utils_1 = require("../../Utils");
var BaseViewModel_1 = require("./BaseViewModel");
var BindingHelpers_1 = require("./BindingHelpers");
exports.reactSpreadRestrictedViewModelProps = Object.assign({ viewModel: undefined }, React_1.reactRestrictedProps);
var BaseView = /** @class */ (function (_super) {
    tslib_1.__extends(BaseView, _super);
    function BaseView(props) {
        var _this = _super.call(this, props) || this;
        // these are Alert helper functions
        _this.createAlert = Utils_1.Alert.create;
        _this.alertForError = Utils_1.Alert.createForError;
        _this.logger = Utils_1.Logging.getLogger(_this.getDisplayName());
        _this.updateSubscription = _this.subscriptions = rxjs_1.Subscription.EMPTY;
        _this.subscriptions = new rxjs_1.Subscription();
        if (props != null) {
            _this.state = _this.createStateFromProps(props);
        }
        return _this;
    }
    Object.defineProperty(BaseView.prototype, "viewModel", {
        get: function () {
            return this.getViewModelFromState(this.state);
        },
        enumerable: true,
        configurable: true
    });
    BaseView.prototype.getSubscriptions = function () {
        if (this.subscriptions === rxjs_1.Subscription.EMPTY) {
            this.subscriptions = new rxjs_1.Subscription();
        }
        return this.subscriptions;
    };
    // -----------------------------------------
    // these are react lifecycle functions
    // -----------------------------------------
    BaseView.prototype.componentWillMount = function () {
        this.initializeView();
        this.subscribeToUpdates(this.props, this.state);
        this.logger.debug('rendering');
    };
    BaseView.prototype.componentDidMount = function () {
        this.loadedView();
    };
    BaseView.prototype.componentWillReceiveProps = function (nextProps, nextContext) {
        // create the next state
        var nextState = this.createStateFromProps(nextProps);
        // get the next view model
        var nextViewModel = this.getViewModelFromState(nextState);
        // if the view model changed we need to do some teardown and setup
        if (nextViewModel !== this.viewModel) {
            this.logger.info('ViewModel Change Detected');
            // unsubscribe from updates
            this.updateSubscription = rxjs_1.Subscription.unsubscribe(this.updateSubscription);
            // ask react to generate new state from the updated props
            this.setState(function (prevState, props) {
                return nextState;
            });
        }
    };
    BaseView.prototype.componentWillUpdate = function (nextProps, nextState, nextContext) {
        this.updatingView(nextProps, nextState);
        // get the next view model
        var nextViewModel = this.getViewModelFromState(nextState);
        // check if we need to re-subscripe to updates (if our view model changed)
        if (nextViewModel !== this.viewModel) {
            // cleanup the old view model
            if (BaseViewModel_1.isViewModelLifecycle(this.viewModel)) {
                this.viewModel.cleanupViewModel();
            }
            // then initialize the view model
            if (BaseViewModel_1.isViewModelLifecycle(nextViewModel)) {
                nextViewModel.initializeViewModel();
            }
            // now sub to the view model observables
            this.subscribeToUpdates(nextProps, nextState);
            if (BaseViewModel_1.isViewModelLifecycle(nextViewModel)) {
                // finally inform the view model it has been (re-)loaded
                nextViewModel.loadedViewModel();
            }
        }
        this.logger.debug('re-rendering');
    };
    BaseView.prototype.componentDidUpdate = function (prevProps, prevState, prevContext) {
        this.updatedView(prevProps, prevState);
    };
    BaseView.prototype.componentWillUnmount = function () {
        this.cleanupView();
        this.unsubscribe();
    };
    // -----------------------------------------
    // -----------------------------------------
    // these are internal lifecycle functions
    // -----------------------------------------
    BaseView.prototype.initializeView = function () {
        if (BaseViewModel_1.isViewModelLifecycle(this.viewModel)) {
            this.viewModel.initializeViewModel();
        }
        this.initialize();
    };
    BaseView.prototype.loadedView = function () {
        this.loaded();
        if (BaseViewModel_1.isViewModelLifecycle(this.viewModel)) {
            this.viewModel.loadedViewModel();
        }
    };
    BaseView.prototype.updatingView = function (nextProps, nextState) {
        this.updating(nextProps, nextState);
    };
    BaseView.prototype.updatedView = function (prevProps, prevState) {
        this.updated(prevProps, prevState);
    };
    BaseView.prototype.cleanupView = function () {
        this.cleanup();
        if (BaseViewModel_1.isViewModelLifecycle(this.viewModel)) {
            this.viewModel.cleanupViewModel();
        }
    };
    // -----------------------------------------
    // -----------------------------------------
    // these are overridable lifecycle functions
    // -----------------------------------------
    BaseView.prototype.initialize = function () {
        // do nothing by default
    };
    BaseView.prototype.loaded = function () {
        // do nothing by default
    };
    BaseView.prototype.updating = function (nextProps, nextState) {
        // do nothing by default
    };
    BaseView.prototype.updated = function (prevProps, prevState) {
        // do nothing by default
    };
    BaseView.prototype.cleanup = function () {
        // do nothing by default
    };
    // -----------------------------------------
    BaseView.prototype.subscribeToUpdates = function (props, state) {
        var _this = this;
        var viewModel = this.getViewModelFromState(state);
        var updateProps = ix_1.Iterable.from(this.updateOn(viewModel)).toArray();
        if (updateProps.length > 0) {
            this.updateSubscription = rxjs_1.Observable.merge.apply(rxjs_1.Observable, tslib_1.__spread(updateProps)).debounceTime(this.getRateLimit())
                .subscribe(function () {
                _this.renderView();
            }, function (x) {
                _this.alertForError(x);
            });
        }
    };
    // -----------------------------------------
    // these overridable view functions
    // -----------------------------------------
    BaseView.prototype.updateOn = function (viewModel) {
        return [];
    };
    BaseView.prototype.getDisplayName = function () {
        return Object.getName(this);
    };
    BaseView.prototype.getRateLimit = function () {
        return 100;
    };
    BaseView.prototype.renderView = function () {
        this.forceUpdate();
    };
    // -----------------------------------------
    // -----------------------------------------
    // these are binding helper methods
    // -----------------------------------------
    /**
     * Binds an observable to a command on the view model
     */
    BaseView.prototype.bindObservableToCommand = function (observable, commandSelector, onNext, onError, onCompleted) {
        return BindingHelpers_1.bindObservableToCommand(this.viewModel, observable, commandSelector, onNext, onError, onCompleted);
    };
    /**
     * Binds a DOM event to an observable property on the view model
     */
    BaseView.prototype.bindEventToProperty = function (targetSelector, valueSelector) {
        return BindingHelpers_1.bindEventToProperty(this.viewModel, targetSelector, valueSelector);
    };
    /**
     * Binds a DOM event to an observable command on the view model
     */
    BaseView.prototype.bindEventToCommand = function (commandSelector, paramSelector, conditionSelector, onNext, onError, onCompleted) {
        return BindingHelpers_1.bindEventToCommand(this.viewModel, commandSelector, paramSelector, conditionSelector, onNext, onError, onCompleted);
    };
    // -----------------------------------------
    BaseView.prototype.createStateFromProps = function (props) {
        return {
            viewModel: props.viewModel,
        };
    };
    BaseView.prototype.getViewModelFromState = function (state) {
        return state.viewModel;
    };
    BaseView.prototype.addSubscription = function (subscription) {
        return this.getSubscriptions().addSubscription(subscription);
    };
    BaseView.prototype.addSubscriptions = function () {
        var subscriptions = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            subscriptions[_i] = arguments[_i];
        }
        return (_a = this.getSubscriptions()).addSubscriptions.apply(_a, tslib_1.__spread(subscriptions));
        var _a;
    };
    BaseView.prototype.unsubscribe = function () {
        this.updateSubscription = rxjs_1.Subscription.unsubscribe(this.updateSubscription);
        this.subscriptions = rxjs_1.Subscription.unsubscribe(this.subscriptions);
    };
    // -----------------------------------------
    // this is the property destruction helper
    // this functions will remove key, ref, and viewModel props automatically
    // -----------------------------------------
    BaseView.prototype.restProps = function (propsCreator, restrictedProps) {
        if (restrictedProps == null) {
            restrictedProps = Object.assign({ viewModel: undefined }, React_1.reactRestrictedProps);
        }
        return _super.prototype.restProps.call(this, propsCreator, restrictedProps);
    };
    BaseView.displayName = 'BaseView';
    return BaseView;
}(React.Component));
exports.BaseView = BaseView;
//# sourceMappingURL=BaseView.js.map