Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var rxjs_1 = require("rxjs");
var Events_1 = require("../../Events");
var RouteManager_1 = require("../../Routing/RouteManager");
var Utils_1 = require("../../Utils");
var Logging_1 = require("../../Utils/Logging");
var WebRx_1 = require("../../WebRx");
function isRoutingStateHandler(value) {
    if (value == null) {
        return false;
    }
    var handler = value;
    return (handler.isRoutingStateHandler instanceof Function &&
        handler.isRoutingStateHandler());
}
exports.isRoutingStateHandler = isRoutingStateHandler;
function isViewModelLifecycle(viewModel) {
    return (viewModel.initializeViewModel instanceof Function &&
        viewModel.loadedViewModel instanceof Function &&
        viewModel.updatedViewModel instanceof Function &&
        viewModel.cleanupViewModel instanceof Function);
}
exports.isViewModelLifecycle = isViewModelLifecycle;
function isViewModel(source) {
    if (source == null) {
        return false;
    }
    var viewModel = source;
    if (viewModel.isViewModel instanceof Function) {
        return viewModel.isViewModel();
    }
    return false;
}
exports.isViewModel = isViewModel;
function getRoutingStateValue(value, arg2, selector) {
    if (value == null) {
        return undefined;
    }
    if (arg2 instanceof Function) {
        return arg2(value);
    }
    if (arg2 != null && value === arg2) {
        return undefined;
    }
    if (selector != null) {
        return selector(value);
    }
    // we need a direct cast here because no selector was supplied
    // this just means that T === R
    return value;
}
exports.getRoutingStateValue = getRoutingStateValue;
var BaseViewModel = /** @class */ (function (_super) {
    tslib_1.__extends(BaseViewModel, _super);
    function BaseViewModel(unsubscribe) {
        var _this = _super.call(this, unsubscribe) || this;
        _this.logger = Logging_1.getLogger(_this.getDisplayName());
        _this.wx = WebRx_1.wx;
        _this.getRoutingStateValue = getRoutingStateValue;
        _this.createAlert = Utils_1.Alert.create;
        _this.alertForError = Utils_1.Alert.createForError;
        _this.routeManager = RouteManager_1.routeManager;
        _this.isLoggingMemberObservables = false;
        return _this;
    }
    // -----------------------------------------
    // These are special methods that handle the
    // lifecycle internally (do not override!!!)
    // -----------------------------------------
    BaseViewModel.prototype.initializeViewModel = function () {
        this.initialize();
        this.initializeRoutingStateHandler(this);
        if (this.logger.level <= Logging_1.LogLevel.Debug &&
            this.isLoggingMemberObservables === false) {
            this.isLoggingMemberObservables = true;
            this.addSubscriptions.apply(this, tslib_1.__spread(this.wx.logMemberObservables(this.logger, this)));
        }
    };
    BaseViewModel.prototype.loadedViewModel = function () {
        this.loaded();
    };
    BaseViewModel.prototype.updatedViewModel = function () {
        this.updated();
    };
    BaseViewModel.prototype.cleanupViewModel = function () {
        this.cleanup();
    };
    BaseViewModel.prototype.initializeRoutingStateHandler = function (source) {
        if (isRoutingStateHandler(source)) {
            this.stateChanged = this.wx.command(function (context) {
                var stateChanged = {
                    source: source,
                    context: context,
                };
                Utils_1.PubSub.publish(Events_1.RoutingStateChangedKey, stateChanged);
                return stateChanged;
            });
        }
    };
    BaseViewModel.prototype.initialize = function () {
        // do nothing by default
    };
    BaseViewModel.prototype.loaded = function () {
        // do nothing by default
    };
    BaseViewModel.prototype.updated = function () {
        // do nothing by default
    };
    BaseViewModel.prototype.cleanup = function () {
        // do nothing by default
    };
    BaseViewModel.prototype.notifyChanged = function (context) {
        if (this.stateChanged != null) {
            this.stateChanged.execute(context);
        }
    };
    BaseViewModel.prototype.navTo = function (path, state, replace, uriEncode) {
        if (replace === void 0) { replace = false; }
        if (uriEncode === void 0) { uriEncode = false; }
        this.routeManager.navTo(path, state, replace, uriEncode);
    };
    BaseViewModel.prototype.isViewModel = function () {
        return true;
    };
    BaseViewModel.prototype.getDisplayName = function () {
        return Object.getName(this);
    };
    BaseViewModel.displayName = 'BaseViewModel';
    // WebRx helper functions (so you don't need to import them every time)
    BaseViewModel.wx = WebRx_1.wx;
    return BaseViewModel;
}(rxjs_1.Subscription));
exports.BaseViewModel = BaseViewModel;
//# sourceMappingURL=BaseViewModel.js.map