Object.defineProperty(exports, "__esModule", { value: true });
var WebRx_1 = require("../../WebRx");
/**
 * Binds an observable to a command on the view model
 */
function bindObservableToCommand(viewModel, observableLike, commandSelector, onNext, onError, onCompleted) {
    return viewModel.addSubscription(WebRx_1.getObservable(observableLike).invokeCommand(commandSelector(viewModel), onNext, onError, onCompleted));
}
exports.bindObservableToCommand = bindObservableToCommand;
/**
 * Binds a DOM event to an observable property on the view model
 */
function bindEventToProperty(viewModel, targetSelector, valueSelector) {
    // this needs to be any instead of Function to support React.EventHandler<T>
    return function (eventKey, event) {
        // this ensures that we can still use this function for basic HTML events
        event = event || eventKey;
        var prop = targetSelector(viewModel);
        var value = valueSelector == null ? eventKey : valueSelector(eventKey, event);
        prop.value = value;
    };
}
exports.bindEventToProperty = bindEventToProperty;
/**
 * Binds a DOM event to an observable command on the view model
 */
function bindEventToCommand(viewModel, commandSelector, paramSelector, conditionSelector, onNext, onError, onCompleted) {
    // this needs to be any instead of Function to support React.EventHandler<T>
    return function (eventKey, event) {
        // this ensures that we can still use this function for basic HTML events
        event = event || eventKey;
        var param = paramSelector == null ? eventKey : paramSelector(eventKey, event);
        var canExecute = conditionSelector == null || conditionSelector(event, eventKey);
        if (canExecute) {
            var cmd = commandSelector(viewModel);
            if (WebRx_1.isCommand(cmd)) {
                cmd.execute(param, onNext, onError, onCompleted);
            }
        }
    };
}
exports.bindEventToCommand = bindEventToCommand;
//# sourceMappingURL=BindingHelpers.js.map