Object.defineProperty(exports, "__esModule", { value: true });
var classNamesFunc = require("classnames");
var ix_1 = require("ix");
var react_dom_1 = require("react-dom");
var Compare_1 = require("../../Utils/Compare");
function renderIterable(source, selector, projector, sortKey, sortComparer, defaultSelector) {
    if (selector === void 0) { selector = function (item) {
        return item;
    }; }
    if (projector === void 0) { projector = function (items) { return items; }; }
    if (sortComparer === void 0) { sortComparer = Compare_1.ValueComparer.DefaultComparison; }
    if (defaultSelector === void 0) { defaultSelector = function () { return undefined; }; }
    if (source == null) {
        return projector([]);
    }
    var iterable = ix_1.Iterable.from(source);
    if (sortKey != null) {
        iterable = iterable.orderBy(sortKey, sortComparer);
    }
    return projector(iterable
        .defaultIfEmpty(defaultSelector())
        .filterNull()
        .toArray()
        .map(function (x, i, a) { return selector(x, i, a); }));
}
exports.renderIterable = renderIterable;
function renderConditional(condition, trueContent, falseContent) {
    if (falseContent === void 0) { falseContent = function () { return false; }; }
    return (condition == null
        ? false
        : typeof condition === 'boolean'
            ? condition
            : condition.value)
        ? trueContent()
        : falseContent();
}
exports.renderConditional = renderConditional;
function renderNullable(item, notNullContent, nullContent, constraint) {
    return renderConditional(item != null && (constraint == null || constraint(item)), function () { return notNullContent(item); }, nullContent);
}
exports.renderNullable = renderNullable;
// focus the provided element (if it is focusable) and optionally perform an action on that element after focus
function focusElement(instance, action) {
    if (instance) {
        var elem = react_dom_1.findDOMNode(instance);
        if (elem && elem.focus instanceof Function) {
            elem.focus();
            if (action) {
                action(elem);
            }
        }
    }
}
exports.focusElement = focusElement;
// classNames type defs don't export the internal types so we have to decouple here
function classNames() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    return classNamesFunc.apply(this, arguments);
}
exports.classNames = classNames;
//# sourceMappingURL=RenderHelpers.js.map