Object.defineProperty(exports, "__esModule", { value: true });
var ix_1 = require("ix");
function isObject(value) {
    return typeof value === 'object' || value instanceof Object;
}
exports.isObject = isObject;
function trim(obj, trimNull) {
    if (trimNull === void 0) { trimNull = true; }
    if (isObject(obj)) {
        return ix_1.Iterable.from(Object.keys(obj)).reduce(function (o, x) {
            if ((trimNull && o[x] === null) || o[x] === undefined) {
                delete o[x];
            }
            return o;
        }, obj);
    }
    return obj;
}
exports.trim = trim;
// this extension solves the Unknown Prop Warning that is experienced in
// typescript when using Rest and Spread Properties
// see: https://facebook.github.io/react/warnings/unknown-prop.html
// see: https://facebook.github.io/react/docs/transferring-props.html
function rest(data, propsCreator, removals) {
    var map = {
        data: data || {},
        props: (propsCreator == null ? undefined : propsCreator(data)) || {},
        removals: removals || {},
    };
    var result = {
        rest: {},
        removals: {},
    };
    var restParams = Object.keys(data).forEach(function (key) {
        if (map.removals.hasOwnProperty(key)) {
            result.removals[key] = map.data[key];
        }
        else if (map.props.hasOwnProperty(key) === false) {
            result.rest[key] = map.data[key];
        }
    });
    return {
        props: map.props,
        rest: result.rest,
        removals: result.removals,
    };
}
exports.rest = rest;
function getName(source, undefinedValue, isStatic) {
    if (undefinedValue === void 0) { undefinedValue = 'undefined'; }
    if (isStatic === void 0) { isStatic = false; }
    if (source == null) {
        return undefinedValue;
    }
    // first check if we're a string
    if (String.isString(source)) {
        return source;
    }
    // check if this is a function and try and extract the name directly
    if (source instanceof Function && String.isString(source.name)) {
        return source.name;
    }
    var nameSource = source;
    // now check the instance sources
    if (!String.isNullOrEmpty(nameSource.displayName)) {
        return nameSource.displayName;
    }
    if (!String.isNullOrEmpty(nameSource.typeName)) {
        return nameSource.typeName;
    }
    if (!String.isNullOrEmpty(nameSource.name)) {
        return nameSource.name;
    }
    // then check the static sources
    if (nameSource.constructor != null) {
        if (!String.isNullOrEmpty(nameSource.constructor.displayName)) {
            return nameSource.constructor.displayName;
        }
        if (!String.isNullOrEmpty(nameSource.constructor.typeName)) {
            return nameSource.constructor.typeName;
        }
        if (!String.isNullOrEmpty(nameSource.constructor.name)) {
            return nameSource.constructor.name;
        }
    }
    // try and extract the name from the toString() reprenstation
    // IE is pretty dumb and doesn't expose any useful naming properties
    // but this seems to work reliably for some objects
    if (nameSource.constructor != null) {
        var match = /function (.+)\(/.exec(nameSource.constructor.toString());
        if (match != null && match.length >= 2) {
            if (String.isNullOrEmpty(match[1]) === false) {
                return match[1];
            }
        }
    }
    // finally fallback onto simple toString()
    return nameSource.toString();
}
exports.getName = getName;
function getEnumPropertyDescriptors(type) {
    return Object.keys(type)
        .asIterable()
        .map(function (name) { return ({ name: name, value: parseInt(type[name]) }); })
        .filter(function (x) { return x.value >= 0; })
        .map(function (x) { return Object.assign(x, { type: type[x.name] }); });
}
exports.getEnumPropertyDescriptors = getEnumPropertyDescriptors;
function getEnumNames(type) {
    return getEnumPropertyDescriptors(type)
        .map(function (x) { return x.name; })
        .toArray();
}
exports.getEnumNames = getEnumNames;
function getEnumValues(type) {
    return getEnumPropertyDescriptors(type)
        .map(function (x) { return x.type; })
        .toArray();
}
exports.getEnumValues = getEnumValues;
/**
 * Get object property name
 * see: http://stackoverflow.com/questions/37048274/typescript-how-to-get-objects-property-name-from-its-value
 * @param {p: (x: T) => any} function which returns any object property
 * @returns {string} property name
 */
function getPropName(p) {
    return (/\.([^\.;]+);?\s*\}$/.exec(p.toString()) || [])[1];
}
exports.getPropName = getPropName;
Object.isObject = isObject;
Object.trim = trim;
Object.rest = rest;
Object.getName = getName;
Object.getEnumPropertyDescriptors = getEnumPropertyDescriptors;
Object.getEnumNames = getEnumNames;
Object.getEnumValues = getEnumValues;
Object.getPropName = getPropName;
//# sourceMappingURL=Object.js.map