// tslint:disable:no-shadowed-variable
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
function trimPropsStatic(props) {
    return Object.trim(props, false);
}
exports.trimPropsStatic = trimPropsStatic;
exports.reactRestrictedProps = {
    key: undefined,
    ref: undefined,
    className: undefined,
    children: undefined,
};
// this extension makes 'resting' React props much easier
// i.e.,
// const { className, children, props, rest } = this.restProps(x => {
//   const { header, footer } = x;
//   return { header, footer };
// }, 'exclude1', 'exclude2');
// You may omit any of the className, children, props, or rest props from the return value
// you may additionally choose to omit any properties by name from the rest
// object that is returned (like 'children' for example).
function restPropsStatic(props, propsCreator, restrictedProps) {
    if (restrictedProps == null) {
        restrictedProps = exports.reactRestrictedProps;
    }
    var result = Object.rest(props, propsCreator, restrictedProps);
    return {
        className: result.removals.className,
        children: result.removals.children,
        props: result.props,
        rest: result.rest,
        removals: result.removals,
    };
}
exports.restPropsStatic = restPropsStatic;
function restProps(propsCreator, restrictedProps) {
    return restPropsStatic(this.props, propsCreator, restrictedProps);
}
exports.restProps = restProps;
function isTypeStatic(elem, type) {
    if (WEBPACK_DEV_SERVER) {
        // react-hot-loader injects proxy types so we cannot perform standard type checking while using it
        // this branch allows us to perform a less efficient type check that works with react-hot-loader.
        return Object.getName(elem.type) === Object.getName(type);
    }
    return elem.type === type;
}
exports.isTypeStatic = isTypeStatic;
function isValidTypeStatic(value, type) {
    if (!React.isValidElement(value)) {
        return false;
    }
    return isTypeStatic(value, type);
}
exports.isValidTypeStatic = isValidTypeStatic;
React.Component.prototype.trimProps = trimPropsStatic;
React.Component.prototype.restProps = restProps;
React.Component.trimProps = trimPropsStatic;
React.Component.restProps = restPropsStatic;
React.isType = isTypeStatic;
React.isValidType = isValidTypeStatic;
//# sourceMappingURL=React.js.map