Object.defineProperty(exports, "__esModule", { value: true });
var rxjs_1 = require("rxjs");
var WebRx_1 = require("../WebRx");
// this is a more comprehensive hash manager that does support history replacement
// however, this hash manager requires at least IE11
// if support is not detected, then the window location hash manager is used instead
var HistoryStateHashManager = /** @class */ (function () {
    function HistoryStateHashManager(fallbackManager) {
        this.fallbackManager = fallbackManager;
        this.changeHash = WebRx_1.wx.command();
    }
    HistoryStateHashManager.prototype.updateHash = function (hash, state, title, replace) {
        if (history != null &&
            history.replaceState instanceof Function &&
            history.pushState instanceof Function) {
            if (replace === true) {
                history.replaceState(state, title, hash);
            }
            else {
                history.pushState(state, title, hash);
            }
            this.changeHash.execute(hash);
        }
        else {
            // history API not supported, fallback onto window location hash manager
            this.fallbackManager.updateHash(hash, state, title, replace);
        }
    };
    Object.defineProperty(HistoryStateHashManager.prototype, "hashChanged", {
        get: function () {
            return (rxjs_1.Observable
                // we need to merge both hash change requests (from updateHash)
                // as well as manual URI hash changes from the window location hash manager
                .merge(this.changeHash.results, this.fallbackManager.hashChanged));
        },
        enumerable: true,
        configurable: true
    });
    return HistoryStateHashManager;
}());
exports.HistoryStateHashManager = HistoryStateHashManager;
// this is a low level fallback hash manager (in case we can't construct a real one)
exports.nullHashManager = {
    hashChanged: rxjs_1.Observable.never(),
    updateHash: function () {
        // do nothing
    },
};
// this is a very basic hash manager, it does not support history replacement
// it's possible that the environment doesn't support `window` so we need to check it first
exports.windowLocationHashManager = typeof window === 'undefined'
    ? undefined
    : {
        hashChanged: rxjs_1.Observable.fromEvent(window, 'hashchange').map(function () { return window.location.hash; }),
        updateHash: function (hash) {
            window.location.hash = hash;
        },
    };
exports.historyStateHashManager = new HistoryStateHashManager(exports.windowLocationHashManager || exports.nullHashManager);
//# sourceMappingURL=HashManager.js.map