Object.defineProperty(exports, "__esModule", { value: true });
var HashCodec_1 = require("./HashCodec");
function getWindowLocation() {
    if (typeof window === 'undefined') {
        return undefined;
    }
    return (window || {}).location;
}
exports.getWindowLocation = getWindowLocation;
function getPath(state) {
    var path;
    if (state != null &&
        state.route != null &&
        String.isNullOrEmpty(state.route.path) === false) {
        path = state.route.path;
        delete state.route;
    }
    return path;
}
exports.getPath = getPath;
function trimPath(path, removeLeadingSlash) {
    if (removeLeadingSlash === void 0) { removeLeadingSlash = false; }
    return path
        .replace(/^\/+/, removeLeadingSlash ? '' : '/')
        .replace(/\/+$/, '');
}
exports.trimPath = trimPath;
function joinPath(path, base) {
    if (String.isNullOrEmpty(base)) {
        return trimPath(path);
    }
    return trimPath(base) + "/" + trimPath(path, true);
}
exports.joinPath = joinPath;
function normalizePath(path, currentPath, hashCodec) {
    if (String.isNullOrEmpty(path) === false) {
        if (path[0] !== '/') {
            if (String.isNullOrEmpty(currentPath)) {
                var windowLocation = getWindowLocation();
                var currentHash = windowLocation == null ? undefined : windowLocation.hash;
                currentPath = (hashCodec || HashCodec_1.HashCodec.Default).decode(currentHash, function (x) { return x; });
            }
            // relative path
            path = currentPath
                .split('/')
                .slice(0, -1)
                .join('/') + "/" + path;
        }
        // manage relative path elements (..)
        var pathElems = path.split('/');
        for (var i = 0; i < pathElems.length; ++i) {
            if (pathElems[i] === '..') {
                if (i === 0) {
                    pathElems.shift();
                    --i;
                }
                else {
                    pathElems.splice(i - 1, 2);
                    i -= 2;
                }
            }
            else if (pathElems[i] === '.') {
                pathElems.splice(i--, 1);
            }
            else if (pathElems[i] === '') {
                // trim out empty path elements (except for the beginning and end)
                if (i > 0 && i < pathElems.length - 1) {
                    pathElems.splice(i--, 1);
                }
            }
        }
        path = pathElems.join('/');
    }
    return path;
}
exports.normalizePath = normalizePath;
//# sourceMappingURL=Helpers.js.map