Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var rxjs_1 = require("rxjs");
var Utils_1 = require("../Utils");
var HashCodec_1 = require("./HashCodec");
var HashManager_1 = require("./HashManager");
var Helpers_1 = require("./Helpers");
var RouteManager = /** @class */ (function (_super) {
    tslib_1.__extends(RouteManager, _super);
    function RouteManager(hashManager, hashCodec) {
        if (hashCodec === void 0) { hashCodec = HashCodec_1.HashCodec.Default; }
        var _this = _super.call(this) || this;
        _this.hashCodec = hashCodec;
        _this.logger = Utils_1.Logging.getLogger(RouteManager.displayName);
        if (hashManager == null) {
            hashManager = HashManager_1.historyStateHashManager;
        }
        _this.hashManager = hashManager;
        var routingEvents = _this.hashManager.hashChanged
            .startWith(window.location.hash)
            .distinctUntilChanged()
            .map(function (x) {
            var change = {
                route: hashCodec.decode(x, function (path, params, state) { return ({ path: path, params: params, state: state }); }),
                redirect: false,
            };
            // reconstruct the route hash
            var hash = '#' + change.route.path;
            if (change.route.params && change.route.params.length > 0) {
                hash += change.route.params;
            }
            // if the reconstructed route hash differs from the current hash
            // re-route to the constructed hash so that we have a consistent route
            // NOTE: this can happen if the passed in route is not sanitized, as the
            //       decode function will sanitize input
            // i.e., if no route is supplied (no hash) then a default route of #/ is
            //       coerced from the decode function.
            if (hash !== x) {
                change.redirect = true;
            }
            return change;
        })
            // we must use the async scheduler to prevent recursive re-entrancy calls
            // re-entrancy can occur on redirects, we want each new route to be handled
            // as its own routing state change in sequence, which the async scheduler permits.
            .observeOn(rxjs_1.Scheduler.async)
            .share();
        _this.currentRoute = routingEvents
            .filter(function (x) { return x.redirect === false; })
            .map(function (x) { return x.route; })
            .toProperty();
        _this.addSubscription((_this.redirectSubscription = routingEvents
            .filter(function (x) { return x.redirect; })
            .subscribe(function (x) {
            _this.navTo(x.route.path, x.route.state, true);
        })));
        return _this;
    }
    RouteManager.prototype.navTo = function (path, state, replace, uriEncode) {
        if (replace === void 0) { replace = false; }
        if (uriEncode === void 0) { uriEncode = false; }
        path = Helpers_1.getPath(state) || path;
        if (String.isNullOrEmpty(path) === false) {
            if (path[0] === '#') {
                path = path.substring(1);
            }
            path = Helpers_1.normalizePath(path, undefined, this.hashCodec);
            var hash = this.hashCodec.encode(path, state, uriEncode);
            this.logger.debug("Routing to Hash: " + hash, state);
            if (state != null) {
                this.logger.debug(JSON.stringify(state, null, 2));
            }
            this.hashManager.updateHash(hash, state, '', replace);
        }
    };
    RouteManager.displayName = 'RouteManager';
    return RouteManager;
}(rxjs_1.Subscription));
exports.RouteManager = RouteManager;
exports.routeManager = new RouteManager();
//# sourceMappingURL=RouteManager.js.map