Object.defineProperty(exports, "__esModule", { value: true });
var rxjs_1 = require("rxjs");
var Routing_1 = require("../Routing");
var Logging_1 = require("../Utils/Logging");
var WebRx_1 = require("../WebRx");
var Helpers_1 = require("./Helpers");
var Interfaces_1 = require("./Interfaces");
var ObservableApi = /** @class */ (function () {
    function ObservableApi(path, baseOrSampleData, sampleDataCreator) {
        var windowLocation = Routing_1.getWindowLocation() || {};
        this.logger = Logging_1.getLogger(ObservableApi.displayName);
        this.wx = WebRx_1.wx;
        if (WEBPACK_DEV_SERVER) {
            this.sampleDataCreator = sampleDataCreator;
        }
        if (baseOrSampleData instanceof Function) {
            if (WEBPACK_DEV_SERVER) {
                this.sampleDataCreator = baseOrSampleData;
            }
            baseOrSampleData = undefined;
        }
        if (String.isNullOrEmpty(baseOrSampleData)) {
            this.base =
                (windowLocation.origin || 'http://localhost') +
                    (windowLocation.pathname || '/');
        }
        else {
            this.base = baseOrSampleData;
        }
        if (WEBPACK_DEV_SERVER) {
            this.path = Routing_1.joinPath(path, 'sample');
        }
        else {
            this.path = path;
        }
        this.logger.name += " (" + this.path + ")";
        this.baseUri = Routing_1.joinPath(this.path, this.base);
    }
    ObservableApi.prototype.getSampleData = function () {
        if (this.sampleData == null && this.sampleDataCreator != null) {
            this.sampleData = this.sampleDataCreator();
        }
        return this.sampleData;
    };
    ObservableApi.prototype.getRequestUri = function (action, baseUri) {
        return Routing_1.joinPath(action, baseUri || this.baseUri);
    };
    ObservableApi.prototype.observe = function (action, params, data, method, options, baseUri) {
        var _this = this;
        var sampleData = this.getSampleData();
        return rxjs_1.Observable.defer(function () {
            // use sampleData if it has been defined
            return sampleData == null
                ? Helpers_1.getRequest(action, _this.getRequestUri(action, baseUri), _this.logger, method, params, data, options)
                : sampleData.observe(action, params, data);
        });
    };
    ObservableApi.prototype.getObservable = function (action, params, options, baseUri) {
        return this.observe(action, params, undefined, Interfaces_1.HttpRequestMethod.GET, options, baseUri);
    };
    ObservableApi.prototype.postObservable = function (action, data, params, options, baseUri) {
        return this.observe(action, params, data, Interfaces_1.HttpRequestMethod.POST, options, baseUri);
    };
    ObservableApi.prototype.getSampleStoreValue = function (name, selector) {
        if (this.sampleData == null) {
            return undefined;
        }
        return this.sampleData.getStoreValue(name, selector);
    };
    ObservableApi.displayName = 'ObservableApi';
    return ObservableApi;
}());
exports.ObservableApi = ObservableApi;
//# sourceMappingURL=ObservableApi.js.map