Object.defineProperty(exports, "__esModule", { value: true });
var clone = require("clone");
var rxjs_1 = require("rxjs");
var Logging_1 = require("../../Utils/Logging");
var WebRx_1 = require("../../WebRx");
var ObservableSampleDataApi = /** @class */ (function () {
    function ObservableSampleDataApi(arg1) {
        var stores = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            stores[_i - 1] = arguments[_i];
        }
        var _this = this;
        this.logger = Logging_1.getLogger(ObservableSampleDataApi.displayName);
        this.stores = {};
        this.actions = {};
        if (Number.isNumber(arg1)) {
            this.delay = arg1;
        }
        else {
            this.delay = 0;
            stores.unshift(arg1);
        }
        stores.filterNull().forEach(function (store) {
            _this.addStore(store);
        });
    }
    ObservableSampleDataApi.prototype.addStore = function (store) {
        Object.assign(this.actions, store.actions);
        this.stores[Object.getName(store)] = store;
    };
    ObservableSampleDataApi.prototype.observe = function (action, params, data, cloneResult) {
        var _this = this;
        if (cloneResult === void 0) { cloneResult = true; }
        return rxjs_1.Observable.of(this.actions[action])
            .map(function (sampleDataAction) {
            if (sampleDataAction == null) {
                throw new Error("Sample DataStore Action Not Found: " + action);
            }
            return sampleDataAction;
        })
            .do(function () {
            _this.logger.info("Sample API Request: " + action, params, data);
        })
            .delay(this.delay)
            .flatMap(function (sampleDataAction) {
            return rxjs_1.Observable.defer(function () {
                return WebRx_1.wx.asObservable(sampleDataAction(params, data));
            }).map(function (x) { return (x != null && cloneResult ? clone(x) : x); });
        })
            .do(function (x) {
            _this.logger.info("Sample API  Result: " + action, x);
        }, function (e) {
            _this.logger.error("Sample API  ERROR: " + action, e);
        });
    };
    ObservableSampleDataApi.prototype.getStoreValue = function (name, selector, cloneResult) {
        if (cloneResult === void 0) { cloneResult = false; }
        var store = this.stores[name];
        if (store == null) {
            return undefined;
        }
        var value = selector(store);
        return value == null ? undefined : cloneResult ? clone(value) : value;
    };
    ObservableSampleDataApi.displayName = 'ObservableSampleDataApi';
    return ObservableSampleDataApi;
}());
exports.ObservableSampleDataApi = ObservableSampleDataApi;
//# sourceMappingURL=ObservableSampleDataApi.js.map