Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var Events_1 = require("../Events");
require("../Extensions/String");
var Logging_1 = require("./Logging");
var PubSub_1 = require("./PubSub");
var Alert = /** @class */ (function () {
    function Alert(pubSub) {
        this.pubSub = pubSub;
        this.logger = Logging_1.getLogger(Alert.displayName);
    }
    Alert.prototype.create = function (content, header, style, timeout) {
        if (String.isNullOrEmpty(content) === false ||
            String.isNullOrEmpty(header) === false) {
            this.pubSub.publish(Events_1.AlertCreatedKey, {
                content: content,
                header: header,
                style: style,
                timeout: timeout,
            });
        }
    };
    Alert.prototype.createForError = function (error, header, style, timeout, formatter, logErrorObject) {
        if (style === void 0) { style = 'danger'; }
        if (logErrorObject === void 0) { logErrorObject = false; }
        if (error != null) {
            var content = void 0;
            var text = void 0;
            var anyError = error;
            var childError = anyError.error;
            if (formatter != null) {
                content = formatter(error);
                text = String.stringify(content);
            }
            else {
                var code = anyError.status || anyError.Status || anyError.code || anyError.Code;
                if (code == null && childError != null) {
                    code =
                        childError.status ||
                            childError.Status ||
                            childError.code ||
                            childError.Code;
                }
                var reason = anyError.reason || anyError.Reason;
                if (code == null && childError != null) {
                    code = childError.reason || childError.Reason;
                }
                var message = anyError.message || anyError.Message;
                if (message == null && childError != null) {
                    message = childError.message || childError.Message;
                }
                if (message == null && String.isString(anyError)) {
                    message = anyError;
                }
                var messageDetail = anyError.messageDetail || anyError.MessageDetail;
                if (messageDetail == null && childError != null) {
                    messageDetail = childError.messageDetail || childError.MessageDetail;
                }
                var codeText = String.isNullOrEmpty(code) ? '' : "Error " + code + ": ";
                var reasonText = String.isNullOrEmpty(reason) ? '' : "(" + reason + "): ";
                text = "" + codeText + reasonText + (message || String.stringify(error));
                header = header || reason || 'Unknown Error';
                var uri = anyError.uri || anyError.Uri || anyError.url || anyError.Url;
                if (uri == null && childError != null) {
                    uri =
                        childError.uri ||
                            childError.Uri ||
                            childError.url ||
                            childError.Url;
                }
                content = (React.createElement("div", { className: "Alert-content" },
                    React.createElement("div", { className: "Alert-title" },
                        String.isNullOrEmpty(code) ? null : (React.createElement("span", { className: "Alert-code" }, "Error " + code + ": ")),
                        String.isNullOrEmpty(reason) ? null : (React.createElement("span", { className: "Alert-code" }, "" + reason))),
                    React.createElement("div", { className: "Alert-text" },
                        String.isNullOrEmpty(message) ? null : (React.createElement("div", { className: "Alert-message" }, message)),
                        String.isNullOrEmpty(messageDetail) ? null : (React.createElement("div", { className: "Alert-messageDetail" }, messageDetail))),
                    String.isNullOrEmpty(uri) ? null : (React.createElement("div", { className: "Alert-uri" },
                        React.createElement("a", { href: uri }, uri)))));
            }
            // allow build to override default value
            if (DEBUG) {
                logErrorObject = true;
            }
            if (logErrorObject === true) {
                this.logger.error(header + ": " + text, error);
            }
            else {
                this.logger.error(header + ": " + text);
            }
            this.create(content, header, style, timeout);
        }
    };
    Alert.displayName = 'Alert';
    return Alert;
}());
exports.Alert = Alert;
exports.Default = new Alert(PubSub_1.Default);
function create(content, header, style, timeout) {
    exports.Default.create(content, header, style, timeout);
}
exports.create = create;
function createForError(error, header, style, timeout, formatter) {
    exports.Default.createForError(error, header, style, timeout, formatter);
}
exports.createForError = createForError;
//# sourceMappingURL=Alert.js.map