import { Iterable } from 'ix';
export interface Comparable<T> {
    compareTo(other: T): number;
}
export declare function isComparable<T>(obj: any): obj is Comparable<T>;
export declare type ValueComparison<T> = (a: T, B: T) => number;
export interface Comparer<T> {
    compare: ValueComparison<T>;
}
export declare class ValueComparer<T = any> implements Comparer<T> {
    readonly comparison: ValueComparison<T>;
    static displayName: string;
    static readonly Default: ValueComparer<any>;
    static DefaultComparison(a: any, b: any): number;
    constructor(comparison?: ValueComparison<T>);
    compare(a: T, b: T): number;
}
export declare function compare(a: any, b: any): boolean;
export declare enum SortDirection {
    Ascending = 1,
    Descending = 2,
}
export declare type FieldValueSelector<TObj, TValue> = (source: TObj, field: string) => TValue;
export interface FieldSelector<TObj, TValue> {
    field: string;
    valueSelector?: FieldValueSelector<TObj, TValue>;
}
export interface FieldComparer<TObj, TValue> extends FieldSelector<TObj, TValue>, Comparer<TValue> {
}
export declare class ObjectComparer<T extends StringMap<any>> {
    static displayName: string;
    static DefaultComparerKey: string;
    static createFieldComparer<TObj, TValue>(field: string, comparison: ValueComparison<TValue>, valueSelector?: (source: TObj, field: string) => TValue): FieldComparer<TObj, TValue>;
    readonly comparerMap: StringMap<FieldComparer<T, any>>;
    readonly defaultComparer: FieldComparer<T, any> | undefined;
    constructor(defaultSortField: string | undefined, ...comparers: Array<FieldComparer<T, any>>);
    constructor(...comparers: Array<FieldComparer<T, any>>);
    getComparer(field?: string): FieldComparer<T, any>;
    getCompare(comparer: Comparer<any>): ValueComparison<any>;
    getValue(source: T, comparer: FieldComparer<T, any>): any;
    sortIterable(source: Iterable<T>, field: string, direction?: SortDirection): Iterable<T>;
}
