// tslint:disable:max-classes-per-file
Object.defineProperty(exports, "__esModule", { value: true });
var moment = require("moment");
/**
 * Conversion from .NET Ticks to millseconds
 * NOTE: this means all moment values have millisecond precision
 */
exports.TicksPerMillisecond = 10000;
// some helper constants to interact with milliseconds
exports.MillisecondsPerDay = 86400000;
exports.MillisecondsPerYear = 31536000000;
/**
 * Offset from .NET milliseconds to unix milliseconds
 */
exports.EpochOffset = 62135596800000;
/**
 * Parse format for the default string serialization of a .NET DateTime
 */
exports.DefaultDateTimeFormat = 'YYYY-MM-DD hh:mm:ss A';
/**
 * Parse format for the default string serialization of a .NET DateTimeOffset
 */
exports.DefaultDateTimeOffsetFormat = exports.DefaultDateTimeFormat + ' Z';
var DefaultDateTimeFormats = [
    exports.DefaultDateTimeFormat,
    exports.DefaultDateTimeOffsetFormat,
];
/**
 * Converts between .NET DateTime.Ticks and moment values
 */
var DateTime = /** @class */ (function () {
    function DateTime() {
    }
    DateTime.fromString = function (value) {
        var formats = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            formats[_i - 1] = arguments[_i];
        }
        return String.isNullOrEmpty(value)
            ? undefined
            : moment.utc(value, formats.length === 0 ? DefaultDateTimeFormats : formats);
    };
    DateTime.fromNumber = function (value, offset) {
        if (value == null) {
            return undefined;
        }
        var result = moment(value / exports.TicksPerMillisecond - exports.EpochOffset);
        return offset == null ? result.local() : result.utcOffset(offset);
    };
    DateTime.toNumber = function (value) {
        return value != null && value.isValid()
            ? (value.valueOf() + exports.EpochOffset) * exports.TicksPerMillisecond
            : undefined;
    };
    /**
     * Standardized string representation of a moment
     */
    DateTime.format = function (value, format, defaultValue) {
        if (format === void 0) { format = DateTime.DefaultLongFormat; }
        if (defaultValue === void 0) { defaultValue = null; }
        return value == null ? defaultValue : value.format(format);
    };
    DateTime.formatLong = function (value, defaultValue) {
        if (defaultValue === void 0) { defaultValue = null; }
        return DateTime.format(value, DateTime.DefaultLongFormat, defaultValue);
    };
    DateTime.formatShort = function (value, defaultValue) {
        if (defaultValue === void 0) { defaultValue = null; }
        return DateTime.format(value, DateTime.DefaultShortFormat, defaultValue);
    };
    // some default formatting values
    DateTime.DefaultLongFormat = 'dddd, LL';
    DateTime.DefaultShortFormat = 'DD-MMM-YYYY';
    return DateTime;
}());
exports.DateTime = DateTime;
/**
 * Converts between .NET TimeSpan.Ticks and moment values
 */
var TimeSpan = /** @class */ (function () {
    function TimeSpan() {
    }
    TimeSpan.fromString = function (value) {
        return String.isNullOrEmpty(value) ? undefined : moment.duration(value);
    };
    TimeSpan.fromNumber = function (value) {
        return value == null
            ? undefined
            : moment.duration(value / exports.TicksPerMillisecond);
    };
    TimeSpan.toNumber = function (value) {
        return value == null
            ? undefined
            : value.asMilliseconds() * exports.TicksPerMillisecond;
    };
    /**
     * Standardized hours string representation of a duration
     */
    TimeSpan.formatHours = function (value, precision, defaultValue) {
        if (precision === void 0) { precision = TimeSpan.DefaultDurationHoursPrecision; }
        if (defaultValue === void 0) { defaultValue = null; }
        return value == null
            ? defaultValue
            : value.asHours().toFixed(precision) + " Hours";
    };
    /**
     * Standardized days string representation of a duration
     */
    TimeSpan.formatDays = function (value, defaultValue) {
        if (defaultValue === void 0) { defaultValue = null; }
        return value == null ? defaultValue : value.humanize();
    };
    /**
     * Standardized duration string representation that will format hours or days based on maxHours
     */
    TimeSpan.format = function (value, maxHours, precision, defaultValue) {
        if (maxHours === void 0) { maxHours = TimeSpan.DefaultFormatMaxHours; }
        if (precision === void 0) { precision = TimeSpan.DefaultDurationHoursPrecision; }
        if (defaultValue === void 0) { defaultValue = null; }
        if (value == null) {
            return defaultValue;
        }
        return value.asHours() < maxHours
            ? TimeSpan.formatHours(value, precision)
            : TimeSpan.formatDays(value);
    };
    // some default formatting values
    TimeSpan.DefaultDurationHoursPrecision = 1;
    TimeSpan.DefaultDurationDaysPrecision = 1;
    TimeSpan.DefaultFormatMaxHours = 24;
    return TimeSpan;
}());
exports.TimeSpan = TimeSpan;
//# sourceMappingURL=Moment.js.map