Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var rxjs_1 = require("rxjs");
var Utils_1 = require("./Utils");
var ObservableCommand = /** @class */ (function (_super) {
    tslib_1.__extends(ObservableCommand, _super);
    function ObservableCommand(executeAction, interrogationAction, condition, initialCondition) {
        if (interrogationAction === void 0) { interrogationAction = function (x) { return ObservableCommand.coerceCondition(x); }; }
        var _this = _super.call(this) || this;
        _this.executeAction = executeAction;
        _this.interrogationAction = interrogationAction;
        _this.condition = condition;
        _this.initialCondition = initialCondition;
        _this.isExecutingSubject = _this.addSubscription(new rxjs_1.BehaviorSubject(false));
        _this.conditionSubject = _this.addSubscription(new rxjs_1.BehaviorSubject(initialCondition));
        _this.requestsSubject = _this.addSubscription(new rxjs_1.Subject());
        _this.resultsSubject = _this.addSubscription(new rxjs_1.Subject());
        _this.thrownErrorsSubject = _this.addSubscription(new rxjs_1.Subject());
        if (condition != null) {
            _this.add(condition.subscribe(_this.conditionSubject));
        }
        _this.canExecuteSubject = _this.addSubscription(new rxjs_1.BehaviorSubject(condition == null ||
            _this.conditionValue == null ||
            _this.interrogationAction(_this.conditionValue)));
        var canExecute = condition == null
            ? Utils_1.asObservable(true)
            : _this.conditionSubject.map(function (x) { return _this.interrogationAction(x); });
        _this.add(canExecute
            .combineLatest(_this.isExecutingSubject, function (ce, ie) { return ce === true && ie === false; })
            .catch(function (e) {
            Utils_1.handleError(e, _this.thrownErrorsSubject);
            return Utils_1.asObservable(false);
        })
            .distinctUntilChanged()
            .subscribe(_this.canExecuteSubject));
        return _this;
    }
    ObservableCommand.coerceCondition = function (condition) {
        if (typeof condition === 'boolean') {
            return condition;
        }
        return condition != null;
    };
    Object.defineProperty(ObservableCommand.prototype, "isExecutingObservable", {
        get: function () {
            return this.isExecutingSubject.distinctUntilChanged();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ObservableCommand.prototype, "isExecuting", {
        get: function () {
            return this.isExecutingSubject.getValue();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ObservableCommand.prototype, "conditionObservable", {
        get: function () {
            return this.conditionSubject.distinctUntilChanged();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ObservableCommand.prototype, "canExecuteObservable", {
        get: function () {
            return this.canExecuteSubject.distinctUntilChanged();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ObservableCommand.prototype, "conditionValue", {
        get: function () {
            return this.conditionSubject.getValue();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ObservableCommand.prototype, "canExecute", {
        get: function () {
            return this.canExecuteSubject.getValue();
        },
        enumerable: true,
        configurable: true
    });
    ObservableCommand.prototype.isCommand = function () {
        return true;
    };
    ObservableCommand.prototype.canExecuteFor = function (parameter) {
        return this.interrogationAction(this.conditionSubject.getValue(), parameter);
    };
    ObservableCommand.prototype.observeExecution = function (parameter) {
        var _this = this;
        if (this.canExecute === false) {
            var error = new Error('canExecute currently forbids execution');
            Utils_1.handleError(error);
            return rxjs_1.Observable.throw(error);
        }
        return (rxjs_1.Observable.of(parameter)
            .do(function (x) {
            _this.requestsSubject.next(x);
            _this.isExecutingSubject.next(true);
        })
            .flatMap(function (x) {
            return Utils_1.asObservable(_this.executeAction(x));
        })
            .do(function (x) {
            _this.resultsSubject.next(x);
            _this.isExecutingSubject.next(false);
        }, function (e) {
            // capture the error, but don't swallow it
            Utils_1.handleError(e, _this.thrownErrorsSubject);
            _this.isExecutingSubject.next(false);
        }, function () {
            _this.isExecutingSubject.next(false);
        })
            // this will prevent execution if nobody is subscribing to the result
            .share());
    };
    ObservableCommand.prototype.execute = function (parameter, observerOrNext, error, complete) {
        if (error === void 0) { error = function () {
            return;
        }; }
        var obs = this.observeExecution(parameter);
        return obs.subscribe.apply(obs, [observerOrNext, error, complete]);
    };
    Object.defineProperty(ObservableCommand.prototype, "requests", {
        get: function () {
            return this.requestsSubject.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ObservableCommand.prototype, "results", {
        get: function () {
            return this.resultsSubject.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ObservableCommand.prototype, "thrownErrors", {
        get: function () {
            return this.thrownErrorsSubject.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    return ObservableCommand;
}(rxjs_1.Subscription));
exports.ObservableCommand = ObservableCommand;
function command(arg1, arg2, interrogation, initialCondition) {
    var condition;
    var execute = function (x) { return x; };
    if (Utils_1.isObservable(arg1)) {
        condition = arg1;
        if (arg2 instanceof Function) {
            execute = arg2;
        }
    }
    else if (Utils_1.isObservable(arg2)) {
        condition = arg2;
        if (arg1 instanceof Function) {
            execute = arg1;
        }
    }
    else {
        // no boolean observable passed in for can execute
        // just check if arg1 is a function
        if (arg1 instanceof Function) {
            execute = arg1;
        }
    }
    if (condition == null) {
        initialCondition = true;
    }
    return new ObservableCommand(execute, interrogation, condition, initialCondition);
}
exports.command = command;
//# sourceMappingURL=Command.js.map