// we can disable shadowed variables here since we are performing type augmentations
// tslint:disable no-shadowed-variable
Object.defineProperty(exports, "__esModule", { value: true });
var ix_1 = require("ix");
var Observable_1 = require("rxjs/Observable");
var Subscription_1 = require("rxjs/Subscription");
var Property_1 = require("./Property");
var Utils_1 = require("./Utils");
function filterNullIterable(callbackfn) {
    return this.filter(function (x, i) {
        if (x == null) {
            return false;
        }
        return callbackfn == null ? true : callbackfn(x, i);
    });
}
exports.filterNullIterable = filterNullIterable;
function addSubscription(subscription) {
    this.add(subscription);
    return subscription;
}
exports.addSubscription = addSubscription;
function addSubscriptions() {
    var _this = this;
    var subscriptions = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        subscriptions[_i] = arguments[_i];
    }
    return subscriptions.map(function (x) { return _this.addSubscription(x); });
}
exports.addSubscriptions = addSubscriptions;
function unsubscribeStatic(subscription, unsubscribedValue) {
    if (Utils_1.isSubscription(subscription)) {
        subscription.unsubscribe();
        return unsubscribedValue || Subscription_1.Subscription.EMPTY;
    }
    return subscription;
}
exports.unsubscribeStatic = unsubscribeStatic;
function filterNullObservable(callbackfn) {
    return this.filter(function (x, i) {
        if (x == null) {
            return false;
        }
        return callbackfn == null ? true : callbackfn(x, i);
    });
}
exports.filterNullObservable = filterNullObservable;
function toProperty(initialValue, compare, keySelector) {
    return Property_1.property(initialValue, compare, keySelector, this);
}
exports.toProperty = toProperty;
function observeCommand(command) {
    // see the ReactiveUI project for the inspiration behind this function:
    // https://github.com/reactiveui/ReactiveUI/blob/master/src/ReactiveUI/ReactiveCommand.cs#L1078
    return this.map(function (x) { return ({
        parameter: x,
        command: command instanceof Function ? command(x) : command,
    }); })
        .debounce(function (x) {
        return x.command.canExecuteObservable
            .startWith(x.command.canExecute)
            .filter(function (y) { return y; })
            .map(function () { return 0; });
    })
        .map(function (x) {
        return x.command.observeExecution(x.parameter);
    })
        .switch();
}
exports.observeCommand = observeCommand;
function invokeCommand(command, observerOrNext, error, complete) {
    var obs = this.observeCommand(command);
    return obs.subscribe.apply(obs, [observerOrNext, error, complete]);
}
exports.invokeCommand = invokeCommand;
ix_1.Iterable.prototype.filterNull = filterNullIterable;
Subscription_1.Subscription.prototype.addSubscription = addSubscription;
Subscription_1.Subscription.prototype.addSubscriptions = addSubscriptions;
Subscription_1.Subscription.unsubscribe = unsubscribeStatic;
Observable_1.Observable.prototype.filterNull = filterNullObservable;
Observable_1.Observable.prototype.toProperty = toProperty;
Observable_1.Observable.prototype.observeCommand = observeCommand;
Observable_1.Observable.prototype.invokeCommand = invokeCommand;
//# sourceMappingURL=Extensions.js.map