Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var rxjs_1 = require("rxjs");
var Utils_1 = require("./Utils");
var ObservableProperty = /** @class */ (function (_super) {
    tslib_1.__extends(ObservableProperty, _super);
    function ObservableProperty(initialValue, compare, keySelector, source) {
        if (source === void 0) { source = new rxjs_1.Subject(); }
        var _this = _super.call(this) || this;
        _this.source = source;
        _this.changedSubject = _this.addSubscription(new rxjs_1.BehaviorSubject(initialValue));
        _this.thrownErrorsSubject = _this.addSubscription(new rxjs_1.Subject());
        _this.add(_this.source
            // seed the observable subscription with the initial value so that
            // distinctUntilChanged knows what the initial value is
            .startWith(initialValue)
            .distinctUntilChanged(compare, keySelector)
            .subscribe(function (x) {
            _this.changedSubject.next(x);
        }, function (e) {
            Utils_1.handleError(e, _this.thrownErrorsSubject);
        }));
        return _this;
    }
    Object.defineProperty(ObservableProperty.prototype, "isReadOnly", {
        get: function () {
            return Utils_1.isSubject(this.source) === false;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ObservableProperty.prototype, "value", {
        get: function () {
            return this.changedSubject.getValue();
        },
        set: function (newValue) {
            if (Utils_1.isSubject(this.source)) {
                this.source.next(newValue);
            }
            else {
                throw new Error('attempt to write to a read-only observable property');
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ObservableProperty.prototype, "changed", {
        get: function () {
            return (this.changedSubject
                // BehaviorSubject fires immediately, so skip the first event
                .skip(1));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ObservableProperty.prototype, "thrownErrors", {
        get: function () {
            return this.thrownErrorsSubject.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    ObservableProperty.prototype.isProperty = function () {
        return true;
    };
    return ObservableProperty;
}(rxjs_1.Subscription));
exports.ObservableProperty = ObservableProperty;
function property() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    var initialValue = args.shift();
    var compare;
    var keySelector;
    var source;
    var arg = args.shift();
    if (Utils_1.isObservable(arg)) {
        source = arg;
    }
    else {
        // if arg is true, then we use the default comparator function
        if (arg === false) {
            // this results in every value being interpreted as a new value
            compare = function () { return false; };
        }
        else if (arg instanceof Function) {
            compare = arg;
        }
    }
    arg = args.shift();
    if (Utils_1.isObservable(arg)) {
        source = arg;
    }
    else {
        keySelector = arg;
    }
    arg = args.shift();
    if (Utils_1.isObservable(arg)) {
        source = arg;
    }
    return new ObservableProperty(initialValue, compare, keySelector, source);
}
exports.property = property;
//# sourceMappingURL=Property.js.map