Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var ix_1 = require("ix");
var isiterable_1 = require("ix/internal/isiterable");
exports.isAsyncIterable = isiterable_1.isAsyncIterable;
exports.isIterable = isiterable_1.isIterable;
var rxjs_1 = require("rxjs");
var Utils_1 = require("../Utils");
var Console_1 = require("../Utils/Logging/Adapters/Console");
function isSubscription(value) {
    if (value == null) {
        return false;
    }
    var sub = value;
    return sub.unsubscribe instanceof Function;
}
exports.isSubscription = isSubscription;
function isObservable(value) {
    return value instanceof rxjs_1.Observable;
}
exports.isObservable = isObservable;
function isObserver(value) {
    if (value == null) {
        return false;
    }
    var obs = value;
    return (obs.next instanceof Function &&
        obs.error instanceof Function &&
        obs.complete instanceof Function);
}
exports.isObserver = isObserver;
function isSubject(value) {
    return value instanceof rxjs_1.Subject;
}
exports.isSubject = isSubject;
function isProperty(value) {
    if (value == null) {
        return false;
    }
    var prop = value;
    return prop.isProperty instanceof Function && prop.isProperty();
}
exports.isProperty = isProperty;
function isCommand(value) {
    if (value == null) {
        return false;
    }
    var cmd = value;
    return cmd.isCommand instanceof Function && cmd.isCommand();
}
exports.isCommand = isCommand;
function asObservable(value) {
    return isObservable(value) ? value : rxjs_1.Observable.of(value);
}
exports.asObservable = asObservable;
function getObservable(observableLike) {
    if (isProperty(observableLike)) {
        return observableLike.changed.startWith(observableLike.value);
    }
    if (isCommand(observableLike)) {
        return observableLike.results;
    }
    if (isObservable(observableLike)) {
        return observableLike;
    }
    if (observableLike != null) {
        return rxjs_1.Observable.of(observableLike);
    }
    return rxjs_1.Observable.never();
}
exports.getObservable = getObservable;
function getProperty(observableLike, initialValue, compare, keySelector) {
    if (isProperty(observableLike)) {
        return observableLike;
    }
    if (isCommand(observableLike)) {
        return observableLike.results.toProperty(initialValue);
    }
    if (isObservable(observableLike)) {
        return observableLike.toProperty(initialValue);
    }
    if (initialValue == null && observableLike != null) {
        initialValue = observableLike;
    }
    return new rxjs_1.Subject().toProperty(initialValue, compare, keySelector);
}
exports.getProperty = getProperty;
function handleError(e) {
    var optionalParams = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        optionalParams[_i - 1] = arguments[_i];
    }
    var err;
    if (e instanceof Error) {
        err = e;
    }
    else if (String.isString(e)) {
        err = new Error(e);
    }
    else if (String.isString(e.message) || String.isString(e.Message)) {
        err = new Error(e.message || e.Message);
    }
    else {
        err = new Error('Undefined Error');
    }
    // trim off the subject if it was provided with the optional params
    var subject = isSubject(optionalParams[0])
        ? optionalParams.shift()
        : undefined;
    if (DEBUG || subject == null) {
        // in debug mode we want to emit any webrx errors
        // if there is no subject receiving the error then we should be emitting to the console
        logError.apply(void 0, tslib_1.__spread([err], optionalParams));
    }
    if (isSubject(subject)) {
        subject.next(err);
    }
}
exports.handleError = handleError;
// replace this function to inject your own global error handling
function logError(err) {
    var optionalParams = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        optionalParams[_i - 1] = arguments[_i];
    }
    Console_1.Default.logToConsole.apply(Console_1.Default, tslib_1.__spread([Utils_1.Logging.LogLevel.Error, err], optionalParams));
}
exports.logError = logError;
function logObservable(logger, observable, name) {
    return observable.subscribe(function (x) {
        if (x == null) {
            logger.debug(name);
        }
        else if (Object.isObject(x)) {
            var value = Object.getName(x);
            if (value === 'Object') {
                value = '';
            }
            logger.debug(name + " = " + value, x);
        }
        else {
            logger.debug(name + " = " + x);
        }
    }, function (e) { return logger.error(name + ": " + e); });
}
exports.logObservable = logObservable;
function logMemberObservables(logger, source) {
    return ix_1.Iterable.from(Object.keys(source))
        .map(function (key) { return ({ key: key, member: source[key] }); })
        .filter(function (x) {
        return isObservable(x.member) || isProperty(x.member) || isCommand(x.member);
    })
        .flatMap(function (x) {
        if (isCommand(x.member)) {
            return [
                { key: "<" + x.key + ">...", observable: x.member.requests },
                { key: "<" + x.key + ">", observable: x.member.results },
            ];
        }
        return [{ key: x.key, observable: getObservable(x.member) }];
    })
        .map(function (x) { return logObservable(logger, x.observable, x.key); })
        .toArray();
}
exports.logMemberObservables = logMemberObservables;
function getObservableOrAlert(observableFactory, header, style, timeout, errorFormatter, errorResult) {
    if (errorResult === void 0) { errorResult = rxjs_1.Observable.empty(); }
    return rxjs_1.Observable.defer(observableFactory).catch(function (err) {
        Utils_1.Alert.createForError(err, header, style, timeout, errorFormatter);
        return errorResult;
    });
}
exports.getObservableOrAlert = getObservableOrAlert;
function getObservableResultOrAlert(resultFactory, header, style, timeout, errorFormatter, errorResult) {
    return getObservableOrAlert(function () { return getObservable(resultFactory()); }, header, style, timeout, errorFormatter, errorResult);
}
exports.getObservableResultOrAlert = getObservableResultOrAlert;
function subscribeOrAlert(observableFactory, header, onNext, style, timeout, errorFormatter, errorResult) {
    return getObservableOrAlert(observableFactory, header, style, timeout, errorFormatter, errorResult).subscribe(function (x) {
        try {
            onNext(x);
        }
        catch (err) {
            Utils_1.Alert.createForError(err, header, style, timeout, errorFormatter);
        }
    });
}
exports.subscribeOrAlert = subscribeOrAlert;
//# sourceMappingURL=Utils.js.map